@@ -42,8 +42,6 @@ void PwAddChild(Widget, Widget);
 static void pwSetPixmap(PaintWidget w, Pixmap pix, int flag);
 static void fatCallback(Widget, XtPointer, XtPointer);
 static void realExposeProc(PaintWidget, XExposeEvent *, Boolean);
-static void drawVisibleGrid(PaintWidget, Widget, Boolean,
-			    int, int, int, int, int);
 
 static XtResource resources[] =
 {
@@ -71,8 +69,22 @@ static XtResource resources[] =
      offset(foreground), XtRString, XtDefaultForeground},
     {XtNlineForeground, XtCLineForeground, XtRPixel, sizeof(Pixel),
      offset(lineForeground), XtRString, XtDefaultForeground},
-    {XtNsnap, XtCSnap, XtRInt, sizeof(int),
-     offset(snap), XtRImmediate, (XtPointer) 10},
+    {XtNmenubar, XtCMenubar, XtRBoolean, sizeof(Boolean),
+     offset(menubar), XtRImmediate, (XtPointer) True},
+    {XtNfullmenu, XtCFullMenu, XtRBoolean, sizeof(Boolean),
+     offset(fullmenu), XtRImmediate, (XtPointer) False},
+    {XtNgridmode, XtCGridMode, XtRInt, sizeof(int),
+     offset(gridmode), XtRImmediate, (XtPointer) (1<<31)},
+    {XtNinterpolation, XtCInterpolation, XtRInt, sizeof(int),
+     offset(interpolation), XtRImmediate, (XtPointer) 0},
+    {XtNtransparent, XtCTransparent, XtRInt, sizeof(int),
+     offset(transparent), XtRImmediate, (XtPointer) 1},
+    {XtNgridcolor, XtCGridColor, XtRPixel, sizeof(Pixel),
+     offset(gridcolor), XtRString, (XtPointer) "Green80"},
+    {XtNsnapX, XtCSnapX, XtRInt, sizeof(int),
+     offset(snap_x), XtRImmediate, (XtPointer) 10},
+    {XtNsnapY, XtCSnapY, XtRInt, sizeof(int),
+     offset(snap_y), XtRImmediate, (XtPointer) 10},
     {XtNregionCallback, XtCRegionCallback, XtRCallback, sizeof(XtPointer),
      offset(regionCalls), XtRCallback, (XtPointer) None},
     {XtNsizeChanged, XtCSizeChanged, XtRCallback, sizeof(XtPointer),
@@ -95,6 +107,10 @@ static XtResource resources[] =
      offset(paint), XtRImmediate, (XtPointer) None},
     {XtNdirty, XtCDirty, XtRBoolean, sizeof(Boolean),
      offset(dirty), XtRImmediate, (XtPointer) False},
+    {XtNeditable, XtCEditable, XtRBoolean, sizeof(Boolean),
+     offset(editable), XtRImmediate, (XtPointer) True},
+    {XtNlocked, XtCLocked, XtRBoolean, sizeof(Boolean),
+     offset(locked), XtRImmediate, (XtPointer) False},
     {XtNfillRule, XtCFillRule, XtRInt, sizeof(int),
      offset(fillRule), XtRImmediate, (XtPointer) FillSolid},
     {XtNlineFillRule, XtCLineFillRule, XtRInt, sizeof(int),
@@ -210,7 +226,8 @@ undoBuffer(PaintWidget w)
     new->box.y = 0;
     new->box.width = 0;
     new->box.height = 0;
-    new->pixmap = None;
+    new->alphapix.pixmap = None;
+    new->alphapix.alpha = NULL;
     new->next = w->paint.head;
     new->pushed = False;
     w->paint.head = new;
@@ -231,7 +248,7 @@ InitializeProc(Widget w, Widget newArg)
 
     if (zoom == PwZoomParent)
 	zoom = 1;
-    else if (zoom < 0)
+    else if (zoom == -1)
 	zoom = new->paint.zoom = 1;
 
     if (new->paint.sourcePixmap != None) {
@@ -250,10 +267,18 @@ InitializeProc(Widget w, Widget newArg)
 	new->paint.drawWidth = parent->paint.drawWidth;
 	new->paint.drawHeight = parent->paint.drawHeight;
     }
-    if (new->core.width == 0)
-	new->core.width = new->paint.drawWidth * zoom;
-    if (new->core.height == 0)
-	new->core.height = new->paint.drawHeight * zoom;
+    if (new->core.width == 0) {
+        if (zoom>0)
+	    new->core.width = new->paint.drawWidth * zoom;
+        else
+	    new->core.width = (new->paint.drawWidth-zoom-1)/(-zoom);
+    }
+    if (new->core.height == 0) {
+        if (zoom>0)
+	    new->core.height = new->paint.drawHeight * zoom;
+        else
+	    new->core.height = (new->paint.drawHeight-zoom-1)/(-zoom);
+    }
 
     new->paint.foreground = BlackPixelOfScreen(XtScreen(new));
     new->paint.background = WhitePixelOfScreen(XtScreen(new));
@@ -277,10 +302,11 @@ InitializeProc(Widget w, Widget newArg)
 	cur->next = new->paint.head;
 	UndoInitialize(new, new->paint.undoSize);
 	new->paint.undo = new->paint.head;
-	new->paint.current = new->paint.head->pixmap;
+	new->paint.current = new->paint.head->alphapix;
     } else {
 	new->paint.undo = NULL;
-	new->paint.current = None;
+	new->paint.current.pixmap = None;
+	new->paint.current.alpha = NULL;
     }
 
     /*
@@ -296,6 +322,7 @@ InitializeProc(Widget w, Widget newArg)
     new->paint.region.sourceImg = NULL;
     new->paint.region.maskImg = NULL;
     new->paint.region.notMaskImg = NULL;
+    new->paint.region.alpha = NULL;
     new->paint.region.proc = NULL;
 
     for (i = 0; i < XtNumber(new->paint.region.grip); i++)
@@ -448,10 +475,13 @@ RealizeProc(Widget w, Mask * valueMask,
     /*
     **	Create the backing undo buffers
      */
-    for (cur = pw->paint.head; cur && (cur->pixmap == None); cur = cur->next)
-	cur->pixmap = XCreatePixmap(XtDisplay(w), XtWindow(w),
+    for (cur = pw->paint.head; cur && (cur->alphapix.pixmap == None); 
+         cur = cur->next) {
+	cur->alphapix.pixmap = XCreatePixmap(XtDisplay(w), XtWindow(w),
 			       pw->paint.drawWidth, pw->paint.drawHeight,
 				    depth);
+        cur->alphapix.alpha = NULL;
+    }
 
     /*
     **	If this paint widget is the child of another paint
@@ -459,12 +489,12 @@ RealizeProc(Widget w, Mask * valueMask,
     **	 or just use the information from our parent.
      */
     if (pw->paint.paint == None) {
-	pw->paint.current = pw->paint.head->pixmap;
+	pw->paint.current = pw->paint.head->alphapix;
 
 	if (pw->paint.sourcePixmap != None)
 	    pwSetPixmap(pw, pw->paint.sourcePixmap, FALSE);
 	else
-	    XFillRectangle(XtDisplay(w), pw->paint.current,
+	    XFillRectangle(XtDisplay(w), pw->paint.current.pixmap,
 			   pw->paint.igc, 0, 0,
 			   pw->paint.drawWidth, pw->paint.drawHeight);
     } else
@@ -525,8 +555,14 @@ DestroyProc(Widget w)
     cur = pw->paint.head;
     do {
 	nxt = cur->next;
-	if (cur->pixmap != None)
-	    XFreePixmap(XtDisplay(pw), cur->pixmap);
+	if (cur->alphapix.pixmap != None) {
+	    XFreePixmap(XtDisplay(pw), cur->alphapix.pixmap);
+            cur->alphapix.pixmap = None;
+	}
+	if (cur->alphapix.alpha != NULL) {
+	    XtFree((char *)cur->alphapix.alpha);
+            cur->alphapix.alpha = NULL;
+	}
 	XtFree((XtPointer) cur);
 	cur = nxt;
     }
@@ -545,45 +581,6 @@ ResizeProc(Widget w)
 {
 }
 
-static void
-drawVisibleGrid(PaintWidget pw, Widget w, Boolean flag,
-		int zoom, int sx, int sy, int ex, int ey)
-{
-    static GC tgc = None;
-    GC gc;
-    Display *dpy = XtDisplay(w);
-    Window win = XtWindow(w);
-    int i;
-
-    if (zoom <= 1)
-	return;
-
-    if (flag) {
-	if (tgc == None) {
-	    XGCValues values;
-
-	    values.foreground = BlackPixelOfScreen(XtScreen(w));
-	    tgc = XtGetGC(w, GCForeground, &values);
-	}
-	gc = tgc;
-    } else {
-	/*
-	**  Turning off visible grid, and we need to clear
-	**    to no space between pixels...
-	 */
-	if (zoom <= ZOOM_THRESH) {
-	    XClearArea(dpy, win, sx, sy, ex - sx, ey - sy, True);
-	    return;
-	}
-	gc = pw->paint.igc;
-    }
-
-    for (i = sx; i < ex; i += zoom)
-	XDrawLine(dpy, win, gc, i, sy, i, ey);
-    for (i = sy; i < ey; i += zoom)
-	XDrawLine(dpy, win, gc, sx, i, ex, i);
-}
-
 /*
 **  Update the zoomed region as specified by rect.
 **    rect is in screen coordinates.
@@ -593,7 +590,7 @@ drawVisibleGrid(PaintWidget pw, Widget w, Boolean flag,
 **
  */
 
-void
+static void
 zoomDrawRects(PaintWidget pw, Widget w, GC gc, XImage * xim, XImage * mask,
 	      Boolean isExpose, int xstart, int ystart, int zoom,
 	      XRectangle * rect)
@@ -682,32 +679,150 @@ zoomDrawImage(PaintWidget pw, Widget w, GC gc, XImage * xim, XImage * mask,
 	      XRectangle * rect)
 {
     Display *dpy = XtDisplay(w);
+    Pixmap pix;
     int width = rect->width;
     int height = rect->height;
+    int widthp, heightp;
     XImage *dst;
-    XImage *dstMsk = NULL;
-    int x, y, x1, y1, dx, dy;
-    int numDraw = zoom - ((zoom > ZOOM_THRESH) ? 1 : 0);
-    int endX = rect->x + rect->width;
-    int endY = rect->y + rect->height;
+    XImage *dstMask = NULL;
+    int bytes_per_pixel, zoom_bytes_per_pixel;
+    int i, j, x, y, x1, y1, dx, dy;
+    int numDraw;
+    int endX;
+    int endY;
+    int interpolation;
+
+    if (zoom<0) {
+        zoom = -zoom;
+        XtVaGetValues((Widget)pw, XtNinterpolation, &interpolation, NULL);
+        widthp = (width+zoom-1)/zoom;
+        heightp = (height+zoom-1)/zoom;
+        dst = XCreateImage(dpy, pw->paint.visual, xim->depth, xim->format,
+		           0, NULL, widthp, heightp, 32, 0);
+        dst->data = (char *) XtMalloc(heightp * dst->bytes_per_line);
+
+        if (mask != NULL) {
+            dstMask = XCreateImage(dpy, pw->paint.visual,
+			           mask->depth, mask->format,
+			           0, NULL, widthp, heightp, 8, 0);
+	dstMask->data = (char *) 
+                     XtMalloc(heightp * dstMask->bytes_per_line);
+	memset(dstMask->data, 0, heightp * dstMask->bytes_per_line);
+	}
+        if (mask == NULL) {
+	    bytes_per_pixel = xim->bits_per_pixel/8;
+            zoom_bytes_per_pixel = zoom*bytes_per_pixel;
+	    if (xim->bits_per_pixel != 8*bytes_per_pixel)
+            /* slow procedure */
+	    for (y = 0; y < heightp; y++) {
+	        dy = y*zoom ;
+	        for (x = 0; x < widthp; x++) {
+		    Pixel p;
+	            dx = x*zoom ;
+		    p = XGetPixel(xim, rect->x+dx, rect->y+dy);
+		    XPutPixel(dst, x, y, p);
+		}
+	    } else
+            /* faster procedure */
+	    for (y = 0; y < heightp; y++) {
+	        dy = y*zoom;
+                j = (rect->y+dy) * xim->bytes_per_line +
+                    rect->x * bytes_per_pixel;
+                i = y * dst->bytes_per_line;
+                if (interpolation) {
+		    unsigned int u;
+		    int k, l, m, zx, zy;
+                    if (y<heightp-1) zy = zoom; else zy = height-y*zoom;
+	            for (x = 0; x < widthp; x++) {
+                        if (x<widthp-1) zx = zoom; else zx = width-x*zoom;
+		        for (k=0; k<bytes_per_pixel; k++) { 
+		             u = 0;
+		             for (l=0; l<zx; l++)
+		             for (m=0; m<zy; m++)
+			       u += (unsigned char)xim->data[j+k+l*bytes_per_pixel+m*xim->bytes_per_line];
+                             dst->data[i+k]= (u/(zx*zy))&255;
+			}
+		        i += bytes_per_pixel;
+                        j += zoom_bytes_per_pixel;
+		    }
+		} else
+	        for (x = 0; x < widthp; x++) {
+		    memcpy(&dst->data[i], &xim->data[j], bytes_per_pixel);
+		    i += bytes_per_pixel;
+                    j += zoom_bytes_per_pixel;
+		}
+	    }
+            XPutImage(XtDisplay(w), XtWindow(w), gc, dst, 0, 0,
+		      xstart/zoom, ystart/zoom,
+		      widthp, heightp);
+	} else {
+	    bytes_per_pixel = xim->bits_per_pixel/8;
+            zoom_bytes_per_pixel = zoom*bytes_per_pixel;
+	    if (xim->bits_per_pixel != 8*bytes_per_pixel)
+	    /* slow procedure */
+	    for (y = 0; y < heightp; y++)
+	    for (x = 0; x < widthp; x++) {
+		Pixel p;
+	        dx = x*zoom;
+	        dy = y*zoom;
+                if (!XGetPixel(mask, rect->x+dx, rect->y+dy)) continue;
+		p = XGetPixel(xim, rect->x+dx, rect->y+dy);
+		XPutPixel(dst, x, y, p);
+		XPutPixel(dstMask, x, y, True);
+	    } else 
+            /* faster procedure */
+	    for (y = 0; y < heightp; y++) {
+	        dy = y * zoom;
+                j = (rect->y+dy) * xim->bytes_per_line +
+                    rect->x * bytes_per_pixel;
+                i = y * dst->bytes_per_line;
+	        for (x = 0; x < widthp; x++) {
+		    if (XGetPixel(mask, rect->x+x*zoom, rect->y+dy)) {
+		        memcpy(&dst->data[i], &xim->data[j], bytes_per_pixel);
+                        XPutPixel(dstMask, x, y, True);
+		    }
+		    i += bytes_per_pixel;
+                    j += zoom_bytes_per_pixel;
+		}
+	    }
+            /* {static int jj=0; printf("??? %d\n",  ++jj);} */
+	    pix = XCreatePixmap(XtDisplay(w), XtWindow(w),
+				widthp, heightp, 1);
+  	    XPutImage(XtDisplay(w), pix, pw->paint.mgc, dstMask,
+		      0, 0, 0, 0, widthp, heightp);
+	    XSetClipMask(XtDisplay(w), gc, pix);
+	    XSetClipOrigin(XtDisplay(w), gc, xstart/zoom, ystart/zoom);
+	    XPutImage(XtDisplay(w), XtWindow(w), gc, dst, 0, 0,
+		      xstart/zoom, ystart/zoom,
+		      widthp, heightp);
+            XSetClipMask(XtDisplay(w), gc, None);
+	    XFreePixmap(XtDisplay(w), pix);
+	}
+        goto end;
+    }
+
+    numDraw = zoom - ((zoom > ZOOM_THRESH) ? 1 : 0);
+    endX = rect->x + rect->width;
+    endY = rect->y + rect->height;
 
     dst = XCreateImage(dpy, pw->paint.visual, xim->depth, xim->format,
-		       0, NULL, width * zoom, height * zoom, 32, 0);
+  	               0, NULL, width * zoom, height * zoom, 32, 0);
     dst->data = (char *) XtMalloc(height * zoom * dst->bytes_per_line);
+
     if (mask != NULL) {
-	dstMsk = XCreateImage(dpy, pw->paint.visual,
-			      mask->depth, mask->format,
-			      0, NULL, width * zoom, height * zoom, 8, 0);
-	dstMsk->data = (char *) XtMalloc(height * zoom * dstMsk->bytes_per_line);
-	memset(dstMsk->data, 0, height * zoom * dstMsk->bytes_per_line);
+        dstMask = XCreateImage(dpy, pw->paint.visual,
+			       mask->depth, mask->format,
+			       0, NULL, width * zoom, height * zoom, 8, 0);
+	dstMask->data = (char *) 
+                     XtMalloc(height * zoom * dstMask->bytes_per_line);
+	memset(dstMask->data, 0, height * zoom * dstMask->bytes_per_line);
     }
+
     if (mask == NULL) {
 	for (dy = 0, y = rect->y; y < endY; y++, dy += zoom) {
 	    for (dx = 0, x = rect->x; x < endX; x++, dx += zoom) {
 		Pixel p;
-
 		p = xxGetPixel(xim, x, y);
-
 		for (y1 = 0; y1 < numDraw; y1++) {
 		    for (x1 = 0; x1 < numDraw; x1++) {
 			xxPutPixel(dst, dx + x1, dy + y1, p);
@@ -722,12 +837,15 @@ zoomDrawImage(PaintWidget pw, Widget w, GC gc, XImage * xim, XImage * mask,
 				   w->core.background_pixel);
 	    }
 	}
+	XPutImage(XtDisplay(w), XtWindow(w), gc, dst, 0, 0,
+		  xstart * zoom, ystart * zoom,
+		  width * zoom, height * zoom);
     } else {
 	for (dy = 0, y = rect->y; y < endY; y++, dy += zoom) {
 	    for (dx = 0, x = rect->x; x < endX; x++, dx += zoom) {
-		Pixel p;
+	        Pixel p;
 
-		if (!xxGetPixel(mask, x, y))
+		if (!XGetPixel(mask, x, y))
 		    continue;
 
 		p = xxGetPixel(xim, x, y);
@@ -735,37 +853,30 @@ zoomDrawImage(PaintWidget pw, Widget w, GC gc, XImage * xim, XImage * mask,
 		for (y1 = 0; y1 < numDraw; y1++) {
 		    for (x1 = 0; x1 < numDraw; x1++) {
 			xxPutPixel(dst, dx + x1, dy + y1, p);
-			xxPutPixel(dstMsk, dx + x1, dy + y1, True);
+			xxPutPixel(dstMask, dx + x1, dy + y1, True);
 		    }
 		}
 	    }
 	}
-    }
+        pix = XCreatePixmap(XtDisplay(w), XtWindow(w),
+			    width * zoom, height * zoom, 1);
 
-    if (mask != NULL) {
-	Pixmap pix = XCreatePixmap(XtDisplay(w), XtWindow(w),
-				   width * zoom, height * zoom, 1);
-
-	XPutImage(XtDisplay(w), pix, pw->paint.mgc, dstMsk,
+	XPutImage(XtDisplay(w), pix, pw->paint.mgc, dstMask,
 		  0, 0, 0, 0, width * zoom, height * zoom);
 
-	XSetClipMask(XtDisplay(w), gc, pix);
 	XSetClipOrigin(XtDisplay(w), gc, xstart * zoom, ystart * zoom);
+	XSetClipMask(XtDisplay(w), gc, pix);
 	XPutImage(XtDisplay(w), XtWindow(w), gc, dst, 0, 0,
 		  xstart * zoom, ystart * zoom,
 		  width * zoom, height * zoom);
 	XSetClipMask(XtDisplay(w), gc, None);
-
 	XFreePixmap(XtDisplay(w), pix);
-    } else {
-	XPutImage(XtDisplay(w), XtWindow(w), gc, dst, 0, 0,
-		  xstart * zoom, ystart * zoom,
-		  width * zoom, height * zoom);
     }
-
+ 
+ end:
     XDestroyImage(dst);
-    if (dstMsk != NULL)
-	XDestroyImage(dstMsk);
+    if (dstMask != NULL)
+	XDestroyImage(dstMask);
 }
 
 void
@@ -780,10 +891,17 @@ zoomUpdate(PaintWidget pw, Boolean isExpose, XRectangle * rect)
     if ((isec = RectIntersect(rect, GetRectangle(pw))) == NULL)
 	return;
 
-    zsx = sx = isec->x / zoom;
-    zsy = sy = isec->y / zoom;
-    w = (isec->width + (isec->x % zoom) + zoom - 1) / zoom;
-    h = (isec->height + (isec->y % zoom) + zoom - 1) / zoom;
+    if (zoom>0) {
+        zsx = sx = isec->x / zoom;
+        zsy = sy = isec->y / zoom;
+        w = (isec->width + (isec->x % zoom) + zoom - 1) / zoom;
+        h = (isec->height + (isec->y % zoom) + zoom - 1) / zoom;
+    } else {
+        zsx = sx = isec->x * (-zoom);
+        zsy = sy = isec->y * (-zoom);
+        w = isec->width * (-zoom);
+        h = isec->height * (-zoom);
+    }
 
     grab.x = sx + pw->paint.zoomX;
     grab.y = sy + pw->paint.zoomY;
@@ -825,21 +943,14 @@ void
 _PwZoomDraw(PaintWidget pw, Widget w, GC gc, XImage * src, XImage * mask,
 	    Boolean flag, int sx, int sy, int zoom, XRectangle * rect)
 {
-    if (rect->width * rect->height < 1024 + 256)
+    if (zoom>0 && rect->width * rect->height < 1024 + 256)
 	zoomDrawRects(pw, w, gc, src, mask, flag, sx, sy, zoom, rect);
     else
 	zoomDrawImage(pw, w, gc, src, mask, flag, sx, sy, zoom, rect);
 
-    if (pw->paint.grid) {
-	int xstart, xend, ystart, yend;
-
-	xstart = sx * zoom - 1;
-	ystart = sy * zoom - 1;
-	xend = xstart + rect->width * zoom + zoom - 1;
-	yend = ystart + rect->height * zoom + zoom - 1;
-
-	drawVisibleGrid(pw, w, True, zoom, xstart, ystart, xend, yend);
-    }
+    if (pw->paint.grid)
+	PwDrawVisibleGrid(pw, w, True, sx, sy, 
+                          sx + rect->width, sy + rect->height);
 }
 
 
@@ -866,7 +977,7 @@ PwUpdateDrawable(Widget w, Drawable draw, XRectangle * rect)
 static void
 realExposeProc(PaintWidget pw, XExposeEvent * event, Boolean flag)
 {
-    int x, y, width, height;
+  int x, y, width, height, zoom;
     XRectangle rect;
 
     if (!XtIsRealized(((Widget) pw)))
@@ -897,15 +1008,21 @@ realExposeProc(PaintWidget pw, XExposeEvent * event, Boolean flag)
     if (y + height > pw->core.height)
 	height = pw->core.height - y;
 
+    zoom = GET_ZOOM(pw);
     rect.x = x;
     rect.y = y;
-    rect.width = width;
-    rect.height = height;
+    rect.width = width+(zoom<0);
+    rect.height = height+(zoom<0);
 
-    if (GET_ZOOM(pw) <= 1)
+    if (zoom == 1) {
 	PwUpdateDrawable((Widget) pw, XtWindow((Widget) pw), &rect);
-    else
+        if (pw->paint.grid)
+	    PwDrawVisibleGrid(pw, (Widget)pw, True, x, y, 
+                             x + rect.width, y + rect.height);
+    } else
 	zoomUpdate(pw, flag, &rect);
+
+    XFlush(XtDisplay(pw));
 }
 
 static void
@@ -915,11 +1032,13 @@ ExposeProc(Widget w, XExposeEvent * event)
 }
 
 static void
-resizePixmap(PaintWidget w, Pixmap * pixmap)
+resizePixmap(PaintWidget w, AlphaPixmap * alphapix)
 {
     Pixmap n;
+    unsigned char * alpha = NULL;
+    int w1, h1, w2, h2, y;
 
-    if (pixmap == NULL || *pixmap == None)
+    if (alphapix == NULL || alphapix->pixmap == None)
 	return;
 
     n = XCreatePixmap(XtDisplay(w), XtWindow(w),
@@ -928,11 +1047,27 @@ resizePixmap(PaintWidget w, Pixmap * pixmap)
     XFillRectangle(XtDisplay(w), n,
 		   w->paint.igc, 0, 0,
 		   w->paint.drawWidth, w->paint.drawHeight);
-    XCopyArea(XtDisplay(w), *pixmap, n,
+    XCopyArea(XtDisplay(w), alphapix->pixmap, n,
 	      w->paint.gc, 0, 0,
 	      w->paint.drawWidth, w->paint.drawHeight, 0, 0);
-    XFreePixmap(XtDisplay(w), *pixmap);
-    *pixmap = n;
+
+    if (alphapix->alpha) {
+        GetPixmapWHD(XtDisplay(w), alphapix->pixmap, &w1, &h1, &y);
+        y = w->paint.drawWidth * w->paint.drawHeight;
+        alpha = (unsigned char *) XtMalloc(y);
+        memset(alpha, (unsigned char)Global.alpha_default, y);
+        w2 = w1;
+        if (w2>w->paint.drawWidth) w2 = w->paint.drawWidth;
+        h2 = h1;
+        if (h2>w->paint.drawHeight) h2 = w->paint.drawHeight;
+        for (y=0; y<h2; y++)
+	    memcpy(alpha+y*w->paint.drawWidth, alphapix->alpha+y*w1, w2);
+	free(alphapix->alpha);
+    }
+
+    XFreePixmap(XtDisplay(w), alphapix->pixmap);
+    alphapix->pixmap = n;
+    alphapix->alpha = alpha;
 }
 
 static Boolean
@@ -943,7 +1078,7 @@ SetValuesProc(Widget curArg, Widget request,
     PaintWidget new = (PaintWidget) newArg;
     int ret = False;
     int sizeChanged = False;
-    int i;
+    int i, zoom;
 
     if (cur->paint.sourcePixmap != new->paint.sourcePixmap)
 	pwSetPixmap(new, new->paint.sourcePixmap, True);
@@ -992,11 +1127,11 @@ SetValuesProc(Widget curArg, Widget request,
 	if (new->paint.paint == None) {
 	    cur = new->paint.head;
 	    do {
-		if (cur->pixmap == new->paint.current)
+		if (cur->alphapix.pixmap == new->paint.current.pixmap)
 		    gotit = 1;
-		resizePixmap(new, &cur->pixmap);
+		resizePixmap(new, &cur->alphapix);
 		if (gotit) {
-		    new->paint.current = cur->pixmap;
+		    new->paint.current = cur->alphapix;
 		    gotit = 0;
 		}
 	    } while ((cur = cur->next) != new->paint.head);
@@ -1011,13 +1146,20 @@ SetValuesProc(Widget curArg, Widget request,
 	    XDestroyRegion(new->paint.imageRegion);
 	    new->paint.image = NULL;
 	}
-	if (XtMakeResizeRequest((Widget) new,
+	if (new->paint.zoom>0 && XtMakeResizeRequest((Widget) new,
 				new->paint.drawWidth * new->paint.zoom,
 				new->paint.drawHeight * new->paint.zoom,
 				NULL, NULL) == XtGeometryDone) {
 	    ret = True;
 	    sizeChanged = True;
 	}
+	if (new->paint.zoom<0 && XtMakeResizeRequest((Widget) new,
+                (new->paint.drawWidth-new->paint.zoom-1)/(-new->paint.zoom),
+                (new->paint.drawHeight-new->paint.zoom-1)/(-new->paint.zoom),
+		NULL, NULL) == XtGeometryDone) {
+	    ret = True;
+	    sizeChanged = True;
+	}
 	for (i = 0; i < new->paint.paintChildrenSize; i++) {
 	    XtVaSetValues(new->paint.paintChildren[i],
 			  XtNdrawWidth, new->paint.drawWidth,
@@ -1028,10 +1170,18 @@ SetValuesProc(Widget curArg, Widget request,
     if (cur->paint.zoom != new->paint.zoom) {
 	ret = True;
 
-	XtMakeResizeRequest((Widget) new,
+        if (new->paint.zoom>0)
+	    XtMakeResizeRequest((Widget) new,
 			    new->paint.drawWidth * new->paint.zoom,
 			    new->paint.drawHeight * new->paint.zoom,
 			    NULL, NULL);
+        else {
+	    zoom = - new->paint.zoom;
+	    XtMakeResizeRequest((Widget) new,
+				(new->paint.drawWidth+zoom-1)/zoom,
+			        (new->paint.drawHeight+zoom-1)/zoom,
+			        NULL, NULL);
+	}
 	sizeChanged = True;
     }
     if (cur->paint.zoomX != new->paint.zoomX ||
@@ -1046,16 +1196,15 @@ SetValuesProc(Widget curArg, Widget request,
 	realExposeProc(new, &event, False);
 	pwRegionZoomPosChanged(new);
     }
+    zoom = GET_ZOOM(new);
     if (cur->paint.grid != new->paint.grid) {
 	XRectangle *rect = GetRectangle(new);
 
-	rect->x -= rect->x % new->paint.zoom + 1;
-	rect->y -= rect->y % new->paint.zoom + 1;
-
-	drawVisibleGrid(new, (Widget) new,
-			new->paint.grid, GET_ZOOM(new),
+	PwDrawVisibleGrid(new, (Widget) new,
+			new->paint.grid,
 			rect->x, rect->y,
-			rect->x + rect->width, rect->y + rect->height);
+			rect->x + rect->width, 
+                        rect->y + rect->height);
 
 	for (i = 0; i < new->paint.paintChildrenSize; i++) {
 	    XtVaSetValues(new->paint.paintChildren[i],
@@ -1083,8 +1232,10 @@ SetValuesProc(Widget curArg, Widget request,
 	this = cur->paint.head;
 	do {
 	    nxt = this->next;
-	    if (this->pixmap != cur->paint.current)
-		XFreePixmap(XtDisplay(new), this->pixmap);
+	    if (this->alphapix.pixmap != cur->paint.current.pixmap) {
+		XFreePixmap(XtDisplay(new), this->alphapix.pixmap);
+                XtFree((char *)this->alphapix.alpha);
+	    }
 	    free(this);
 	    this = nxt;
 	} while (this != cur->paint.head);
@@ -1093,15 +1244,26 @@ SetValuesProc(Widget curArg, Widget request,
 	    this = undoBuffer(new);
 	    if (i == 0) {
 		first = this;
-		this->pixmap = cur->paint.current;
+		this->alphapix = cur->paint.current;
+                if (cur->paint.current.alpha) {
+                    int m = cur->paint.drawWidth * cur->paint.drawHeight;
+		    if (!this->alphapix.alpha)
+		        this->alphapix.alpha = XtMalloc(m);
+                    memcpy(this->alphapix.alpha, cur->paint.current.alpha, m);
+		}
 	    } else {
-		this->pixmap = XCreatePixmap(XtDisplay(new), XtWindow(new),
-					     new->paint.drawWidth,
-					     new->paint.drawHeight,
-					 DefaultDepthOfScreen(XtScreen(new)));
-		XFillRectangle(XtDisplay(new), this->pixmap,
+		this->alphapix.pixmap = 
+                    XCreatePixmap(XtDisplay(new), XtWindow(new),
+			          new->paint.drawWidth,
+				  new->paint.drawHeight,
+				  DefaultDepthOfScreen(XtScreen(new)));
+		XFillRectangle(XtDisplay(new), this->alphapix.pixmap,
 			       new->paint.igc, 0, 0,
 			       new->paint.drawWidth, new->paint.drawHeight);
+                if (this->alphapix.alpha)
+		    memset(this->alphapix.alpha, 
+                           (unsigned char) Global.alpha_default, 
+                           new->paint.drawWidth * new->paint.drawHeight);
 	    }
 	}
 	first->next = new->paint.head;
@@ -1158,9 +1320,16 @@ QueryGeometryProc(Widget w, XtWidgetGeometry * proposed,
  */
 #define SETRET(ret,x)	ret = (x > ret) ? x : ret
     PaintWidget pw = (PaintWidget) w;
-    int width = pw->paint.drawWidth * pw->paint.zoom;
-    int height = pw->paint.drawHeight * pw->paint.zoom;
+    int width; 
+    int height;
 
+    if (pw->paint.zoom>0) {
+        width = pw->paint.drawWidth * pw->paint.zoom;
+        height = pw->paint.drawHeight * pw->paint.zoom;
+    } else {
+        width = (pw->paint.drawWidth-pw->paint.zoom-1)/(-pw->paint.zoom);
+        height = (pw->paint.drawHeight-pw->paint.zoom-1)/(-pw->paint.zoom);
+    }
     reply->request_mode = CWWidth | CWHeight;
     reply->width = width;
     reply->height = height;
@@ -1263,6 +1432,7 @@ PwGetImage(Widget w, XRectangle * rect)
     return pw->paint.image;
 }
 
+/*  Get a copy of the pixmap of the current canvas image */
 void
 PwGetPixmap(Widget w, Pixmap * pix, int *width, int *height)
 {
@@ -1273,6 +1443,7 @@ PwGetPixmap(Widget w, Pixmap * pix, int *width, int *height)
 	PwGetPixmap((Widget) pw->paint.paint, pix, width, height);
 	return;
     }
+
     if (width != NULL)
 	*width = pw->paint.drawWidth;
     if (height != NULL)
@@ -1281,10 +1452,13 @@ PwGetPixmap(Widget w, Pixmap * pix, int *width, int *height)
     *pix = XCreatePixmap(XtDisplay(w), XtWindow(w),
 			 pw->paint.drawWidth, pw->paint.drawHeight,
 			 pw->core.depth);
-    PwUpdateDrawable(w, *pix, NULL);
+
+    if (*pix != None)
+        XCopyArea(XtDisplay(w), GET_PIXMAP(pw), *pix, pw->paint.gc,
+	          0, 0, pw->paint.drawWidth, pw->paint.drawHeight, 0, 0);
 }
 
-/*  Get the pixmap id of the current region image */
+/*  Get the pixmap id of the current canvas image */
 Pixmap
 PwGetRawPixmap(Widget w)
 {
@@ -1321,10 +1495,17 @@ fatCallback(Widget parent, XtPointer w, XtPointer rectArg)
     /*
     **	Make this look like an expose event on the fatbits region
      */
-    event.x = (rect->x - pw->paint.zoomX) * zoom;
-    event.y = (rect->y - pw->paint.zoomY) * zoom;
-    event.width = rect->width * zoom;
-    event.height = rect->height * zoom;
+    if (zoom>0) {
+        event.x = (rect->x - pw->paint.zoomX) * zoom;
+        event.y = (rect->y - pw->paint.zoomY) * zoom;
+        event.width = rect->width * zoom;
+        event.height = rect->height * zoom;
+    } else {
+        event.x = (rect->x - pw->paint.zoomX) / (-zoom);
+        event.y = (rect->y - pw->paint.zoomY) / (-zoom);
+        event.width = (rect->width-zoom-1) / (-zoom);
+        event.height = (rect->height-zoom-1) / (-zoom);
+    }
 
     realExposeProc(pw, &event, False);
 }
@@ -1367,8 +1548,14 @@ PwUpdate(Widget w, XRectangle * rect, Boolean force)
     XtCallCallbackList((Widget) usePW, usePW->paint.fatcalls,
 		       (XtPointer) rect);
 
-    if (force || (pw->paint.zoom > 1) || parent != None)
+    if (force || (abs(pw->paint.zoom) > 1) || parent != None)
 	fatCallback((Widget) usePW, (XtPointer) usePW, (XtPointer) rect);
+
+    if (pw->paint.grid)
+	PwDrawVisibleGrid(pw, w, True, rect->x, rect->y, 
+                          rect->x + rect->width, rect->y + rect->height);
+
+    XFlush(XtDisplay(pw));
 }
 
 /*
@@ -1387,11 +1574,14 @@ PwUpdateFromLast(Widget w, XRectangle * rect)
     if (pw->paint.paint != None)
 	pw = (PaintWidget) pw->paint.paint;
 
-    pix = pw->paint.undo->pixmap;
+    pix = pw->paint.undo->alphapix.pixmap;
 
     XCopyArea(XtDisplay(w), pix, draw, pw->paint.gc,
 	      rect->x, rect->y, rect->width, rect->height,
 	      rect->x, rect->y);
+
+    memcpy(pw->paint.current.alpha, pw->paint.undo->alphapix.alpha,
+           rect->width * rect->height);
     PwUpdate(w, rect, True);
 }
 
@@ -1409,7 +1599,7 @@ PwPutPixmap(Widget w, Pixmap pix)
 {
     PaintWidget pw = (PaintWidget) w;
     GC gc = pw->paint.gc;
-    Pixmap dst;
+    AlphaPixmap dst;
     Window root;
     int x, y;
     unsigned int width, height, bw, depth;
@@ -1421,8 +1611,10 @@ PwPutPixmap(Widget w, Pixmap pix)
     rect.width = width;
     rect.height = height;
     dst = PwUndoStart(w, &rect);
-    XCopyArea(XtDisplay(pw), pix, dst, gc, 0, 0,
+    XCopyArea(XtDisplay(pw), pix, dst.pixmap, gc, 0, 0,
 	      width, height, 0, 0);
+    if (pw->paint.current.alpha)
+        memcpy(dst.alpha, pw->paint.current.alpha, width*height);
     PwUpdate(w, &rect, True);
 }
 
