@@ -16,7 +16,7 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: Paint.h,v 1.8 1996/04/19 08:52:40 torsten Exp $ */
+/* $Id: Paint.h,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
 
 /****************************************************************
  *
@@ -45,13 +45,21 @@
 
 #define XtNpaintResource "paintResource"
 #define XtNpattern	 "pattern"
+#define XtNmenubar	 "menubar"
+#define XtNfullmenu	 "fullmenu"
 #define XtNgrid	 	 "grid"
-#define XtNsnap	 	 "snap"
+#define XtNgridmode 	 "gridmode"
+#define XtNgridcolor 	 "gridcolor"
+#define XtNinterpolation "interpolation"
+#define XtNtransparent   "transparent"
+#define XtNsnapX 	 "snapX"
+#define XtNsnapY 	 "snapY"
 #define XtNsnapOn 	 "snapOn"
 #define XtNlineWidth	 "lineWidth"
 #define XtNfatBack	 "fatBack"
 #define XtNsizeChanged	 "sizeChanged"
 #define XtNundoSize	 "undoSize"
+#define XtNlocked 	 "locked"
 #define XtNzoom	 	 "zoom"
 #define XtNpaint	 "paint"
 #define XtNzoomX	 "zoomX"
@@ -60,6 +68,7 @@
 #define XtNdrawHeight	 "drawHeight"
 #define XtNcompress	 "compress"
 #define XtNdirty	 "dirty"
+#define XtNeditable	 "editable"
 #define XtNfillRule	 "fillRule"
 #define XtNregionCallback	"regionSetCallback"
 #define XtNdownX		"downX"
@@ -78,12 +87,20 @@
 
 #define XtCPaintResource "PaintResource"
 #define XtCPattern	 "Pattern"
+#define XtCMenubar	 "Menubar"
+#define XtCFullMenu	 "FullMenu"
 #define XtCGrid		 "Grid"
-#define XtCSnap		 "Snap"
+#define XtCGridMode	 "GridMode"
+#define XtCGridColor	 "GridColor"
+#define XtCInterpolation "Interpolation"
+#define XtCTransparent   "Transparent"
+#define XtCSnapX	 "SnapX"
+#define XtCSnapY	 "SnapY"
 #define XtCSnapOn	 "SnapOn"
 #define XtCLineWidth	 "LineWidth"
 #define XtCFatBack	 "FatBack"
 #define XtCSizeChanged	 "SizeChanged"
+#define XtCLocked	 "Locked"
 #define XtCUndoSize	 "UndoSize"
 #define XtCZoom	  	 "Zoom"
 #define XtCPaint	 "Paint"
@@ -93,6 +110,7 @@
 #define XtCDrawHeight	 "DrawHeight"
 #define XtCCompress	 "Compress"
 #define XtCDirty	 "Dirty"
+#define XtCEditable	 "Editable"
 #define XtCFillRule	 "FillRule"
 #define XtCRegionCallback	"RegionSetCallback"
 #define XtCDownX		"DownX"
@@ -118,6 +136,12 @@ extern WidgetClass paintWidgetClass;
 /*
 **  Operation callback information
  */
+
+typedef struct {
+    Pixmap pixmap;
+    unsigned char *alpha;
+} AlphaPixmap;
+
 typedef enum {
     opPixmap = 0x01, opWindow = 0x02
 } OpSurface;
@@ -139,12 +163,12 @@ typedef float pwMatrix[2][2];
 
 /* Brush Item structure */
 typedef struct {
-    Pixmap pixmap;
-    Cursor cursor;
-    char **bits;
     int width, height;
-    char *brushbits;
     int numpixels;		/* Total number of set pixels */
+    char *brushbits;
+    Pixmap pixmap, bw;
+    Cursor cursor;
+    Widget icon;
 } BrushItem;
 
 typedef void (*OpEventProc) (Widget, void *, XEvent *, OpInfo *);
@@ -156,7 +180,7 @@ void UndoStartPoint(Widget, OpInfo *, int, int);
 void UndoSetRectangle(Widget, XRectangle *);
 void UndoGrow(Widget, int, int);
 
-Pixmap PwUndoStart(Widget, XRectangle *);
+AlphaPixmap PwUndoStart(Widget, XRectangle *);
 void PwUndoSetRectangle(Widget, XRectangle *);
 void PwUndoAddRectangle(Widget, XRectangle *);
 void Undo(Widget);
@@ -177,10 +201,13 @@ void PwGetPixmap(Widget, Pixmap *, int *, int *);
 void PwPutPixmap(Widget w, Pixmap pix);
 XRectangle *PwScaleRectangle(Widget, XRectangle *);
 XImage *PwGetImage(Widget, XRectangle *);
+void zoomUpdate(PaintWidget pw, Boolean isExpose, XRectangle * rect);
 
 /*
 **  Region routines
  */
+void PwDrawVisibleGrid(PaintWidget, Widget, Boolean, int, int, int, int);
+void PwRegionExpose(Widget, PaintWidget, XEvent *, Boolean *);
 void PwRegionSet(Widget, XRectangle *, Pixmap, Pixmap);
 void PwRegionSetRawPixmap(Widget, Pixmap);
 void PwRegionTear(Widget);
