@@ -16,7 +16,7 @@
 /* |								       | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: PaintP.h,v 1.8 1996/04/19 09:03:59 torsten Exp $ */
+/* $Id: PaintP.h,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
 
 #include "Paint.h"
 
@@ -44,7 +44,7 @@ extern PaintClassRec paintClassRec;
 
 typedef struct s_undoStack {
     XRectangle box;
-    Pixmap pixmap;
+    AlphaPixmap alphapix;
     Boolean pushed;
     struct s_undoStack *next;
 } UndoStack;
@@ -58,27 +58,25 @@ typedef struct {
     int lineWidth;
     XFontStruct *fontInfo;
     Pixel foreground, background, lineForeground;
-    int undoSize;		/* number of undo stacks */
 
     /*
     **	The undo system
      */
-    Pixmap current;		/* The currently displayed pixmap */
+    AlphaPixmap current;	/* The currently displayed pixmap */
     UndoStack * undo, * head;
-    Boolean dirty;
     /* The undo/redo stacks: */
-    int nlevels;		/* Maximum stack depth */
-    Pixmap * undostack, * redostack;
+    AlphaPixmap * undostack, * redostack;
     int undobot, undoitems, redobot, redoitems;
+    int nlevels;		/* Maximum stack depth */
+    int undoSize;		/* number of undo stacks */
 
-    Boolean snapOn, grid;
-    int snap;
+    Boolean dirty, editable, locked, menubar, fullmenu, snapOn, grid, compress;
+    int snap_x, snap_y, gridmode, interpolation, transparent;
     void *eventList;
     XtCallbackList fatcalls;
     XtCallbackList sizecalls;
     XtCallbackList regionCalls;
-    int zoomX, zoomY;
-    int zoom;
+    int zoomX, zoomY, zoom;
     GC tgc;			/* scratch GC, for stuffing clip masks into */
     GC mgc;			/* region mask GC, 1 bit deep */
     GC xgc;			/* XOR (invert) GC */
@@ -86,7 +84,7 @@ typedef struct {
     int paintChildrenSize;
     Widget *paintChildren;
     Pixel linePixel;
-    Boolean compress;
+    Pixel gridcolor;
     Cursor cursor;
 
     /*
@@ -109,9 +107,10 @@ typedef struct {
 
 	Pixmap source, mask, notMask;
 	XImage *sourceImg, *maskImg, *notMaskImg;
+        unsigned char *alpha;
 
 	Boolean isAttached, isVisible, needResize;
-	Pixmap undoPixmap, unfilterPixmap;
+	Pixmap unfilterPixmap;
 	/*
 	**  The scale{X,Y} are combined with the rotMat
 	**    to produce mat.
@@ -127,7 +126,6 @@ typedef struct {
     Boolean invalidateRegion;
     Region imageRegion;
     XImage *image;
-
     Position downX, downY;	/* Last button down X & Y pos */
 
     /*
@@ -150,7 +148,7 @@ typedef struct _PaintRec {
 } PaintRec;
 
 
-#define _GET_PIXMAP(pw) ((pw)->paint.current)
+#define _GET_PIXMAP(pw) ((pw)->paint.current.pixmap)
 #define GET_PIXMAP(pw) (pw->paint.paint ? \
 		_GET_PIXMAP((PaintWidget)pw->paint.paint) : _GET_PIXMAP(pw))
 
