@@ -24,18 +24,21 @@
 #define	GET_PW(w)	((PaintWidget)(((PaintWidget)w)->paint.paint == None ? \
 				       w : ((PaintWidget)w)->paint.paint))
 
+static AlphaPixmap NULLAP = { None, NULL};
+
+static void UndoPush(PaintWidget pw, AlphaPixmap p);
 static void RedoClear(PaintWidget pw);
-static void UndoPush(PaintWidget pw, Pixmap p);
-static Pixmap UndoPop(PaintWidget pw);
-static void RedoPush(PaintWidget pw, Pixmap p);
-static Pixmap RedoPop(PaintWidget pw);
+static void RedoPush(PaintWidget pw, AlphaPixmap p);
+static AlphaPixmap UndoPop(PaintWidget pw);
+static AlphaPixmap RedoPop(PaintWidget pw);
+
 
 void
 UndoInitialize(PaintWidget pw, int n)
 {
     pw->paint.nlevels = n;
-    pw->paint.undostack = n ? malloc(n*sizeof(Pixmap)) : NULL;
-    pw->paint.redostack = n ? malloc(n*sizeof(Pixmap)) : NULL;
+    pw->paint.undostack = n ? xmalloc(n*sizeof(AlphaPixmap)) : NULL;
+    pw->paint.redostack = n ? xmalloc(n*sizeof(AlphaPixmap)) : NULL;
     pw->paint.undobot = pw->paint.undoitems = 0;
     pw->paint.redobot = pw->paint.redoitems = 0;
 }
@@ -43,24 +46,27 @@ UndoInitialize(PaintWidget pw, int n)
 static void
 RedoClear(PaintWidget pw)
 {
-    Pixmap p;
+    AlphaPixmap p;
     
-    while ((p = RedoPop(pw)) != None)
-	;
+    do {
+        p = RedoPop(pw);
+    } while (p.pixmap != None);
 }
 
 static void
-UndoPush(PaintWidget pw, Pixmap p)
+UndoPush(PaintWidget pw, AlphaPixmap p)
 {
     UndoStack * s, * h;
 
     if (pw->paint.nlevels == 0)
 	return;
     
+    /* printf("push : %d %d\n", p.pixmap, p.alpha); */
+
     h = pw->paint.head;
     s = h;
     do {
-	if (s->pixmap == p) {
+	if (s->alphapix.pixmap == p.pixmap) {
 	    s->pushed = True;
 	    pw->paint.undo = s;
 	    break;
@@ -71,12 +77,12 @@ UndoPush(PaintWidget pw, Pixmap p)
 	pw->paint.undostack[pw->paint.undoitems++] = p;
     else {
 	int bot = pw->paint.undobot;
-	Pixmap bp;
+	AlphaPixmap bp;
 
 	bp = pw->paint.undostack[bot];
 	s = h;
 	do {
-	    if (s->pixmap == bp) {
+	    if (s->alphapix.pixmap == bp.pixmap) {
 		s->pushed = False;
 		break;
 	    }
@@ -87,21 +93,21 @@ UndoPush(PaintWidget pw, Pixmap p)
     }
 }
 
-static Pixmap
+static AlphaPixmap
 UndoPop(PaintWidget pw)
 {
-    Pixmap p;
+    AlphaPixmap p;
     UndoStack * s, * h;
     
     if (pw->paint.undoitems == 0)
-        return None;
+        return NULLAP;
     --pw->paint.undoitems;
     p = pw->paint.undostack[(pw->paint.undobot + pw->paint.undoitems)
 			   % pw->paint.nlevels];
     h = pw->paint.head;
     s = h;
     do {
-	if (s->pixmap == p) {
+	if (s->alphapix.pixmap == p.pixmap) {
 	    s->pushed = False;
 	    break;
 	}
@@ -111,7 +117,7 @@ UndoPop(PaintWidget pw)
 }
 
 static void
-RedoPush(PaintWidget pw, Pixmap p)
+RedoPush(PaintWidget pw, AlphaPixmap p)
 {
     UndoStack * s, * h;
 
@@ -120,7 +126,7 @@ RedoPush(PaintWidget pw, Pixmap p)
     h = pw->paint.head;
     s = h;
     do {
-	if (s->pixmap == p) {
+	if (s->alphapix.pixmap == p.pixmap) {
 	    s->pushed = True;
 	    break;
 	}
@@ -130,12 +136,12 @@ RedoPush(PaintWidget pw, Pixmap p)
 	pw->paint.redostack[pw->paint.redoitems++] = p;
     else {
 	int bot = pw->paint.redobot;
-	Pixmap bp;
+	AlphaPixmap bp;
 
 	bp = pw->paint.redostack[bot];
 	s = h;
 	do {
-	    if (s->pixmap == bp) {
+	    if (s->alphapix.pixmap == bp.pixmap) {
 		s->pushed = False;
 		break;
 	    }
@@ -147,21 +153,21 @@ RedoPush(PaintWidget pw, Pixmap p)
     }
 }
 
-static Pixmap
+static AlphaPixmap
 RedoPop(PaintWidget pw)
 {
-    Pixmap p;
+    AlphaPixmap p;
     UndoStack * s, * h;
     
     if (pw->paint.redoitems == 0)
-	return None;
+	return NULLAP;
     --pw->paint.redoitems;
     p = pw->paint.redostack[(pw->paint.redobot + pw->paint.redoitems)
 			   % pw->paint.nlevels];
     h = pw->paint.head;
     s = h;
     do {
-	if (s->pixmap == p) {
+	if (s->alphapix.pixmap == p.pixmap) {
 	    s->pushed = False;
 	    break;
 	}
@@ -174,10 +180,10 @@ void
 Undo(Widget w)
 {
     PaintWidget pw = GET_PW(w);
-    Pixmap popped;
+    AlphaPixmap popped;
 
     popped = UndoPop(pw);
-    if (popped == None) {
+    if (popped.pixmap == None) {
 #ifdef ERRORBEEP
 	XBell(XtDisplay(w), 100);
 #endif
@@ -185,6 +191,7 @@ Undo(Widget w)
     }
     RedoPush(pw, pw->paint.current);
     pw->paint.current = popped;
+    /* printf("undo : %d %d\n", popped.pixmap, popped.alpha); */
     PwUpdate(w, NULL, True);
 }
 
@@ -192,10 +199,10 @@ void
 Redo(Widget w)
 {
     PaintWidget pw = GET_PW(w);
-    Pixmap popped;
+    AlphaPixmap popped;
 
     popped = RedoPop(pw);
-    if (popped == None) {
+    if (popped.pixmap == None) {
 #ifdef ERRORBEEP
 	XBell(XtDisplay(w), 100);
 #endif
@@ -209,17 +216,19 @@ Redo(Widget w)
 /*
  * Called when starting a drawing operation.
  */
-Pixmap
+AlphaPixmap
 PwUndoStart(Widget wid, XRectangle * rect)
 {
     PaintWidget pw = GET_PW(wid);
     UndoStack * s;
-    Pixmap cur;
-    int x, y, w, h;
+    AlphaPixmap cur;
+    int x, y, w, h, m;
 
     pw->paint.dirty = True;
 
-    cur = GET_PIXMAP(pw);
+    cur.pixmap = GET_PIXMAP(pw);
+    cur.alpha = pw->paint.current.alpha;
+
     UndoPush(pw, cur);
     RedoClear(pw);
     
@@ -244,21 +253,30 @@ PwUndoStart(Widget wid, XRectangle * rect)
     s->box.width = w;
     s->box.height = h;
 
-    XCopyArea(XtDisplay(pw), cur, s->pixmap,
-    pw->paint.gc, 0, 0, pw->paint.drawWidth, pw->paint.drawHeight, 0, 0);
+    XCopyArea(XtDisplay(pw), cur.pixmap, s->alphapix.pixmap,
+        pw->paint.gc, 0, 0, pw->paint.drawWidth, pw->paint.drawHeight, 0, 0);
 
-    pw->paint.current = s->pixmap;
-    return s->pixmap;
+    if (cur.alpha) {
+        m = pw->paint.drawWidth * pw->paint.drawHeight;
+        if (!s->alphapix.alpha)
+	    s->alphapix.alpha = (unsigned char *)XtMalloc(m);
+        memcpy(s->alphapix.alpha, cur.alpha, m);
+    }
+
+    pw->paint.current = s->alphapix;
+    return s->alphapix;
 }
 
 
 void 
 UndoStart(Widget w, OpInfo * info)
 {
+    AlphaPixmap start;
     if (info->surface != opPixmap)
 	return;
 
-    info->drawable = PwUndoStart(w, NULL);
+    start = PwUndoStart(w, NULL);
+    info->drawable = start.pixmap;
 }
 
 
@@ -288,6 +306,8 @@ UndoGrow(Widget w, int x, int y)
     XRectangle *rect;
     int dx, dy;
 
+    /* printf("point %d %d\n", x, y); */
+
     if (pw->paint.undo == NULL)
 	return;
 
@@ -319,6 +339,11 @@ UndoGrow(Widget w, int x, int y)
     rect->y -= pw->paint.lineWidth;
     rect->width += pw->paint.lineWidth * 2 + 1;
     rect->height += pw->paint.lineWidth * 2 + 1;
+
+    /*
+    printf("undogrow (%d) %d %d [%d %d]\n",
+           rect, rect->x, rect->y, rect->width, rect->height);
+    */
 }
 
 void 
