@@ -21,14 +21,13 @@
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
+#include <X11/xpm.h>
 
 #include "xaw_incdir/Box.h"
 #include "xaw_incdir/Form.h"
 #include "xaw_incdir/Command.h"
 #include "xaw_incdir/Toggle.h"
 
-#include <xpm.h>
-
 #include "xpaint.h"
 #include "misc.h"
 #include "Paint.h"
@@ -40,61 +39,11 @@
 #include "ops.h"
 #include "rc.h"
 
-
-#include "bitmaps/brushes/paintA.xpm"
-#include "bitmaps/brushes/paintB.xpm"
-#include "bitmaps/brushes/paintC.xpm"
-#include "bitmaps/brushes/paintD.xpm"
-#include "bitmaps/brushes/paintE.xpm"
-#include "bitmaps/brushes/paintF.xpm"
-#include "bitmaps/brushes/paintG.xpm"
-#include "bitmaps/brushes/paintH.xpm"
-#include "bitmaps/brushes/paintI.xpm"
-#include "bitmaps/brushes/paintJ.xpm"
-#include "bitmaps/brushes/paintK.xpm"
-#include "bitmaps/brushes/paintL.xpm"
-#include "bitmaps/brushes/paintM.xpm"
-#include "bitmaps/brushes/paintN.xpm"
-#include "bitmaps/brushes/paintO.xpm"
-#include "bitmaps/brushes/paintP.xpm"
-#include "bitmaps/brushes/paintQ.xpm"
-#include "bitmaps/brushes/paintR.xpm"
-#include "bitmaps/brushes/paintS.xpm"
-#include "bitmaps/brushes/paintT.xpm"
-
 enum { OPAQUE, TRANSPARENT, STAIN };
 
-#define BRUSH(name) (char **) CONCAT(name, _xpm)
-
 /* this blend function ensures to reach the final color - ACZ: */
 #define BLEND(a, b, x)	((x)*(b) + (1.0039-(x))*(a))
 
-static BrushItem baseBrushList[] =
-{
-    {None, None, BRUSH(paintA)},
-    {None, None, BRUSH(paintB)},
-    {None, None, BRUSH(paintC)},
-    {None, None, BRUSH(paintD)},
-    {None, None, BRUSH(paintE)},
-    {None, None, BRUSH(paintF)},
-    {None, None, BRUSH(paintG)},
-    {None, None, BRUSH(paintH)},
-    {None, None, BRUSH(paintI)},
-    {None, None, BRUSH(paintJ)},
-    {None, None, BRUSH(paintK)},
-    {None, None, BRUSH(paintL)},
-    {None, None, BRUSH(paintM)},
-    {None, None, BRUSH(paintN)},
-    {None, None, BRUSH(paintO)},
-    {None, None, BRUSH(paintP)},
-    {None, None, BRUSH(paintQ)},
-    {None, None, BRUSH(paintR)},
-    {None, None, BRUSH(paintS)},
-    {None, None, BRUSH(paintT)}
-};
-
-#define BBNUMBER XtNumber(baseBrushList)
-
 static XpmColorSymbol monoColorSymbols[5] =
 {
     {"A", NULL, 0},
@@ -109,7 +58,7 @@ typedef enum {
 } BrushType;
 
 typedef struct {
-    Widget form, box, close;   
+    Widget shell, form, box, close;   
 } BrushboxInfo;
 
 typedef struct {
@@ -122,11 +71,14 @@ typedef struct {
     Palette *brushPalette;
 } LocalInfo;
 
-static BrushItem *currentBrush = NULL;
-static Boolean eraseMode = True;
-static int transparentMode = 0;
+static BrushItem * brushList = NULL;
+static int brushnum = 0, BBNUM = 0;
+static int brushdef = 0;
+static int currentBrush;
 static float brushOpacity = 0.2;
 static XImage *brushImage;
+static Boolean eraseMode = True;
+static int transparentMode = 0;
 
 /* RPC */
 static void stain(Widget w, OpInfo * info, GC gc, LocalInfo * l, int sx, int sy);
@@ -139,6 +91,7 @@ static void wbrush(Widget w, OpInfo * info, GC gc,
 static void 
 draw(Widget w, OpInfo * info, LocalInfo * l, int x, int y)
 {
+    PaintWidget pw = (PaintWidget) w;
     XRectangle undo;
     int sx = x - l->width / 2;
     int sy = y - l->height / 2;
@@ -163,24 +116,26 @@ draw(Widget w, OpInfo * info, LocalInfo * l, int x, int y)
 	stain(w, info, gc, l, sx, sy );
 /* RPC */
     } else {			/* plain opaque brush */
-	XFillRectangle(XtDisplay(w), info->drawable,
+        XFillRectangle(XtDisplay(w), info->drawable,
 		       gc, sx, sy, l->width, l->height);
     }
 
-    if (info->surface == opPixmap) {
-	XYtoRECT(sx, sy, sx + l->width, sy + l->height, &undo);
-	UndoGrow(w, sx, sy);
-	UndoGrow(w, sx + l->width, sy + l->height);
-	PwUpdate(w, &undo, False);
+    if (info->surface == opPixmap || pw->paint.grid) {
+        XYtoRECT(sx, sy, sx + l->width, sy + l->height, &undo);
+        UndoGrow(w, sx, sy);
+        UndoGrow(w, sx + l->width, sy + l->height);
+        PwUpdate(w, &undo, pw->paint.grid);
     }
 }
 
 static void 
 press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 {
+    BrushItem * brush = &brushList[currentBrush];
     /*
     **  Check to make sure all buttons are up, before doing this
      */
+
     if (event->button >= Button4) return;   
     if ((event->state & (Button1Mask | Button2Mask | Button3Mask |
 			 Button4Mask | Button5Mask)) != 0)
@@ -191,8 +146,8 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 	return;
 
     l->useSecond = (event->button == Button2);
-    l->width = currentBrush->width;
-    l->height = currentBrush->height;
+    l->width = brush->width;
+    l->height = brush->height;
     l->tracking = True;
 
     XSetClipMask(XtDisplay(w), info->first_gc, l->pixmap);
@@ -380,6 +335,7 @@ hsvTOrgb(float h, float s, float v, float *r, float *g, float *b)
 static void
 stain(Widget wid, OpInfo * info, GC gc, LocalInfo * l, int sx, int sy)
 {
+    BrushItem * brush = &brushList[currentBrush];
     int x, y, dx, dy, w, h, d;
     unsigned char *brushbits;
     Display *dpy = XtDisplay(wid);
@@ -428,7 +384,7 @@ stain(Widget wid, OpInfo * info, GC gc, LocalInfo * l, int sx, int sy)
     XGetSubImage(dpy, info->drawable, sx, sy, w, h,
 		 AllPlanes, ZPixmap, brushImage, dx, dy);
 
-    brushbits = (unsigned char *) currentBrush->brushbits;
+    brushbits = (unsigned char *) brush->brushbits;
 
     for (y = 0; y < l->height; ++y)
 	for (x = 0; x < l->width; ++x)
@@ -466,6 +422,7 @@ stain(Widget wid, OpInfo * info, GC gc, LocalInfo * l, int sx, int sy)
 static void 
 smear(Widget wid, OpInfo * info, GC gc, LocalInfo * l, int sx, int sy)
 {
+    BrushItem * brush = &brushList[currentBrush];
     int x, y, n, dx, dy, w, h, d, m;
     unsigned long r, g, b;
     unsigned char *brushbits;
@@ -503,7 +460,7 @@ smear(Widget wid, OpInfo * info, GC gc, LocalInfo * l, int sx, int sy)
 
     /* compute average of pixels inside brush */
     r = g = b = 0;
-    brushbits = (unsigned char *) currentBrush->brushbits;
+    brushbits = (unsigned char *) brush->brushbits;
     for (y = 0; y < l->height; ++y)
 	for (x = 0; x < l->width; ++x)
 	    if (*brushbits++) {
@@ -513,13 +470,13 @@ smear(Widget wid, OpInfo * info, GC gc, LocalInfo * l, int sx, int sy)
 		g += col->green;
 		b += col->blue;
 	    }
-    n = currentBrush->numpixels;
+    n = brush->numpixels;
     r = r / 256 / n;
     g = g / 256 / n;
     b = b / 256 / n;
 
     /* now blend each surface pixel with average */
-    brushbits = (unsigned char *) currentBrush->brushbits;
+    brushbits = (unsigned char *) brush->brushbits;
     for (y = 0; y < l->height; ++y)
 	for (x = 0; x < l->width; ++x) {
 	    if ((m = *brushbits++) != 0) {
@@ -544,6 +501,7 @@ smear(Widget wid, OpInfo * info, GC gc, LocalInfo * l, int sx, int sy)
 static void 
 wbrush(Widget wid, OpInfo * info, GC gc, LocalInfo * l, int sx, int sy)
 {
+    BrushItem * brush = &brushList[currentBrush];
     int x, y, dx, dy, m, w, h, d;
     unsigned long r, g, b;
     unsigned char *brushbits;
@@ -587,7 +545,7 @@ wbrush(Widget wid, OpInfo * info, GC gc, LocalInfo * l, int sx, int sy)
     r = col->red / 256;
     g = col->green / 256;
     b = col->blue / 256;
-    brushbits = (unsigned char *) currentBrush->brushbits;
+    brushbits = (unsigned char *) brush->brushbits;
     for (y = 0; y < l->height; ++y)
 	for (x = 0; x < l->width; ++x)
 	    if ((m = *brushbits++) != 0) {
@@ -636,8 +594,8 @@ setCursor(Widget wid, void *brushArg)
 	XQueryColors(dpy, map, xcols, XtNumber(xcols));
     }
     if (brush->cursor == None) {
-	Pixmap source, mask;
-	XImage *src, *msk;
+	Pixmap src, mask;
+	XImage *im_src, *im_mask;
 	GC gc;
 	int x, y, w, h, ow, oh, n;
 	unsigned char *brushbits;
@@ -660,22 +618,25 @@ setCursor(Widget wid, void *brushArg)
 	xpmAttr.valuemask = XpmColorSymbols;
 	xpmAttr.numsymbols = 5;
 	xpmAttr.colorsymbols = colorsymbols;
+	/*
 	XpmCreatePixmapFromData(dpy, RootWindowOfScreen(XtScreen(wid)),
-				brush->bits, &source, NULL, &xpmAttr);
-	GetPixmapWHD(dpy, source, NULL, NULL, (int *) &xpmAttr.depth);
-	src = NewXImage(dpy, NULL, xpmAttr.depth, w, h);
-	memset(src->data, 0, src->bytes_per_line * h);
+				brush->bits, &src, NULL, &xpmAttr);
+	*/
+	GetPixmapWHD(dpy, brush->pixmap, NULL, NULL, (int *) &xpmAttr.depth);
+	im_src = NewXImage(dpy, NULL, xpmAttr.depth, w, h);
+	memset(im_src->data, 0, im_src->bytes_per_line * h);
 
 	/* copy colour pixmap to center of XImage */
-	XGetSubImage(dpy, source, 0, 0, ow, oh, AllPlanes, ZPixmap, src, 1, 1);
-	msk = NewXImage(dpy, NULL, 1, w, h);
+	XGetSubImage(dpy, brush->pixmap, 0, 0, ow, oh, 
+                     AllPlanes, ZPixmap, im_src, 1, 1);
+	im_mask = NewXImage(dpy, NULL, 1, w, h);
 	brushbits = (unsigned char *) xmalloc(ow * oh);
 	brush->brushbits = (char *) brushbits;
 	
 	n = 0;
 	for (y = 0; y < h; y++)
 	    for (x = 0; x < w; x++) {
-		Pixel p = XGetPixel(src, x, y);
+		Pixel p = XGetPixel(im_src, x, y);
 
 		if ((y != 0) && (y != h - 1) && (x != 0) && (x != w - 1))
 		    *brushbits++ = p;
@@ -684,44 +645,42 @@ setCursor(Widget wid, void *brushArg)
 		    ++n;
 
 		if (!p && x > 0)
-		    p = XGetPixel(src, x - 1, y);
+		    p = XGetPixel(im_src, x - 1, y);
 		if (!p && x < w - 1)
-		    p = XGetPixel(src, x + 1, y);
+		    p = XGetPixel(im_src, x + 1, y);
 		if (!p && y > 0)
-		    p = XGetPixel(src, x, y - 1);
+		    p = XGetPixel(im_src, x, y - 1);
 		if (!p && y < h - 1)
-		    p = XGetPixel(src, x, y + 1);
-		XPutPixel(msk, x, y, p ? 1 : 0);
+		    p = XGetPixel(im_src, x, y + 1);
+		XPutPixel(im_mask, x, y, p ? 1 : 0);
 	    }
-	XFreePixmap(dpy, source);
-	XDestroyImage(src);
+	XDestroyImage(im_src);
 	brush->numpixels = n;
 
-	source = XCreatePixmap(dpy, brush->pixmap, w, h, 1);
+	src = XCreatePixmap(dpy, brush->pixmap, w, h, 1);
 	mask = XCreatePixmap(dpy, brush->pixmap, w, h, 1);
 	gc = XCreateGC(dpy, mask, 0, 0);
 	XSetForeground(dpy, gc, 0);
-	XFillRectangle(dpy, source, gc, 0, 0, w, h);
-	XCopyArea(dpy, brush->pixmap, source, gc, 0, 0, ow, oh, 1, 1);
-	XPutImage(dpy, mask, gc, msk, 0, 0, 0, 0, w, h);
-	XDestroyImage(msk);
+	XFillRectangle(dpy, src, gc, 0, 0, w, h);
+	XCopyArea(dpy, brush->pixmap, src, gc, 0, 0, ow, oh, 1, 1);
+	XPutImage(dpy, mask, gc, im_mask, 0, 0, 0, 0, w, h);
+	XDestroyImage(im_mask);
 	XFreeGC(dpy, gc);
-
-	brush->cursor = XCreatePixmapCursor(dpy, source, mask,
+	brush->cursor = XCreatePixmapCursor(dpy, src, mask,
 					    &xcols[1], &xcols[0],
 					    w / 2, h / 2);
-	XFreePixmap(dpy, source);
+	XFreePixmap(dpy, src);
 	XFreePixmap(dpy, mask);
     }
     FatCursorSet(wid, brush->pixmap);
 
     /* don't set cursor to brush shape if zoom is larger than 1 */
-    if (paint->paint.zoom <= 1)
-	XtVaSetValues(wid, XtNcursor, brush->cursor, NULL);
-    else {
+    if (paint->paint.zoom == 1) {
+        XtVaSetValues(wid, XtNcursor, brush->cursor, NULL);
+    } else {
 	SetCrossHairCursor(wid);
-	FatCursorAddZoom(paint->paint.zoom, wid);
-	XtAddCallback(wid, XtNdestroyCallback, FatCursorDestroyCallback, wid);
+	FatCursorAddZoom((paint->paint.zoom>0)?paint->paint.zoom:2, wid);
+        XtAddCallback(wid, XtNdestroyCallback, FatCursorDestroyCallback, wid);
     }
 }
 
@@ -758,13 +717,14 @@ BrushAdd(Widget w)
 {
     static LocalInfo *l;
     Colormap cmap;
+    BrushItem * brush = &brushList[currentBrush];
 
     l = XtNew(LocalInfo);
 
     XtVaSetValues(w, XtNcompress, False, NULL);
 
     l->brushtype = PLAIN;
-    l->pixmap = currentBrush->pixmap;
+    l->pixmap = brush->pixmap;
     XtVaGetValues(w, XtNcolormap, &cmap, NULL);
     l->brushPalette = PaletteFind(w, cmap);
     l->useSecond = -1;
@@ -777,7 +737,7 @@ BrushAdd(Widget w)
     OpAddEventHandler(w, opWindow | opPixmap, ButtonReleaseMask,
 		      FALSE, (OpEventProc) release, (XtPointer) l);
 
-    setCursor(w, (void *) currentBrush);
+    setCursor(w, (void *) brush);
 
     return l;
 }
@@ -800,12 +760,13 @@ void *
 EraseAdd(Widget w)
 {
     LocalInfo *l = XtNew(LocalInfo);
+    BrushItem * brush = &brushList[currentBrush];
 
     XtVaSetValues(w, XtNcompress, False, NULL);
 
     l->brushtype = ERASE;
     l->useSecond = -1;
-    l->pixmap = currentBrush->pixmap;
+    l->pixmap = brush->pixmap;
     l->tracking = False;
 
     OpAddEventHandler(w, opWindow | opPixmap, ButtonPressMask,
@@ -815,7 +776,7 @@ EraseAdd(Widget w)
     OpAddEventHandler(w, opWindow | opPixmap, ButtonReleaseMask,
 		      FALSE, (OpEventProc) release, (XtPointer) l);
 
-    setCursor(w, (void *) currentBrush);
+    setCursor(w, (void *) brush);
 
     return l;
 }
@@ -839,12 +800,12 @@ SmearAdd(Widget w)
 {
     LocalInfo *l = XtNew(LocalInfo);
     Colormap cmap;
-
+    BrushItem * brush = &brushList[currentBrush];
 
     XtVaSetValues(w, XtNcompress, False, NULL);
 
     l->brushtype = SMEAR;
-    l->pixmap = currentBrush->pixmap;
+    l->pixmap = brush->pixmap;
     l->useSecond = -1;
     l->tracking = False;
 
@@ -858,7 +819,7 @@ SmearAdd(Widget w)
     OpAddEventHandler(w, opWindow | opPixmap, ButtonReleaseMask,
 		      FALSE, (OpEventProc) release, (XtPointer) l);
 
-    setCursor(w, (void *) currentBrush);
+    setCursor(w, (void *) brush);
 
     return l;
 }
@@ -877,61 +838,72 @@ SmearRemove(Widget w, void *l)
     FatCursorOff(w);
 }
 
-/*
-**  Initializer to create a default brush
- */
-void
-BrushInit(Widget toplevel)
-{
-    XpmAttributes xpmAttr;
-
-    currentBrush = &baseBrushList[0];
-
-    /* force depth of one */
-    xpmAttr.depth = 1;
-    xpmAttr.colorsymbols = monoColorSymbols;
-    xpmAttr.numsymbols = 5;
-    xpmAttr.valuemask = XpmDepth | XpmColorSymbols;
-
-    XpmCreatePixmapFromData(XtDisplay(toplevel),
-			    RootWindowOfScreen(XtScreen(toplevel)),
-			    currentBrush->bits, &currentBrush->pixmap,
-			    NULL, &xpmAttr);
-    currentBrush->width = xpmAttr.width;
-    currentBrush->height = xpmAttr.height;
-
-    brushImage = NewXImage(XtDisplay(toplevel), NULL,
-			   DefaultDepthOfScreen(XtScreen(toplevel)),
-			   currentBrush->width, currentBrush->height);
-}
-
 /*
 **  The brush selection dialog
  */
 
 static void
-closePopup(Widget button, Widget shell)
+closePopup(Widget button, BrushboxInfo * info)
 {
-    XtPopdown(shell);
+    XtPopdown(info->shell);
 }
 
 void
-selectBrush(Widget shell, BrushItem * nc)
+setStandardCursor(Widget w)
 {
-    currentBrush = nc;
+    BrushItem * brush;
+    int zoom;
 
     if ((CurrentOp->add == BrushAdd) ||
 	(CurrentOp->add == EraseAdd) ||
 	(CurrentOp->add == SmearAdd)) {
-	GraphicAll(setCursor, (void *) currentBrush);
-	GraphicAll(setPixmap, (void *) currentBrush);
+        brush =  &brushList[currentBrush];
+        setCursor(w, (void *) brush);
+        XtVaGetValues(w, XtNzoom, &zoom, NULL);
+        if (zoom == 1)
+            FatCursorRemoveZoom(w);
+        else {
+            SetCrossHairCursor(w);
+            FatCursorAddZoom((zoom>0)?zoom:2, w);
+        }
+    } else
+    if ((CurrentOp->add == PencilAdd) ||
+        (CurrentOp->add == DotPencilAdd))
+	SetPencilCursor(w);
+    else
+    if (CurrentOp->add == SprayAdd)
+        XtVaSetValues(w, XtNcompress, False,
+                      XtVaTypedArg, XtNcursor, XtRString, "spraycan", 
+                      sizeof(Cursor), NULL);
+    else
+	SetCrossHairCursor(w);
+}
+
+void
+selectBrush(Widget w, XtPointer nc)
+{
+    BrushItem * brush;
+    int i;
+
+    currentBrush = (int)((long)nc);
+    brush =  &brushList[currentBrush];
+
+    for (i=0; i<brushnum; i++)
+        XtVaSetValues(brushList[i].icon, XtNstate, (i==currentBrush), NULL);
+
+    if ((CurrentOp->add == BrushAdd) ||
+	(CurrentOp->add == EraseAdd) ||
+	(CurrentOp->add == SmearAdd)) {
+	GraphicAll(setCursor, (void *) brush);
+	GraphicAll(setPixmap, (void *) brush);
     }
-    GraphicAll(setBrushIconPixmap, (void *) currentBrush);
+    GraphicAll(setBrushIconPixmap, (void *) brush);
+
     if (brushImage != NULL)
 	XDestroyImage(brushImage);
-    brushImage = NewXImage(XtDisplay(shell), NULL,
-			   DefaultDepthOfScreen(XtScreen(shell)),
-			   currentBrush->width, currentBrush->height);
+    brushImage = NewXImage(XtDisplay(brush->icon), NULL,
+			   DefaultDepthOfScreen(XtScreen(brush->icon)),
+			   brush->width, brush->height);
 }
 
 static void
@@ -954,37 +926,200 @@ brushboxResized(Widget w, BrushboxInfo * l, XConfigureEvent * event, Boolean * f
     XtMoveWidget(l->close, 4, height-25);
 }
 
+static void
+ReadSystemBrushes(Widget w)
+{
+    Display *dpy = XtDisplay(w);
+    Pixmap pix;
+    char brushcfg[256], brushfile[256], buf[256];
+    FILE *fd;
+    char *ptr;
+    XpmAttributes xpmAttr;
+    int i;
 
-static Widget
+    sprintf(brushcfg, "%s/bitmaps/brushbox.cfg", SHAREDIR);
+    fd = fopen(brushcfg, "r");
+    if (!fd) {
+        fprintf(stderr, "\
+Brush configuration file\n  %s\ncouldn't be read !! \
+Will use square 9x9 as the brush ...\n", brushcfg);
+        return;
+    }
+
+    /* First line of brushbox.cfg is default brush */
+    *buf = '\0';
+    fgets(buf, 255, fd);
+    ptr = index(buf, ' ');
+    if (!ptr) ptr = index(buf, '\t');
+    if (ptr) *ptr = '\0';
+    brushdef = atoi(buf);
+    while(fgets(buf, 255, fd)) {
+	    ptr = index(buf, '\n');
+        if (ptr) *ptr = '\0';  
+        if (!strstr(buf, ".xpm")) continue;
+        sprintf(brushfile, "%s/bitmaps/%s", SHAREDIR, buf);
+	/* force depth of one */
+	xpmAttr.depth = 1;
+	xpmAttr.colorsymbols = monoColorSymbols;
+	xpmAttr.numsymbols = 5;
+	xpmAttr.valuemask = XpmDepth | XpmColorSymbols;
+	if (XpmReadFileToPixmap(dpy, RootWindowOfScreen(XtScreen(w)),
+	    brushfile, &pix, NULL, &xpmAttr)==XpmSuccess) {
+            i = BBNUM;
+            ++BBNUM;
+            brushList = (BrushItem *) 
+	            realloc(brushList, BBNUM*sizeof(BrushItem));
+	    brushList[i].width = xpmAttr.width;
+	    brushList[i].height = xpmAttr.height;
+            brushList[i].pixmap = pix;
+            brushList[i].brushbits = NULL;
+            brushList[i].bw = None;
+            brushList[i].cursor = None;
+            brushList[i].icon = None;
+	} else
+	    fprintf(stderr, "Brush file %s couldn't be read !!\n", brushfile);
+    }
+    brushnum = BBNUM;
+}
+
+Widget
 createBrushDialog(Widget w)
 {
-    Widget shell, icon, firstIcon = 0;
+    Display *dpy = XtDisplay(w);
+    Widget icon;
+    static Widget firstIcon = None;
     static BrushboxInfo *info = NULL;
+    RCInfo *rcInfo = NULL;
     GC gc;
     XGCValues values;
-    RCInfo *rcInfo;
-    static BrushItem * brushList = NULL;
-    static int brushnum = BBNUMBER;
-    int i, j;
+    int i, j, newbrushnum;
     Pixel fg, bg;
-    Pixmap pix;
     int nw, nh, ox, oy;
-    XpmAttributes xpmAttr;
     Arg args[4];
     int nargs = 0;
     
-    if (!info) info = (BrushboxInfo *)XtMalloc(sizeof(BrushboxInfo));
-    shell = XtVisCreatePopupShell("brush",
-				 topLevelShellWidgetClass, w,
+    if (!info) {
+        info = (BrushboxInfo *)XtMalloc(sizeof(BrushboxInfo));
+        info->shell = XtVisCreatePopupShell("brush",
+	 			 topLevelShellWidgetClass, w,
 				 args, nargs);
 
-    info->form = XtVaCreateManagedWidget(NULL,
-				   formWidgetClass, shell,
+        info->form = XtVaCreateManagedWidget(NULL,
+				   formWidgetClass, info->shell,
 				   NULL);
 
-    info->box = XtVaCreateManagedWidget("box",
+        info->box = XtVaCreateManagedWidget("box",
 				  boxWidgetClass, info->form,
 				  NULL);
+        AddDestroyCallback(info->shell, (DestroyCallbackFunc) closePopup, info);
+        XtAddEventHandler(info->shell, StructureNotifyMask, False,
+		          (XtEventHandler) brushboxResized, (XtPointer) info);
+        info->close = XtVaCreateManagedWidget("close",
+				    commandWidgetClass, info->form,
+				    XtNfromVert, info->box,
+				    XtNtop, XtChainBottom,
+				    NULL);
+
+        XtAddCallback(info->close, XtNcallback, (XtCallbackProc) closePopup,
+		      (XtPointer) info);
+    }
+
+    /* Read system wide brushes from SHAREDIR */
+    if (BBNUM == 0)
+        ReadSystemBrushes(w);
+
+    /* If this fails, create a default brush to be a 9x9 square */
+    if (BBNUM == 0) {
+        GC gc1;
+        XGCValues values1;
+        int d = 9;
+        brushnum = ++BBNUM;
+        brushdef = 0;
+        brushList = (BrushItem *) 
+	            realloc(brushList, BBNUM*sizeof(BrushItem));
+	brushList[0].width = d;
+	brushList[0].height = d;
+        brushList[0].brushbits = NULL;
+        brushList[0].bw = None;
+        brushList[0].cursor = None;
+        brushList[0].icon = None;
+        brushList[0].pixmap =
+            XCreatePixmap(dpy, RootWindowOfScreen(XtScreen(w)), d, d, 1);
+        values1.foreground = values.background;
+        values1.background = values.foreground;
+  	gc1 = XCreateGC(dpy, brushList[0].pixmap,
+                        GCForeground | GCBackground, &values1);
+        XFillRectangle(dpy, brushList[0].pixmap, gc1, 0, 0, d+1, d+1);
+	XFreeGC(dpy, gc1);
+    }
+
+    /* 
+     *  Clear everything that has been set beyond the system brushes, 
+     *  and read again.
+     */
+    for (i = BBNUM; i < brushnum; i++) {
+        if (brushList[i].pixmap != None) XFreePixmap(dpy, brushList[i].pixmap);
+        if (brushList[i].bw != None) XFreePixmap(dpy, brushList[i].bw);
+        if (brushList[i].icon != None) XtUnrealizeWidget(brushList[i].icon);
+	bzero(&brushList[i], sizeof(BrushItem));
+    }
+
+    /* Read RC file for possible brushes there */
+    rcInfo = ReadDefaultRC();
+
+    newbrushnum = BBNUM+rcInfo->nbrushes+Global.nbrushes;
+
+    if (brushnum<newbrushnum) {
+        brushList = (BrushItem *) 
+	    realloc(brushList, newbrushnum*sizeof(BrushItem));
+	for (i=brushnum; i<newbrushnum; i++)
+	    bzero(&brushList[i], sizeof(BrushItem));
+        brushnum = newbrushnum;
+    }
+
+    for (i = BBNUM; i < brushnum; i++) {
+	GC gc0, gc1;
+        XGCValues values1;
+	int u, v, k, scale;
+	Image * brush;
+	char *ptr;
+
+	j = i - BBNUM;
+	if (j < rcInfo->nbrushes) 
+	    brush = rcInfo->brushes[j];
+	else
+	    brush = (Image *) (Global.brushes[j-rcInfo->nbrushes]);
+	brushList[i].width = brush->width;
+        brushList[i].height = brush->height;
+	brushList[i].pixmap = XCreatePixmap(dpy,
+	                          RootWindowOfScreen(XtScreen(w)),
+	                          brushList[i].width, brushList[i].height, 1);
+	scale = brush->scale;
+	ptr = (char*) brush->data;
+
+        values.background = WhitePixelOfScreen(XtScreen(w));
+        values.foreground = BlackPixelOfScreen(XtScreen(w));
+	gc0 = XCreateGC(dpy, brushList[i].pixmap, 
+                        GCForeground | GCBackground, &values);
+        values1.foreground = values.background;
+        values1.background = values.foreground;
+	gc1 = XCreateGC(dpy, brushList[i].pixmap, 
+                        GCForeground | GCBackground, &values1);
+	for (v = 0; v < brushList[i].height; v++)
+	    for (u = 0; u < brushList[i].width; u++) {
+	        k = 0;
+		while (k<scale && !ptr[k]) ++k;
+		ptr = ptr+scale;
+		if (k==scale)
+                    XDrawPoint(dpy, brushList[i].pixmap, gc1, u, v);
+		else
+                    XDrawPoint(dpy, brushList[i].pixmap, gc0, u, v);
+	    }
+	XFreeGC(dpy, gc0);
+	XFreeGC(dpy, gc1);
+    }
+
+    /* Now create icons and stuff */
 
     values.foreground = WhitePixelOfScreen(XtScreen(w));
     values.background = BlackPixelOfScreen(XtScreen(w));
@@ -992,163 +1127,94 @@ createBrushDialog(Widget w)
     gc = XCreateGC(XtDisplay(w),
 		   RootWindowOfScreen(XtScreen(w)),
 		   GCForeground | GCBackground, &values);
-
-    values.background = WhitePixelOfScreen(XtScreen(w));
-    values.foreground = BlackPixelOfScreen(XtScreen(w));
-
-    rcInfo = ReadDefaultRC();
-
-    if (rcInfo->nbrushes || Global.nbrushes) {
-	/* There are brushes specified from the RC file, */
-        /* or the user defined brushes with the Pattern Editor */
-        brushList = (BrushItem *) 
-	    realloc(brushList, 
-               (BBNUMBER+rcInfo->nbrushes+Global.nbrushes)*sizeof(BrushItem));
-	for (i = 0; i < BBNUMBER; i++)
-	     brushList[i] = baseBrushList[i];
-	for (i=brushnum; i<BBNUMBER+rcInfo->nbrushes+Global.nbrushes; i++)
-	    bzero(&brushList[i++], sizeof(BrushItem));
-	brushnum = BBNUMBER+rcInfo->nbrushes+Global.nbrushes;
-    }
-
-    if (!brushList)
-        brushList = baseBrushList;
-
-    for (i = 0; i < BBNUMBER+rcInfo->nbrushes+Global.nbrushes; i++) {
-        if (i >= BBNUMBER && brushList[i].pixmap == None) {
-	    /* Create a brush specified from RC file */
-	    Display *dpy = XtDisplay(w);
-	    int u, v, k, scale;
-	    GC gc0, gc1;
-	    Image * brush;
-	    char *ptr;
-            XGCValues values1;
-            values1.foreground = values.background;
-            values1.background = values.foreground;
-
-	    j = i - BBNUMBER;
-	    if (j < rcInfo->nbrushes) 
-	        brush = rcInfo->brushes[j];
-	    else
-	        brush = (Image *) (Global.brushes[j-rcInfo->nbrushes]);
-	    brushList[i].width = brush->width;
-            brushList[i].height = brush->height;
-	    brushList[i].pixmap = XCreatePixmap(dpy,
-		RootWindowOfScreen(XtScreen(info->box)),
-	        brushList[i].width, brushList[i].height, 1);
-	    scale = brush->scale;
-	    ptr = (char*) brush->data;
-	    gc0 = XCreateGC(dpy, brushList[i].pixmap, 
-                            GCForeground | GCBackground, &values);
-	    gc1 = XCreateGC(dpy, brushList[i].pixmap, 
-                            GCForeground | GCBackground, &values1);
-	    for (v = 0; v < brushList[i].height; v++)
-	        for (u = 0; u < brushList[i].width; u++) {
-		    k = 0;
-		    while (k<scale && !ptr[k]) ++k;
-		    ptr = ptr+scale;
-		    if (k==scale)
-                        XDrawPoint(dpy, brushList[i].pixmap, gc1, u, v);
-		    else
-                        XDrawPoint(dpy, brushList[i].pixmap, gc0, u, v);
-		}
-	    XFreeGC(dpy, gc0);
-	    XFreeGC(dpy, gc1);
-	    /* force depth of one */
-	    xpmAttr.depth = 1;
-	    xpmAttr.valuemask = XpmDepth;	    
-            XpmCreateDataFromPixmap(dpy, &brushList[i].bits, 
-				    brushList[i].pixmap, None, &xpmAttr);
-	}
-	if (brushList[i].pixmap == None) {
-	    /* force depth of one */
-	    xpmAttr.depth = 1;
-	    xpmAttr.colorsymbols = monoColorSymbols;
-	    xpmAttr.numsymbols = 5;
-	    xpmAttr.valuemask = XpmDepth | XpmColorSymbols;
-	    if (XpmCreatePixmapFromData(XtDisplay(info->box),
-				    RootWindowOfScreen(XtScreen(info->box)),
-				 brushList[i].bits, &brushList[i].pixmap,
-				    NULL, &xpmAttr)) continue;
-	    brushList[i].width = xpmAttr.width;
-	    brushList[i].height = xpmAttr.height;
-	}
-	icon = XtVaCreateManagedWidget("icon",
+    for (i=0; i<brushnum; i++) {
+        if (brushList[i].icon == None) {
+            icon = XtVaCreateManagedWidget("icon",
 				       toggleWidgetClass, info->box,
 				       XtNradioGroup, firstIcon,
 #ifdef XAW3D
 				       XtNforeground, 0xc0c0c0,
 #endif				       
 				       NULL);
-	nw = brushList[i].width;
-	nh = brushList[i].height;
-	ox = oy = 0;
-	if (nw < 16) {
-	    ox = (16 - nw) / 2;
-	    nw = 16;
-	}
-	if (nh < 16) {
-	    oy = (16 - nh) / 2;
-	    nh = 16;
-	}
-	pix = XCreatePixmap(XtDisplay(info->box),
-			    RootWindowOfScreen(XtScreen(info->box)),
-			    nw, nh,
-			    DefaultDepthOfScreen(XtScreen(info->box)));
+	} else
+ 	    icon = brushList[i].icon;
+	if (firstIcon == None) firstIcon = icon;
 
-	XtVaGetValues(icon, XtNbackground, &bg, NULL);
-        fg = XtScreen(w)->black_pixel;
-	/*
-	**  Clear then draw the clipped rectangle in
-	 */
-	XSetClipMask(XtDisplay(w), gc, None);
-	XSetForeground(XtDisplay(w), gc, bg);
-	XFillRectangle(XtDisplay(w), pix, gc, 0, 0, nw, nh);
-	XSetClipMask(XtDisplay(w), gc, brushList[i].pixmap);
-	XSetClipOrigin(XtDisplay(w), gc, ox, oy);
-	XSetForeground(XtDisplay(w), gc, fg);
-	XFillRectangle(XtDisplay(w), pix, gc, 0, 0, nw, nh);
+        if (brushList[i].bw == None) {
+	    nw = brushList[i].width;
+	    nh = brushList[i].height;
+	    ox = oy = 0;
+	    if (nw < 16) {
+	        ox = (16 - nw) / 2;
+	        nw = 16;
+	    }
+	    if (nh < 16) {
+	        oy = (16 - nh) / 2;
+	        nh = 16;
+	    }
+	    brushList[i].bw = XCreatePixmap(dpy, 
+                                  RootWindowOfScreen(XtScreen(w)),
+	                          nw, nh,
+		                  DefaultDepthOfScreen(XtScreen(w)));
+  	    XtVaGetValues(icon, XtNbackground, &bg, NULL);
+            fg = XtScreen(w)->black_pixel;
+	    /*
+	    **  Clear then draw the clipped rectangle in
+	     */
+	    XSetClipMask(dpy, gc, None);
+	    XSetForeground(dpy, gc, bg);
+	    XFillRectangle(dpy, brushList[i].bw, gc, 0, 0, nw, nh);
+	    XSetClipMask(dpy, gc, brushList[i].pixmap);
+	    XSetClipOrigin(dpy, gc, ox, oy);
+	    XSetForeground(dpy, gc, fg);
+	    XFillRectangle(dpy, brushList[i].bw, gc, 0, 0, nw, nh);
+	}
 
-	XtVaSetValues(icon, XtNbitmap, pix, NULL);
+	XtVaSetValues(icon, XtNbitmap, brushList[i].bw, NULL);
 
-	if (firstIcon == NULL) {
-	    XtVaSetValues(icon, XtNstate, True, NULL);
-	    firstIcon = icon;
+        if (brushList[i].icon == None) {
+	    XtAddCallback(icon, XtNcallback,
+		          (XtCallbackProc) selectBrush, (XtPointer)(long)i);
+            brushList[i].icon = icon;
 	}
-	XtAddCallback(icon, XtNcallback,
-	       (XtCallbackProc) selectBrush, (XtPointer) & brushList[i]);
     }
-
-    info->close = XtVaCreateManagedWidget("close",
-				    commandWidgetClass, info->form,
-				    XtNfromVert, info->box,
-				    XtNtop, XtChainBottom,
-				    NULL);
-
-    XtAddCallback(info->close, XtNcallback, (XtCallbackProc) closePopup,
-		  (XtPointer) shell);
-
+    XtVaSetValues(brushList[brushdef].icon, XtNstate, True, NULL);
     XFreeGC(XtDisplay(w), gc);
 
-    AddDestroyCallback(shell, (DestroyCallbackFunc) closePopup, shell);
-    XtAddEventHandler(shell, StructureNotifyMask, False,
-		      (XtEventHandler) brushboxResized, (XtPointer) info);
+    return info->shell;
+}
 
-    return shell;
+int
+GetTotalNumBrushes()
+{
+    return brushnum;
+}
+
+/*
+**  Initializer to create a default brush
+ */
+void
+BrushInit(Widget toplevel)
+{
+    BrushItem * brush;
+    Global.brushpopup = createBrushDialog(toplevel);
+    selectBrush(toplevel, (void *)(long)brushdef);
+    brush = &brushList[brushdef];
+    brushImage = NewXImage(XtDisplay(toplevel), NULL,
+			   DefaultDepthOfScreen(XtScreen(toplevel)),
+			   brush->width, brush->height);
 }
 
 void 
 BrushSelect(Widget w)
 {
-    if (Global.brushpopup == NULL)
-	Global.brushpopup = createBrushDialog(GetToplevel(w));
-
     XtPopup(Global.brushpopup, XtGrabNone);
     XMapRaised(XtDisplay(Global.brushpopup), XtWindow(Global.brushpopup));
 }
 
 void setBrushIconOnWidget(Widget w)
 {
+    BrushItem * brush = &brushList[currentBrush];
     if (w != None)
-        XtVaSetValues(w, XtNbitmap, currentBrush->pixmap, NULL);
+        XtVaSetValues(w, XtNbitmap, brush->pixmap, NULL);
 }
