@@ -12,7 +12,12 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: fontOp.c,v 1.3 1996/04/19 08:56:26 torsten Exp $ */
+/* $Id: fontOp.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
+
+#ifdef __VMS
+#define XtDisplay XTDISPLAY
+#define XtWindow XTWINDOW
+#endif
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -98,6 +103,13 @@ motion(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 static void 
 press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 {
+    if (event->type == ButtonRelease) {
+        if (Global.popped_up)
+            PopdownMenusGlobal();
+        event->type = None;
+        return;
+    }
+
     if (event->button == Button3) return;
 
     if (event->button == Button1) {
@@ -115,7 +127,10 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 	l->typing = True;
 
 	if (info->surface == opWindow) {
-	    l->insertH = l->height * info->zoom;
+	    if (info->zoom>0)
+	        l->insertH = l->height * info->zoom;
+            else
+	        l->insertH = -l->height / info->zoom;
 	    l->insertY = event->y - l->insertH / 2;
 	    l->insertX = event->x;
 	    l->insertSX = event->x;
@@ -176,7 +191,10 @@ addString(Widget w, LocalInfo * l, int len, char *buf)
 	    l->curX = l->startX;
 	    l->curY += l->height + 2;
 	    l->insertX = l->insertSX;
-	    l->insertY += l->insertH + 2 * l->zoom;
+            if (l->zoom>0)
+	        l->insertY += l->insertH + 2 * l->zoom;
+            else
+	      l->insertY += l->insertH + 2 / (-l->zoom);
 
 	    *l->sptr++ = '\n';
 	} else if (buf[i] == 0x08 || buf[i] == 0x7f) {
@@ -192,13 +210,20 @@ addString(Widget w, LocalInfo * l, int len, char *buf)
 		    width = XTextWidth(l->fi, cp, strlen(cp));
 		    l->curX = l->startX + width;
 		    l->curY -= l->height + 2;
-		    l->insertX = (l->startX + width) * l->zoom;
-		    l->insertY -= l->insertH + 2 * l->zoom;
+                    if (l->zoom>0) {
+		        l->insertX = (l->startX + width) * l->zoom;
+		        l->insertY -= l->insertH + 2 * l->zoom;
+		    } else {
+		        l->insertX = -(l->startX + width) / l->zoom;
+		        l->insertY -= l->insertH + 2 / (-l->zoom);
+		    }
 		} else {
 		    width = XTextWidth(l->fi, l->sptr, 1);
 		    l->curX -= width;
-		    l->insertX -= width * l->zoom;
-
+                    if (l->zoom>0)
+		        l->insertX -= width * l->zoom;
+                    else
+		        l->insertX -= width / (-l->zoom);
 		    /*
 		    **  Now undraw the character.
 		     */
@@ -218,7 +243,10 @@ addString(Widget w, LocalInfo * l, int len, char *buf)
 	    PwUpdate(w, &rect, False);
 
 	    l->curX += width;
-	    l->insertX += width * l->zoom;
+            if (l->zoom>0)
+	        l->insertX += width * l->zoom;
+            else
+	        l->insertX += width / (-l->zoom);
 
 	    UndoGrow(w, l->curX, l->curY + l->height);
 	    UndoGrow(w, l->curX, l->curY + l->height);
@@ -250,7 +278,8 @@ FontAdd(Widget w)
     l->str = (char *) XtMalloc(l->maxStrLen);
     l->gcx = GetGCX(w);
 
-    OpAddEventHandler(w, opPixmap | opWindow, ButtonPressMask, FALSE,
+    OpAddEventHandler(w, opPixmap | opWindow, 
+                      ButtonPressMask|ButtonReleaseMask, FALSE,
 		      (OpEventProc) press, l);
     OpAddEventHandler(w, opWindow, PointerMotionMask, FALSE,
 		      (OpEventProc) motion, l);
@@ -265,7 +294,8 @@ FontRemove(Widget w, void *p)
 {
     LocalInfo *l = (LocalInfo *) p;
 
-    OpRemoveEventHandler(w, opPixmap | opWindow, ButtonPressMask, FALSE,
+    OpRemoveEventHandler(w, opPixmap | opWindow, 
+                         ButtonPressMask|ButtonReleaseMask, FALSE,
 			 (OpEventProc) press, p);
     OpRemoveEventHandler(w, opWindow, PointerMotionMask, FALSE,
 		      (OpEventProc) motion, p);
@@ -292,6 +322,7 @@ FontChanged(Widget w)
     l = (LocalInfo *) GraphicGetData(w);
 
     XtVaGetValues(w, XtNfont, &l->fi, NULL);
+   
     if (l->fi == NULL)
 	return;
 
@@ -299,5 +330,8 @@ FontChanged(Widget w)
     l->height = l->fi->ascent + l->fi->descent;
 
     l->minPos = l->sptr - l->str;
-    l->insertH = l->height * l->zoom;
+    if (l->zoom>0)
+        l->insertH = l->height * l->zoom;
+    else
+        l->insertH = l->height / (-l->zoom);
 }
