@@ -758,9 +758,12 @@ FontSelect(Widget w, Widget paint)
     Widget applyButton, doneButton, label;
     arg_t *arg = XtNew(arg_t);
 
+    PopdownMenusGlobal();
+
     if (shell != None) {
 	XtPopup(shell, XtGrabNone);
 	XMapRaised(XtDisplay(shell), XtWindow(shell));
+        XtSetMinSizeHints(shell, 360, 450);
 	return;
     }
     StateSetBusyWatch(True);
@@ -1033,6 +1036,7 @@ FontSelect(Widget w, Widget paint)
     AddDestroyCallback(arg->shell, (DestroyCallbackFunc) closeCallback, arg);
 
     XtPopup(arg->shell, XtGrabNone);
+    XtSetMinSizeHints(arg->shell, 360, 450);
     StateSetBusyWatch(False);
 
     XtUnmanageChild(arg->spacinglabel);
