@@ -55,15 +55,17 @@ typedef struct {
 
 #define BUFSIZE 65536
 
-static void *dl_filter = NULL;
+void *dl_filter = NULL;
+void *dl_proc = NULL;
+void *dl_image = NULL;
 
 /*
 **  Some real image processing
  */
 
 extern struct imageprocessinfo ImgProcessInfo;
-extern FILE * openTemp(char **np);
-extern void removeTemp(void);
+extern FILE * openTempFile(char **np);
+extern void removeTempFile(void);
 
 #define CLAMP(low, value, high) \
 		if (value < low) value = low; else if (value > high) value = high
@@ -102,9 +104,11 @@ static PaintMenuItem predefMenu[] =
     MI_SIMPLE("layers"),
 #define PREDEF_PROCEDURES 5
     MI_SIMPLE("procedures"),
-#define PREDEF_SEPARATOR 6
+#define PREDEF_BATCH 6
+    MI_SIMPLE("batch"),
+#define PREDEF_SEPARATOR 7
     MI_SEPARATOR(),
-#define PREDEF_HELP 7
+#define PREDEF_HELP 8
     MI_SIMPLECB("help", HelpDialog, "script_editor"),
 };
 
@@ -270,7 +274,7 @@ ImageSmooth(Image * input)
      * Build n x n convolution matrix (all 1's)
      */
     n = ImgProcessInfo.smoothMaskSize;
-    mat = malloc(n * n * sizeof(float));
+    mat = xmalloc(n * n * sizeof(float));
     for (i = 0; i < n * n; ++i)
 	mat[i] = 1.0;
 
@@ -1220,7 +1224,7 @@ diffuse(void)
 		color_cube[CUBEINDEX(r, g, b) + db] +=
 		    color_cube[CUBEINDEX(r, g, b)];
 	    else
-		fprintf(stderr, "lost error term\n");
+		fprintf(stderr, "%s\n", msgText[LOST_ERROR_TERM]);
 	}
     }
     color_cube[CUBEINDEX(r, g, b)] = -1;
@@ -1297,7 +1301,7 @@ ImageQuantize(Image * input)
 	}
 
     /* Initialize logarithmic weighting table */
-    weight_convert = malloc(MAXWEIGHT * sizeof(int));
+    weight_convert = xmalloc(MAXWEIGHT * sizeof(int));
     weight_convert[0] = 0;
     for (i = 1; i < MAXWEIGHT; ++i) {
 	weight_convert[i] = (int) (100.0 * log((double) i));
@@ -1449,7 +1453,7 @@ ImageGrey(Image * input)
     for (y = 0; y < input->height; y++) {
 	for (x = 0; x < input->width; x++) {
 	    p = ImagePixel(input, x, y);
-	    v = (p[0] * 11 + p[1] * 16 + p[2] * 5) >> 5;	/* .33R + .5G + .17B */
+	    v = (32*p[0]+50*p[1]+18*p[2])/100;	/* 0.32R + 0.5G + 0.18B */
 	    *op++ = v;
 	    *op++ = v;
 	    *op++ = v;
@@ -1608,7 +1612,7 @@ static void append_status(char *string)
 char *ptr, *catptr;
 
     XtVaGetValues(head->cclog, XtNstring, &ptr, NULL);
-    catptr = malloc(strlen(ptr)+strlen(string)+10);
+    catptr = xmalloc(strlen(ptr)+strlen(string)+10);
     strcpy(catptr, ptr);
     strcat(catptr, "\n");
     strcat(catptr, string);
@@ -1665,6 +1669,7 @@ static void
 closeCallback(Widget w, XtPointer wlArg, XtPointer junk)
 {
    LocalInfo *info = (LocalInfo *) wlArg;
+   PopdownMenusGlobal();
    XtDestroyWidget(info->shell);
 }
 
@@ -1685,10 +1690,11 @@ compileCallback(Widget w, XtPointer wlArg, XtPointer junk)
    char header[30];
    char * error;
    void *dl_handle = NULL;
+   unsigned char *alpha = NULL;
    int i;
 
    /* create temporary name */
-   fd= openTemp(&tmp);
+   fd= openTempFile(&tmp);
    fclose(fd);
 
    /* copy buffer to temporary C file */
@@ -1714,7 +1720,7 @@ compileCallback(Widget w, XtPointer wlArg, XtPointer junk)
    fclose(fd);
    header[i] = '\0';
 
-   info->mode = PREDEF_IMAGES;
+   info->mode = -1;
    if (strncasecmp(header, "/* Xpaint-image */", 18) == 0)
       info->mode = PREDEF_IMAGES;
    if (strncasecmp(header, "/* Xpaint-filter */", 19) == 0)
@@ -1722,16 +1728,9 @@ compileCallback(Widget w, XtPointer wlArg, XtPointer junk)
    if (strncasecmp(header, "/* Xpaint-procedure */", 22) == 0)
       info->mode = PREDEF_PROCEDURES;
 
-
-   /* free any previous dll */
-   if (info->mode == PREDEF_FILTERS && dl_filter) {
-      dlclose(dl_filter);
-      dl_filter = NULL;
-   }
-
    /* compile C script */
-   sprintf(cmd, "cd /tmp\ngcc -I%s/include -c %s.c -o %s.o\n"
-                "gcc -shared -Wl,-soname,%s.so %s.o -o %s.so\n",
+   sprintf(cmd, "cd /tmp\ngcc -fpic -I%s/include -c %s.c -o %s.o\n"
+                "gcc -fpic -shared -Wl,-soname,%s.so %s.o -o %s.so\n",
 	        GetShareDir(),
                 tmp, tmp, strstr(tmp, "XPaint"), tmp, tmp);
    XtVaSetValues(info->cclog, XtNstring, cmd, NULL);
@@ -1745,14 +1744,13 @@ compileCallback(Widget w, XtPointer wlArg, XtPointer junk)
 
    /* load dll library */
    sprintf(cmd, "%s.so", tmp);
-   if (info->mode == PREDEF_FILTERS)
-      dl_handle = dl_filter = dlopen(cmd, RTLD_LAZY);
-   else
+
+   if (info->mode >= 0)
       dl_handle = dlopen(cmd, RTLD_LAZY);
 
    /* clean dll .so file */
    unlink(cmd);
-   removeTemp();
+   removeTempFile();
 
    if (dl_handle)
       sprintf(cmd, "echo \"%s\"",
@@ -1760,11 +1758,19 @@ compileCallback(Widget w, XtPointer wlArg, XtPointer junk)
    else
       sprintf(cmd, "echo \"\"; echo \"%s\"; echo \"%s\"",
               msgText[C_SCRIPT_COULD_NOT_BE_COMPILED_OR_LINKED], dlerror());
+
    pipe_command(cmd, NULL, CollectOutput, CollectOutput);
+   if (!dl_handle) return;
+
+   if (info->mode == PREDEF_FILTERS) {
+      if (dl_filter) dlclose(dl_filter);
+      dl_filter = dl_handle;
+   }
 
    if (info->mode == PREDEF_IMAGES) {
-      if (!dl_handle) return;
-      iproc = dlsym(dl_handle, "ImageCreate");
+      if (dl_image) dlclose(dl_image);
+      dl_image = dl_handle;   
+      iproc = dlsym(dl_image, "ImageCreate");
       if ((error = dlerror()) != NULL || !iproc) {
 	 if (!error) error = msgText[UNABLE_TO_CREATE_IMAGE];
          Notice(Global.toplevel, error);
@@ -1772,29 +1778,32 @@ compileCallback(Widget w, XtPointer wlArg, XtPointer junk)
       }
       pix = None;
       image = iproc();
-      dlclose(dl_handle);
-      dl_handle = NULL;
       if (image) {
 	 screen = XtScreen(Global.toplevel);
 	 visual = DefaultVisualOfScreen(screen);
 	 cmap = XCreateColormap(XtDisplay(Global.toplevel), 
 		    RootWindowOfScreen(screen), visual, AllocNone);
-	 ImageToPixmap(image, Global.toplevel, &pix, &cmap);
+         alpha = image->alpha;
+	 ImageToPixmap(image, Global.toplevel, &pix, &cmap,Global.alpha_mode|4);
       }
       if (!pix) {
          Notice(Global.toplevel, msgText[UNABLE_TO_CREATE_IMAGE]);
 	 return;
       }
-      graphicCreate(makeGraphicShell(Global.toplevel), 0, 0, -1, pix, cmap);
+      graphicCreate(makeGraphicShell(Global.toplevel), 0, 0, -1, 
+                    pix, cmap, alpha);
       return;
    }
 
    if (info->mode == PREDEF_PROCEDURES) {
-      if (!dl_handle) return;
-      proc = dlsym(dl_handle, "PaintProcedure");
+      if (dl_proc) dlclose(dl_proc);
+      dl_proc = dl_handle;
+      proc = dlsym(dl_proc, "PaintProcedure");
+      if ((error = dlerror()) != NULL) {
+         Notice(Global.toplevel, error);
+         return;
+      }
       if (proc) proc(Global.toplevel);
-      dlclose(dl_handle);
-      dl_handle = NULL;
       return;
    }
 }
@@ -1810,10 +1819,11 @@ editCallback(Widget w, XtPointer wlArg, XtPointer junk)
    char buf[2048];
 
    /* write buffer to file */
-   fd = openTemp(&tmp);
+   PopdownMenusGlobal();
+   fd = openTempFile(&tmp);
    fclose(fd);
    sprintf(name, "%s.c", tmp);
-   removeTemp();
+   removeTempFile();
    fd = fopen(name, "w");
    if (!fd) {
       Notice(w, msgText[UNABLE_TO_OPEN_FILE], name);
@@ -1840,7 +1850,7 @@ editCallback(Widget w, XtPointer wlArg, XtPointer junk)
       return;
    }
 
-   text = malloc(2);
+   text = xmalloc(2);
    *text = '\0';
    l = 0;
    while (fgets(buf, 2040, fd)) {
@@ -1870,7 +1880,7 @@ loadFileCallbackOK(Widget w, XtPointer wlArg, char *file)
    }
    info->scriptfile = file;
    XtVaSetValues(info->name, XtNlabel, file, NULL);
-   text = malloc(2);
+   text = xmalloc(2);
    *text = '\0';
    l = 0;
    while (fgets(buf, 2040, fd)) {
@@ -1954,7 +1964,7 @@ static void
 loadScriptCallback(Widget w, XtPointer wlArg, XtPointer junk)
 {
    LocalInfo *info = (LocalInfo *) wlArg;
-   char directory[256];
+   static char directory[256];
    int i;
 
    info->mode = PREDEF_IMAGES;
@@ -1967,6 +1977,7 @@ loadScriptCallback(Widget w, XtPointer wlArg, XtPointer junk)
    Global.explore = True;
    sprintf(directory, "%s/c_scripts/%s/", GetShareDir(),
 	              predefMenu[info->mode].name);
+
    GetFileName(GetShell(w), 3, directory, 
       (XtCallbackProc) loadFileCallbackOK, wlArg);
    Global.explore = False;
@@ -2012,7 +2023,17 @@ editorResized(Widget w, LocalInfo * l, XConfigureEvent * event, Boolean * flg)
     XtMoveWidget(l->cclog, 4, 0);
 }
 
-void
+static void
+popupHandler(Widget w, LocalInfo * info, XEvent * event, Boolean * flag)
+{
+    if (Global.popped_up) {
+        if (event->type == ButtonRelease)
+            PopdownMenusGlobal();
+        event->type = None;       
+    }
+}
+
+void *
 ScriptEditor(Widget w, Widget paint)
 {
     Display *dpy;
@@ -2024,6 +2045,8 @@ ScriptEditor(Widget w, Widget paint)
     int nargs = 0;
     int i;
 
+    PopdownMenusGlobal();
+
     head = info;
     info->mode = 0;
     info->scriptfile = NULL;
@@ -2038,23 +2061,14 @@ ScriptEditor(Widget w, Widget paint)
                                    NULL);
     topform = XtVaCreateManagedWidget("topform", formWidgetClass, info->pane, 
                                       NULL);
+
     bar = MenuBarCreate(topform, XtNumber(printMenuBar), printMenuBar);
     XtVaSetValues(bar, XtNvertDistance, 2, NULL);
     
-    info->name = XtVaCreateManagedWidget("name",
-				    labelWidgetClass, topform,
-				    XtNlabel, "",
-				    XtNwidth, 350,
-				    XtNfromHoriz, bar,
-                                    XtNhorizDistance, 15,
-                                    XtNvertDistance, 6,
-				    XtNborderWidth, 0,
-                                    NULL);
-
     compile = XtVaCreateManagedWidget("compile",
 		 		    commandWidgetClass, topform,
 				    XtNfromHoriz, bar,
-				    XtNhorizDistance, 400,
+				    XtNhorizDistance, 32,
 				    XtNvertDistance, 5,
 				    NULL);
     cancel = XtVaCreateManagedWidget("close",
@@ -2064,6 +2078,16 @@ ScriptEditor(Widget w, Widget paint)
 				    XtNvertDistance, 5,
 				    NULL);
 
+    info->name = XtVaCreateManagedWidget("name",
+				    labelWidgetClass, topform,
+				    XtNlabel, "",
+				    XtNwidth, 350,
+				    XtNfromHoriz, cancel,
+                                    XtNhorizDistance, 24,
+                                    XtNvertDistance, 6,
+				    XtNborderWidth, 0,
+                                    NULL);
+
     midform = XtVaCreateManagedWidget("midform", 
                                     formWidgetClass, info->pane, NULL);
     botform = XtVaCreateManagedWidget("botform", 
@@ -2079,6 +2103,11 @@ ScriptEditor(Widget w, Widget paint)
 				  XtNscrollHorizontal, XawtextScrollWhenNeeded,
 				  XtNscrollVertical, XawtextScrollWhenNeeded,
 				  NULL);
+    XtInsertRawEventHandler(info->program, 
+			    ButtonPressMask | ButtonReleaseMask,
+                            False, (XtEventHandler) popupHandler, info, 
+                            XtListHead);
+
 
     info->cclog = 
           XtVaCreateManagedWidget("cclog", asciiTextWidgetClass, botform,
@@ -2090,6 +2119,11 @@ ScriptEditor(Widget w, Widget paint)
 				  XtNscrollHorizontal, XawtextScrollWhenNeeded,
 				  XtNscrollVertical, XawtextScrollWhenNeeded,
 				  NULL);
+    XtInsertRawEventHandler(info->cclog, 
+			    ButtonPressMask | ButtonReleaseMask,
+                            False, (XtEventHandler) popupHandler, info, 
+                            XtListHead);
+
 
     AddDestroyCallback(info->shell,
                        (DestroyCallbackFunc) closeCallback, (XtPointer) info);
@@ -2138,6 +2172,8 @@ ScriptEditor(Widget w, Widget paint)
 		      (XtEventHandler) mousewheelScroll, (XtPointer) NULL);
     XtAddEventHandler(info->cclog, ButtonPressMask, False,
 		      (XtEventHandler) mousewheelScroll, (XtPointer) NULL);
+    XtSetMinSizeHints(info->shell, 300, 160);
+    return info;
 }
 
 /* convenience routines */
@@ -2166,13 +2202,36 @@ RegionHeight()
     return pw->paint.region.rect.height;
 }
 
+void ImageToRegion(Image * input, XRectangle rect, Pixmap mask)
+{
+    Pixmap pix;
+    Colormap cmap;
+    ImageToPixmap(input, Global.curpaint, &pix, &cmap, 0);
+    PwRegionSet(Global.curpaint, &rect, pix, mask);
+}
+
+void ImageToCanvas(Image * input, Pixmap mask)
+{
+    Pixmap pix;
+    Colormap cmap;
+    XRectangle rect;
+    if (!input || !Global.curpaint) return;
+    rect.width = input->width;
+    rect.height = input->height;
+    rect.x = 0;
+    rect.y = 0;
+    ImageToPixmap(input, Global.curpaint, &pix, &cmap, Global.alpha_mode);
+    PwRegionSet(Global.curpaint, &rect, pix, mask);
+    PwRegionSet(Global.curpaint, None, None, None);
+}
+
 Image *
-CanvasImage()
+CanvasToImage()
 {
     PaintWidget pw = (PaintWidget)Global.curpaint;
     Image * image;
 
-    image = PixmapToImage((Widget)pw, GET_PIXMAP(pw), Global.region.cmap);
+    image = PixmapToImage((Widget)pw, GET_PIXMAP(pw), Global.clipboard.cmap);
     return image;
 }
 
