@@ -23,11 +23,9 @@
 #define XtWindow XTWINDOW
 #endif
 
-#include <X11/Intrinsic.h>
-#include <X11/Shell.h>
-#include <X11/StringDefs.h>
 #include <ctype.h>
 #include <stdio.h>
+#include <string.h>
 #include <fcntl.h>
 #include <time.h>
 #ifndef NOSTDHDRS
@@ -35,7 +33,10 @@
 #include <unistd.h>
 #endif
 
-#include <xpm.h>
+#include <X11/Intrinsic.h>
+#include <X11/Shell.h>
+#include <X11/StringDefs.h>
+#include <X11/xpm.h>
 
 #define DEFINE_GLOBAL
 #include "xpaint.h"
@@ -58,7 +59,11 @@ int msgNum;
 char lang[10];
 char *routine;
 
+extern int magnifier_closing_down;
+
 extern void BrushInit(Widget toplevel);
+extern int TestScriptC(char *file);
+extern Image * ReadScriptC(char *file);
 
 static char *appDefaults[] =
 {
@@ -76,12 +81,17 @@ typedef struct {
     String rcFile;
     String helpFile;
     String msgFile;
-    int zoom;
-    Boolean fullpopup;
-    Boolean nomenubar;
-    Boolean popped;
+    String dpi;
+    String filter;
+    String proc;
+    String zoom;
+    int undosize;
+    int operation;
+    Boolean fullmenu;
+    Boolean menubar;
+    Boolean canvas;
     Boolean snapshot;
-    Boolean clipboard;
+    Boolean magnifier;
     Boolean horizontal;
     Boolean nowarn;
 } AppInfo;
@@ -91,7 +101,7 @@ static AppInfo appInfo;
 static XtResource resources[] =
 {
     {"visualType", "VisualType", XtRString, sizeof(String),
-   XtOffset(AppInfo *, visualType), XtRImmediate, (XtPointer) "default"},
+     XtOffset(AppInfo *, visualType), XtRImmediate, (XtPointer) "default"},
     {"lang", "Lang", XtRString, sizeof(String),
      XtOffset(AppInfo *, lang), XtRImmediate, (XtPointer) NULL},
     {"size", "Size", XtRString, sizeof(String),
@@ -108,18 +118,28 @@ static XtResource resources[] =
      XtOffset(AppInfo *, helpFile), XtRImmediate, (XtPointer) NULL},
     {"msgFile", "MsgFile", XtRString, sizeof(String),
      XtOffset(AppInfo *, msgFile), XtRImmediate, (XtPointer) NULL},
-    {"zoom", "Zoom", XtRInt, sizeof(int),
-     XtOffset(AppInfo *, zoom), XtRImmediate, (XtPointer) 1},
-    {"fullpopup", "FullPopup", XtRBoolean, sizeof(Boolean),
-     XtOffset(AppInfo *, fullpopup), XtRImmediate, (XtPointer) True},
-    {"nomenubar", "NoMenubar", XtRBoolean, sizeof(Boolean),
-     XtOffset(AppInfo *, nomenubar), XtRImmediate, (XtPointer) False},
-    {"popped", "Popped", XtRBoolean, sizeof(Boolean),
-     XtOffset(AppInfo *, popped), XtRImmediate, (XtPointer) False},
+    {"dpi", "Dpi", XtRString, sizeof(String),
+     XtOffset(AppInfo *, dpi), XtRImmediate, (XtPointer) NULL},
+    {"filter", "Filter", XtRString, sizeof(String),
+     XtOffset(AppInfo *, filter), XtRImmediate, (XtPointer) NULL},
+    {"proc", "Proc", XtRString, sizeof(String),
+     XtOffset(AppInfo *, proc), XtRImmediate, (XtPointer) NULL},
+    {"zoom", "Zoom", XtRString, sizeof(String),
+     XtOffset(AppInfo *, zoom), XtRImmediate, (XtPointer) NULL},
+    {"undosize", "UndoSize", XtRInt, sizeof(int),
+     XtOffset(AppInfo *, undosize), XtRImmediate, (XtPointer) 4},
+    {"operation", "Operation", XtRInt, sizeof(int),
+     XtOffset(AppInfo *, operation), XtRImmediate, (XtPointer) 1},
+    {"fullmenu", "FullMenu", XtRBoolean, sizeof(Boolean),
+     XtOffset(AppInfo *, fullmenu), XtRImmediate, (XtPointer) False},
+    {"menubar", "Menubar", XtRBoolean, sizeof(Boolean),
+     XtOffset(AppInfo *, menubar), XtRImmediate, (XtPointer) True},
+    {"canvas", "Canvas", XtRBoolean, sizeof(Boolean),
+     XtOffset(AppInfo *, canvas), XtRImmediate, (XtPointer) False},
     {"snapshot", "Snapshot", XtRBoolean, sizeof(Boolean),
      XtOffset(AppInfo *, snapshot), XtRImmediate, (XtPointer) False},
-    {"clipboard", "Clipboard", XtRBoolean, sizeof(Boolean),
-     XtOffset(AppInfo *, clipboard), XtRImmediate, (XtPointer) False},
+    {"magnifier", "Magnifier", XtRBoolean, sizeof(Boolean),
+     XtOffset(AppInfo *, magnifier), XtRImmediate, (XtPointer) False},
     {"horizontal", "Horizontal", XtRBoolean, sizeof(Boolean),
      XtOffset(AppInfo *, horizontal), XtRImmediate, (XtPointer) False},
     {"nowarn", "NoWarn", XtRBoolean, sizeof(Boolean),
@@ -137,19 +157,25 @@ static XrmOptionDescRec options[] =
     {"-rcfile", ".rcFile", XrmoptionSepArg, (XtPointer) NULL},
     {"-helpfile", ".helpFile", XrmoptionSepArg, (XtPointer) NULL},
     {"-msgfile", ".msgFile", XrmoptionSepArg, (XtPointer) NULL},
-    {"-zoom", ".zoom", XrmoptionSepArg, (XtPointer) "-1"},
-    {"-simplepopup", ".fullpopup", XrmoptionNoArg, (XtPointer) "False"},
-    {"-fullpopup", ".fullpopup", XrmoptionNoArg, (XtPointer) "True"},
-    {"-nomenubar", ".nomenubar", XrmoptionNoArg, (XtPointer) "True"},
-    {"-popped", ".popped", XrmoptionNoArg, (XtPointer) "True"},
+    {"-filter", ".filter", XrmoptionSepArg, (XtPointer) NULL},
+    {"-proc", ".proc", XrmoptionSepArg, (XtPointer) NULL},
+    {"-zoom", ".zoom", XrmoptionSepArg, (XtPointer) "1"},
+    {"-undosize", ".undosize", XrmoptionSepArg, (XtPointer) "2"},
+    {"-operation", ".operation", XrmoptionSepArg, (XtPointer) "1"},
+    {"-fullmenu", ".fullmenu", XrmoptionNoArg, (XtPointer) "True"},
+    {"-simplemenu", ".fullmenu", XrmoptionNoArg, (XtPointer) "False"},
+    {"-menubar", ".menubar", XrmoptionNoArg, (XtPointer) "True"},
+    {"-nomenubar", ".menubar", XrmoptionNoArg, (XtPointer) "False"},
+    {"-canvas", ".canvas", XrmoptionNoArg, (XtPointer) "True"},
     {"-snapshot", ".snapshot", XrmoptionNoArg, (XtPointer) "True"},
-    {"-clipboard", ".clipboard", XrmoptionNoArg, (XtPointer) "True"},
+    {"-magnifier", ".magnifier", XrmoptionNoArg, (XtPointer) "True"},
     {"-horizontal", ".horizontal", XrmoptionNoArg, (XtPointer) "True"},
     {"-nowarn", ".nowarn", XrmoptionNoArg, (XtPointer) "True"},
     {"-help", ".help", XrmoptionNoArg, (XtPointer) "command"},
     {"+help", ".help", XrmoptionSepArg, (XtPointer) NULL},
     {"-8", ".visualType", XrmoptionNoArg, (XtPointer) "cmap8"},
     {"-visual", ".visualType", XrmoptionSepArg, (XtPointer) NULL},
+    {"-dpi", ".dpi", XrmoptionSepArg, (XtPointer) NULL}
 };
 
 
@@ -157,28 +183,24 @@ static XrmOptionDescRec options[] =
 **  Public query functions for application defaults
  */
 
-void 
-SetDefaultWHZ(int w, int h, int zoom)
+int 
+GetDefaultUndosize()
 {
-    char str[40];
-    if (w<=0) w = 1;
-    if (h<=0) h = 1;
-    if (zoom<=0) zoom = -1;
-    if (zoom>32) zoom = 32;
-    sprintf(str, "%dx%d", w, h);
-    appInfo.size = strdup(str);
-    appInfo.zoom = zoom;
+    if (appInfo.undosize<0) appInfo.undosize = 0;
+    return appInfo.undosize;
 }
 
 void 
-GetDefaultWH(int *w, int *h)
+SetDefaultWHZ(int w, int h, int zoom)
 {
-    int x, y;
-    unsigned int width, height;
-
-    XParseGeometry(appInfo.size, &x, &y, &width, &height);
-    *w = width;
-    *h = height;
+    if (w<=0) w = 1;
+    if (h<=0) h = 1;
+    if (zoom==-1) zoom = 1;
+    if (zoom<-16) zoom = -16;
+    if (zoom>32) zoom = 32;
+    Global.default_width = w;
+    Global.default_height = h;
+    Global.default_zoom = zoom;
 }
 
 void 
@@ -192,36 +214,66 @@ GetPaintWH(int *w, int *h)
     *h = height;
 }
 
-void 
-GetInitZoom(int *z)
+int
+GetInitZoom()
 {
-    *z = appInfo.zoom;
-    if (*z<=0) *z = -1;
-    if (*z>32) *z = 32;
+    char *ptr;
+    int z;
+
+    if (!appInfo.zoom)
+        return 1; 
+
+    ptr = strchr(appInfo.zoom,':');
+    if (!ptr) 
+        ptr=strchr(appInfo.zoom,'/');
+    if (ptr)
+        z = -atoi(ptr+1);
+    else
+        z = atoi(appInfo.zoom);
+    if (z<-8) z = -8;
+    if (z==-1) z = 0;
+    if (z>32) z = 32;
+    return z;
 }
 
 void 
-SetMenuBarHidden(Boolean flag)
+SetMenuBarVisibility(Widget w, Boolean flag)
 {
-    appInfo.nomenubar = flag;
+    XtVaSetValues(w, XtNmenubar, flag, NULL);
 }
 
 Boolean
-ThereIsNoMenuBar()
+IsMenuBarVisible(Widget w)
 {
-    return appInfo.nomenubar;
-}
-
-void 
-SetFullMenuPopup(Boolean flag)
-{
-    appInfo.fullpopup = flag;
+    Boolean v;
+    XtVaGetValues(w, XtNmenubar, &v, NULL);
+    return v;
 }
 
 Boolean
-IsFullPopup()
+IsMenuBarGlobal()
 {
-    return appInfo.fullpopup;
+    return appInfo.menubar;
+}
+
+void 
+SetFullMenu(Widget w, Boolean flag)
+{
+    XtVaSetValues(w, XtNfullmenu, flag, NULL);
+}
+
+Boolean
+IsFullMenuSet(Widget w)
+{
+    Boolean v;
+    XtVaGetValues(w, XtNfullmenu, &v, NULL);
+    return v;
+}
+
+Boolean
+IsFullMenuGlobal()
+{
+    return appInfo.fullmenu;
 }
 
 Boolean
@@ -242,6 +294,12 @@ GetShareDir(void)
     return appInfo.shareDir;
 }
 
+double
+GetDpi(void)
+{
+    return Global.dpi;
+}
+
 /*
 **  Create a nice icon image for XPaint...
  */
@@ -268,7 +326,7 @@ SetIconImage(Widget w)
  	myattributes.closeness = 65535;
  	if (XpmCreatePixmapFromData(dpy, iconWindow,
 				    XPaintIcon_xpm, &icon, NULL, &myattributes)) {
-	    fprintf(stderr, msgText[CANNOT_INSTALL_XPM_ICON]);
+	    fprintf(stderr, "%s",msgText[CANNOT_INSTALL_XPM_ICON]);
 	    exit(1);
  	}
 	GetPixmapWHD(dpy, icon, &iconW, &iconH, NULL);
@@ -302,33 +360,20 @@ typedef struct {
     char **files;
 } LocalInfo;
 
-static Boolean
-workProc(LocalInfo * l)
+static void
+processFile(char *file)
 {
-    char *file;
     void *v;
 
-    if ((l == NULL) || (l->id == None))
-	return TRUE;
-
-    file = l->files[l->pos];
-
     StateSetBusy(True);
 
     if ((v = ReadMagic(file)) != NULL)
-	GraphicOpenFile(l->toplevel, file, v);
+        GraphicOpenFile(Global.toplevel, file, v);
     else
-	Notice(l->toplevel, msgText[UNABLE_TO_OPEN_INPUT_FILE],
+	Notice(Global.toplevel, msgText[UNABLE_TO_OPEN_INPUT_FILE],
 	       file, RWGetMsg());
 
     StateSetBusy(False);
-
-    if (++l->pos == l->nfiles) {
-	XtFree((XtPointer) l->files);
-	XtFree((XtPointer) l);
-	return TRUE;
-    } else
-	return FALSE;
 }
 
 /*
@@ -390,7 +435,7 @@ GetFutureVisual(Widget *toplevel, int desiredDepth, int desiredVisual,
     }
 	
     if (visualsMatched == 0) {
-        fprintf(stderr, msgText[NO_MATCH_FOUND_FOR_VISUAL]);
+        fprintf(stderr, "%s", msgText[NO_MATCH_FOUND_FOR_VISUAL]);
 	exit(1);
     }
 	
@@ -491,7 +536,7 @@ LoadMessages()
             ++msgNum;
             msgText = (String *)realloc(msgText, msgNum*sizeof(String));
 	    if (!msgText) {
-	        fprintf(stderr, failure);
+	        fprintf(stderr, "%s", failure);
 	        exit(1);
 	    }
             i = 0;
@@ -510,7 +555,7 @@ LoadMessages()
 	    buf[j] = '\0';
             msgText[msgNum-1] = strdup(buf);
 	    if (!msgText[msgNum-1]) {
-	        fprintf(stderr, failure);
+	        fprintf(stderr, "%s", failure);
 	        exit(1);
 	    }
         }
@@ -574,7 +619,7 @@ checkRCsize()
 		    ptr = buf + 12;
 		    while (isspace(*ptr)) ++ptr;
 	            if (sscanf(ptr, "%dx%d", &w, &h) == 2) {
-	                 SetDefaultWHZ(w, h, appInfo.zoom);
+	                 SetDefaultWHZ(w, h, GetInitZoom());
 		    }
 		    break;
 		}
@@ -590,19 +635,19 @@ main(int argc, char *argv[])
 {
     Display *dpy;
     Widget toplevel;
-    XtAppContext app;
     char xpTitle[20];
-    int i;
+    int i, j;
     XEvent event;
     Arg args[5];
     int nargs = 0;
     XrmDatabase rdb;
-    Boolean isIcon;
+    Boolean isIcon, valid;
 
     Widget temp;
     int desiredVisual = -1;
+    int opened = 0, mode;
     XVisualInfo vis;
-    Boolean valid;
+    Image * image;
     int depth;
     int desiredDepth;
     int tempargc;
@@ -638,6 +683,7 @@ main(int argc, char *argv[])
      * This widget will not be realized. 
      * There should be a better way to do this...
      */
+    memset(&appInfo, 0, sizeof(AppInfo));
     temp = XtAppInitialize(&Global.appContext, app_def_file,
 			   options, XtNumber(options), &tempargc, tempargv,
 			   appDefaults, args, nargs);
@@ -739,7 +785,7 @@ main(int argc, char *argv[])
 	    Global.vis = vis;
 	}
 	else {
-	    fprintf(stderr, msgText[NO_MATCH_FOUND_FOR_VISUAL]);
+	    fprintf(stderr, "%s", msgText[NO_MATCH_FOUND_FOR_VISUAL]);
 	    exit(1);
 	}
     }
@@ -764,7 +810,8 @@ main(int argc, char *argv[])
 			       options, XtNumber(options), &argc, argv,
 			       appDefaults, args, nargs);
 
-    XtGetApplicationResources(toplevel, (XtPointer) & appInfo,
+    memset(&appInfo, 0, sizeof(AppInfo));
+    XtGetApplicationResources(toplevel, (XtPointer) &appInfo,
 			      resources, XtNumber(resources), NULL, 0);
 
     if (argc != 1 && argv[1][0] == '-')
@@ -780,22 +827,35 @@ main(int argc, char *argv[])
     Global.back = None;
     Global.patternshell = None;
     Global.patterninfo = NULL;
-    Global.region.image = NULL;
-    Global.region.cmap = None;
-    Global.region.width = 0;
-    Global.region.height = 0;
-    Global.region.pix = None;
-    Global.region.mask = None;
+    Global.clipboard.image = NULL;
+    Global.clipboard.cmap = None;
+    Global.clipboard.width = 0;
+    Global.clipboard.height = 0;
+    Global.clipboard.pix = None;
+    Global.clipboard.mask = None;
+    Global.clipboard.alpha = NULL;
     Global.nbrushes = 0;
     Global.brushes = NULL;
-    Global.join = JoinMiter;
+    Global.cap = 0;
+    Global.join = 0;
     Global.escape = False;
+    Global.transparent = 1;
     Global.dashlist[0] = '\0';
     Global.dashnumber = 0;
     Global.dashoffset = 0;
+    Global.dpi = 300.0;
     Global.numregions = 0;
-    Global.transparent = 0;
     Global.regiondata = NULL;
+    Global.curpaint = None;
+    Global.popped_up = None;
+    Global.popped_parent = None;
+    Global.numfiles = 0;
+    Global.loadedfiles = NULL;
+    Global.numpage = 1;
+    Global.alpha_mode = 2;
+    Global.alpha_default = 255;
+    Global.alpha_threshold = 128;
+    Global.background = 0xffffff;
 
     /*
     **
@@ -817,9 +877,9 @@ main(int argc, char *argv[])
 	switch (depth) {
 	    case 8:
 	    case 12:
-		fprintf(stderr, msgText[SEPARATOR_STRIP_ONE]);
-		fprintf(stderr, msgText[WARNING_ONE]);
-		fprintf(stderr, msgText[SEPARATOR_STRIP_ONE]);
+		fprintf(stderr, "%s", msgText[SEPARATOR_STRIP_ONE]);
+		fprintf(stderr, "%s", msgText[WARNING_ONE]);
+		fprintf(stderr, "%s", msgText[SEPARATOR_STRIP_ONE]);
 		fprintf(stderr, "\n");
 		break;
 	    case 1:
@@ -828,17 +888,17 @@ main(int argc, char *argv[])
 	    case 6:
 	    case 15:
 	    case 16:
-		fprintf(stderr, msgText[SEPARATOR_STRIP_TWO]);
-		fprintf(stderr, msgText[WARNING_TWO_A]);
-		fprintf(stderr, msgText[WARNING_TWO_B]);
-		fprintf(stderr, msgText[SEPARATOR_STRIP_TWO]);
+		fprintf(stderr, "%s", msgText[SEPARATOR_STRIP_TWO]);
+		fprintf(stderr, "%s", msgText[WARNING_TWO_A]);
+		fprintf(stderr, "%s", msgText[WARNING_TWO_B]);
+		fprintf(stderr, "%s", msgText[SEPARATOR_STRIP_TWO]);
 		fprintf(stderr, "\n");
 		break;
 	    case 24:
 	    case 32:
-		fprintf(stderr, msgText[WARNING_THREE_A]);
-		fprintf(stderr, msgText[WARNING_THREE_B]);
-		fprintf(stderr, msgText[WARNING_THREE_C]);
+		fprintf(stderr, "%s", msgText[WARNING_THREE_A]);
+		fprintf(stderr, "%s", msgText[WARNING_THREE_B]);
+		fprintf(stderr, "%s", msgText[WARNING_THREE_C]);
 		break;
 	    default:
 		break;
@@ -850,7 +910,8 @@ main(int argc, char *argv[])
      */
     Global.timeToDie = False;
     Global.explore = False;
-    app = XtWidgetToApplicationContext(toplevel);
+
+    Global.appContext = XtWidgetToApplicationContext(toplevel);
     dpy = Global.display = XtDisplay(toplevel);
 
     /*
@@ -858,6 +919,7 @@ main(int argc, char *argv[])
      */
 
     OperationInit(toplevel);
+    if (appInfo.operation>0) OperationSet(NULL, -appInfo.operation);
 
     /*
     **  A few rogue initializers
@@ -876,39 +938,67 @@ main(int argc, char *argv[])
     XtVaSetValues(toplevel, XtNtitle, xpTitle, NULL);
     XtVaSetValues(toplevel, XtNiconName, xpTitle, XtNtitle, xpTitle, NULL);
 
+    if (appInfo.dpi) {
+        Global.dpi = atof(appInfo.dpi);
+        if (Global.dpi<0) Global.dpi = 0.0;
+    }
+
+    XParseGeometry(appInfo.size, &i, &j, 
+                   &Global.default_width, &Global.default_height);
+    Global.default_zoom = GetInitZoom();
+
     if (appInfo.snapshot) {
         SnapshotImage(toplevel, NULL, 0);
     }
+
+    if (appInfo.magnifier) {
+        StartMagnifier(toplevel);
+    }
+
     /*
     **  Realize it (doesn't hurt)
      */
+    if (appInfo.magnifier && argc==1)
+        XtVaSetValues(toplevel, XtNmappedWhenManaged, False, NULL);
     XtRealizeWidget(toplevel);
+    XtSetMinSizeHints(toplevel, 120, 82);
 
     /*
     **  Now open any file on the command line
      */
-    if (argc != 1) {
-        if (appInfo.clipboard) {
-	   Image * image;
-	   for (i = 1; i < argc; i++) {
-  	       image = ImageFromFile(argv[i]);
-               if (image)
-                  ImageToMemory(image);
-	   }
-	} else {
-  	   LocalInfo *l = XtNew(LocalInfo);
-	   l->pos = 0;
-	   l->toplevel = toplevel;
-  	   l->nfiles = argc - 1;
-	   l->files = (char **) XtCalloc(argc, sizeof(char *));
-           for (i = 1; i < argc; i++)
-	       l->files[i - 1] = argv[i];
-	   l->id = XtAppAddWorkProc(app, (XtWorkProc) workProc, (XtPointer) l);
+    mode = 0;
+
+    if (argc > 1)
+    for (i = 1; i < argc; i++) {
+        /* loading a file */
+        if (!strcmp(argv[i], "/l")) mode = 0;
+        else
+        /* loading clipboard */
+        if (!strcmp(argv[i], "/c")) mode = 1;
+        else
+        /* opening a file */
+        if (!strcmp(argv[i], "/o")) mode = 2;
+        else
+	switch(mode) {
+	    case 0:
+	        AddFileToGlobalList(argv[i]);
+	        break;
+	    case 1:
+	        AddFileToGlobalList(argv[i]);
+  	        image = ImageFromFile(argv[i]);
+                if (image)
+                    ImageToMemory(image);
+                break;
+	    case 2:
+	        ++opened;
+                processFile(argv[i]);
+                break;
 	}
-    } else if (appInfo.popped) {
+    } else if (appInfo.canvas) {
 	/*
 	**  If nothing is coming up, bring up a blank canvas
 	 */
+        ++opened;
 	GraphicCreate(toplevel, 0);
     }
 
@@ -917,14 +1007,43 @@ main(int argc, char *argv[])
     XtUnmanageChild(Global.back);
     XMapWindow(dpy, XtWindow(Global.back));
 
+    /* Check if there's a filter being loaded */
+    if (appInfo.filter) {
+        if (TestScriptC(appInfo.filter) == 2)
+           ReadScriptC(appInfo.filter);
+        else
+           Notice(toplevel, msgText[UNABLE_TO_LOAD_FILTER]);
+    }
+
+    /* Check if there's a procedure being loaded */
+    if (appInfo.proc) {
+        if (TestScriptC(appInfo.proc) == 3)
+           ReadScriptC(appInfo.proc);
+        else
+           Notice(toplevel, msgText[UNABLE_TO_LOAD_PROCEDURE]);
+    }      
+
+    if (Global.numfiles>0 && opened==0) {
+        ++opened;
+        processFile(Global.loadedfiles[0]);
+    }
+
+    if (Global.numfiles>opened)
+        GetFileName(Global.toplevel, 4, NULL, GraphicOpenFile, NULL);
+
     /*
     **  MainLoop
      */
     do {
-	XtAppNextEvent(app, &event);
+	XtAppNextEvent(Global.appContext, &event);
 	if (event.type == ButtonPress || event.type == ButtonRelease)
 	    Global.currentTime = event.xbutton.time;
 	XtDispatchEvent(&event);
+        if (appInfo.magnifier && argc==1 && magnifier_closing_down) {
+	    XtVaSetValues(toplevel, XtNmappedWhenManaged, True, NULL);
+            XMapRaised(dpy, XtWindow(toplevel));
+            appInfo.magnifier = 0;
+	}
     }
     while (!Global.timeToDie);
     return 0;
