@@ -24,6 +24,7 @@
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/cursorfont.h>
+#include <X11/xpm.h>
 
 #include "xaw_incdir/Form.h"
 #include "xaw_incdir/SmeBSB.h"
@@ -31,8 +32,6 @@
 #include "xaw_incdir/Toggle.h"
 #include "xaw_incdir/Viewport.h"
 
-#include <xpm.h>
-
 #include "xpaint.h"
 #include "misc.h"
 #include "menu.h"
@@ -98,19 +97,23 @@ static void
 setOperation(Widget w, XtPointer opArg, XtPointer junk2)
 {
     OperationPair *op = (OperationPair *) opArg;
-    OperationAddProc start;
-    OperationRemoveProc stop;
 
-    if (op == CurrentOp)
+    if (op == CurrentOp || op == None)
 	return;
 
-    start = op ? op->add : NULL;
-    stop = CurrentOp ? CurrentOp->remove : NULL;
+    routine = XtName(w);
 
-    GraphicSetOp((OperationRemoveProc) stop, (OperationAddProc) start);
+    /* terminate possibly unfinished operation */
+    if (CurrentOp)
+        GraphicSetOp((OperationRemoveProc) CurrentOp->remove, 
+                     (OperationAddProc) NULL);
 
+
+    /* start new one */
     CurrentOp = op;
     GraphicAll(setToolIconPixmap, NULL);
+    GraphicSetOp((OperationRemoveProc) NULL, 
+                 (OperationAddProc) CurrentOp->add);
 }
 
 /*
@@ -588,7 +591,15 @@ exitPaint(Widget w, XtPointer junk, XtPointer junk2)
 void 
 takeSnapshot(Widget w, XtPointer junk, XtPointer junk2)
 {
-    SnapshotImage(GetToplevel(XtParent(w)), junk, 0);
+    if (XtParent(w)) w = GetToplevel(XtParent(w));
+    if (w) {
+       PopdownMenusGlobal();
+       XUnmapWindow(XtDisplay(Global.toplevel), XtWindow(Global.toplevel));
+       XFlush(XtDisplay(Global.toplevel));
+       usleep(120000);
+       SnapshotImage(w, junk, 0);
+       XMapWindow(XtDisplay(Global.toplevel), XtWindow(Global.toplevel));
+    }
 }
 
 /*
@@ -893,7 +904,7 @@ static PaintMenuItem fractalFillPopup[] =
 void 
 brushPopupCB(Widget w, XtPointer junk, XtPointer junk2)
 {
-    int nm = (int) junk;
+    int nm = (int)(long)junk;
 
     BrushSetMode(nm);
     MenuCheckItem(brushPopup[nm + 1].widget, True);
@@ -913,7 +924,7 @@ ovalPopupCB(Widget w, XtPointer junk, XtPointer junk2)
 static void 
 polygonPopupCB(Widget w, XtPointer junk, XtPointer junk2)
 {
-    int nm = (int) junk;
+    int nm = (int)(long)junk;
 
     MenuCheckItem(polygonPopup[nm + 1].widget, True);
     MenuCheckItem(filledPolygonPopup[nm + 1].widget, True);
@@ -926,7 +937,7 @@ boxPopupCB(Widget w, XtPointer junk, XtPointer junk2)
 {
     Boolean b;
     int m;
-    int nm = (int) junk;
+    int nm = (int)(long)junk;
 
     m = BoxGetStyle(); 
     b = (m & (1<<nm))? False : True;
@@ -941,7 +952,7 @@ boxPopupCB(Widget w, XtPointer junk, XtPointer junk2)
 static void 
 arcPopupCB(Widget w, XtPointer junk, XtPointer junk2)
 {
-    int nm = (int) junk;
+    int nm = (int)(long)junk;
 
     ArcSetMode(nm);
     MenuCheckItem(w, True);
@@ -950,7 +961,7 @@ arcPopupCB(Widget w, XtPointer junk, XtPointer junk2)
 static void 
 splinePopupCB(Widget w, XtPointer junk, XtPointer junk2)
 {
-    int nm = (int) junk;
+    int nm = (int)(long)junk;
 
     MenuCheckItem(splinePopup[nm + 1].widget, True);
     MenuCheckItem(filledSplinePopup[nm + 1].widget, True);
@@ -972,14 +983,14 @@ OperationSelectCall(int n)
 static void 
 selectPopupCB(Widget w, XtPointer junk, XtPointer junk2)
 {
-    int nm = (int) junk;
+    int nm = (int)(long)junk;
     OperationSelectCall(nm);
 }
 
 static void 
 fillPopupCB(Widget w, XtPointer junk, XtPointer junk2)
 {
-    int nm = (int) junk;
+    int nm = (int)(long)junk;
 
     FillSetMode(nm);
     MenuCheckItem(fillPopup[nm+1].widget, True);
@@ -990,7 +1001,7 @@ fillPopupCB(Widget w, XtPointer junk, XtPointer junk2)
 static void 
 gradientFillPopupCB2(Widget w, XtPointer junk, XtPointer junk2)
 {
-    int nm = (int) junk;
+    int nm = (int)(long)junk;
 
     GradientFillSetMode(nm);
     MenuCheckItem(w, True);
@@ -1000,7 +1011,7 @@ gradientFillPopupCB2(Widget w, XtPointer junk, XtPointer junk2)
 static void 
 fractalFillPopupCB2(Widget w, XtPointer junk, XtPointer junk2)
 {
-    int nm = (int) junk;
+    int nm = (int)(long)junk;
 
     FractalFillSetMode(nm);
     MenuCheckItem(w, True);
@@ -1029,7 +1040,7 @@ sprayPopupCB(Widget w, XtPointer junk, XtPointer junk2)
 static void 
 linePopupCB(Widget w, XtPointer junk, XtPointer junk2)
 {
-    int nm = (int) junk;
+    int nm = (int)(long)junk;
     Boolean m;
 
     if (nm == 0) {
@@ -1219,6 +1230,15 @@ OperationSet(String names[], int num)
     IconListItem *match = NULL;
     int i, j;
 
+    if (num<0) {
+        i = -num-1;
+        if (i >= XtNumber(iconList)) i = 0;
+        match = &iconList[i];
+        if (CurrentOp == match->data) return;
+	XawToggleSetCurrent(iconListWidget, (XtPointer) match->name);
+        return;
+    }
+
     for (i = 0; i < XtNumber(iconList); i++) {
 	for (j = 0; j < num; j++) {
 	    if (strcmp(names[j], iconList[i].name) == 0) {
@@ -1266,8 +1286,15 @@ static PaintMenuItem fontMenu[] =
 	    fontSet, "-*-helvetica-medium-r-normal-*-*-120-*-*-p-*-*-*"),
     MI_FLAGCB("Helvetica Bold 12", MF_CHECK | MF_GROUP1,
 	      fontSet, "-*-helvetica-bold-r-normal-*-*-120-*-*-p-*-*-*"),
+#ifndef __NetBSD__
     MI_FLAGCB("Fixed 12", MF_CHECK | MF_GROUP1,
 	      fontSet, "-*-fixed-medium-r-normal-*-*-120-*-*-m-*-*-*"),
+#else
+    MI_FLAGCB("Sony 16", MF_CHECK | MF_GROUP1,
+	      fontSet, "-*-fixed-medium-r-normal--16-120-100-100-c-80-iso8859-1"),
+    MI_FLAGCB("Sony 24", MF_CHECK | MF_GROUP1,
+	      fontSet, "-*-fixed-medium-r-normal--24-170-100-100-c-120-iso8859-1");
+#endif
     MI_FLAGCB("Courier 12", MF_CHECK | MF_GROUP1,
 	      fontSet, "-*-courier-medium-r-normal-*-*-120-*-*-m-*-*-*"),
     MI_SEPARATOR(),
@@ -1288,8 +1315,10 @@ static PaintMenuItem otherMenu[] =
 static PaintMenuItem canvasMenu[] =
 {
     MI_SIMPLECB("new", GraphicCreate, 0),
-    MI_SIMPLECB("new-size", GraphicCreate, 2),
-    MI_SIMPLECB("open", GraphicCreate, 1),
+    MI_SIMPLECB("new-size", GraphicCreate, 1),
+    MI_SIMPLECB("open", GraphicCreate, 2),
+    MI_SIMPLECB("loaded", GraphicCreate, 3),
+    MI_SIMPLECB("magnifier", GraphicCreate, 4),
     MI_SIMPLECB("snapshot", takeSnapshot, NULL),
     MI_SEPARATOR(),
     MI_SIMPLECB("quit", exitPaint, NULL),
@@ -1419,14 +1448,50 @@ switchtoCanvasCallback(Widget w, XtPointer junk, XEvent * event, Boolean * flg)
        XMapRaised(XtDisplay(Global.canvas), XtWindow(Global.canvas));
 }
 
+void
+processEvent(Widget w, XtPointer arg, XEvent * event, Boolean * flg)
+{
+    KeySym keysym;
+    char buf[4];
+    int len;
+
+    if (event->type == KeyPress || event->type == KeyRelease) {
+        len = XLookupString((XKeyEvent *)event, 
+                            buf, sizeof(buf) - 1, &keysym, NULL);
+        if (keysym == XK_Escape)
+            PopdownMenusGlobal();
+        return;
+    }
+
+    if (event->type == ButtonPress || event->type == ButtonRelease) {
+        if (Global.popped_up) {
+	    if (event->type == ButtonRelease)
+                PopdownMenusGlobal();
+            event->type = None;
+            return;
+	}
+        if (arg) {
+            XtVaSetValues(w, XtNstate, True, NULL);
+	    setOperation(w, arg, NULL);
+	}
+    }
+}
+
 void
 toolsResized(Widget w, XtPointer l, XConfigureEvent * event, Boolean * flg)
 {
     Dimension width, height;
+
+    if (event->type != ConfigureNotify) return;
+
     XtVaGetValues(w, XtNwidth, &width,
 		     XtNheight, &height, NULL);
-    XtResizeWidget((Widget)l, (width>20)? width - 8:12 ,
-                                  (height>50)? height - 38:12,
+
+    if (width<40 || height<72)
+        XtVaSetValues(w, XtNsensitive, False, NULL);
+
+    XtResizeWidget((Widget)l, (width>40)? width - 8:32 ,
+                                  (height>70)? height - 38:32,
 #ifdef XAW3D		   
 		                  1);
     XtMoveWidget((Widget)l, 4, 35);
@@ -1436,12 +1501,15 @@ toolsResized(Widget w, XtPointer l, XConfigureEvent * event, Boolean * flg)
 #endif		
 #if defined(XAWPLAIN) || defined(XAW95) || defined(XAW3D)
 #ifdef XAW3DG
-    XtMoveWidget(Global.back, width-20, 8);
+    if (width>20)
+        XtMoveWidget(Global.back, width-20, 8);
 #else
-    XtMoveWidget(Global.back, width-17, 8);
+    if (width>17)
+        XtMoveWidget(Global.back, width-17, 8);
 #endif
 #else
-    XtMoveWidget(Global.back, width-15, 8);
+    if (width>15)
+        XtMoveWidget(Global.back, width-15, 8);
 #endif
 }
 
@@ -1467,7 +1535,11 @@ OperationInit(Widget toplevel)
     int i;
     Pixmap pix;
     Widget form, vport, box, icon, firstIcon = None;
-    char *defTrans = "<BtnDown>,<BtnUp>: set() notify()\n";
+    char *defTrans = "\
+<Key>Escape: escape()\n\
+<BtnDown>,<BtnUp>: set() notify()\n\
+";
+
     XtTranslations trans = XtParseTranslationTable(defTrans);
     IconListItem *cur;
     XpmAttributes attributes;
@@ -1498,7 +1570,6 @@ OperationInit(Widget toplevel)
                   XtNhorizDistance, 0,
 #endif		  
 		  NULL);
-    
     /*
     **  Create the operation icon list
      */
@@ -1513,6 +1584,7 @@ OperationInit(Widget toplevel)
 #endif				    	       
 #ifdef XAW3D
 				    XtNvertDistance, 5,
+                                    XtNborderWidth, 0,
 #endif				    
 				    NULL);
 
@@ -1559,8 +1631,8 @@ OperationInit(Widget toplevel)
 	}
 
 	if (pix == None) {
-	    fprintf(stderr, msgText[UNABLE_TO_ALLOCATE_SUFFICIENT_COLOR_ENTRIES]);
-	    fprintf(stderr, msgText[TRY_EXITING_OTHER_COLOR_INTENSIVE_APPLICATIONS]);
+	    fprintf(stderr, "%s", msgText[UNABLE_TO_ALLOCATE_SUFFICIENT_COLOR_ENTRIES]);
+	    fprintf(stderr, "%s", msgText[TRY_EXITING_OTHER_COLOR_INTENSIVE_APPLICATIONS]);
 	    exit(1);
 	}
 	
@@ -1568,6 +1640,7 @@ OperationInit(Widget toplevel)
 
 	if (pix != None)
 	    XtVaSetValues(icon, XtNbitmap, pix, NULL);
+
 	if (cur->translations != NULL) {
 	    XtTranslations moreTrans;
 
@@ -1581,8 +1654,15 @@ OperationInit(Widget toplevel)
 	    firstIcon = icon;
 	    iconListWidget = icon;
 	}
-	XtAddCallback(icon, XtNcallback,
-		      setOperation, cur->data);
+
+        XtAddEventHandler(icon, ButtonPressMask | ButtonReleaseMask |
+                          KeyPressMask | KeyReleaseMask, False, 
+			  (XtEventHandler) processEvent, cur->data);
+
+	/* 
+         *  More sophisticated event handler above
+         *  XtAddCallback(icon, XtNcallback, setOperation, cur->data);
+         */
 #if defined(XAW3D) && !defined(XAW3DG)
 #ifdef BIGTOOLICONS
         XtVaSetValues(icon, XtNwidth, 52, XtNheight, 48, XtNborderWidth, 0,
@@ -1593,11 +1673,10 @@ OperationInit(Widget toplevel)
 #endif				       
 #if defined(XAW3D) && defined(XAW3DG)
 #ifdef BIGTOOLICONS
-        XtVaSetValues(icon, XtNwidth, 48, XtNheight, 48, XtNborderWidth, 0,
+	XtResizeWidget(icon, 48, 48, 0);
 #else
-        XtVaSetValues(icon, XtNwidth, 32, XtNheight, 32, XtNborderWidth, 0,
+	XtResizeWidget(icon, 34, 32, 0);
 #endif
-                            NULL);
 #endif				       
 #ifdef XAW3D
         XtVaGetValues(icon, XtNbackground, &bg, NULL);
@@ -1608,70 +1687,80 @@ OperationInit(Widget toplevel)
     }
 
     if (ToolsAreHorizontal())
-        XtVaSetValues(vport,
+        XtResizeWidget(vport,
 #if defined(XAW3D) && !defined(XAW3DG)		      
 #ifdef BIGTOOLICONS
-	          XtNwidth, 565, XtNheight, 157,
+	          565, 157, 0);
 #else
-	          XtNwidth, 404, XtNheight, 108,
+	          424, 108, 0);
 #endif
 #endif
 #ifdef XAW3DG
 #ifdef BIGTOOLICONS
-	          XtNwidth, 529, XtNheight, 149,
+	          529, 149, 0);
 #else
-	          XtNwidth, 368, XtNheight, 112,
+	          388, 112, 0);
 #endif
 #endif
 #ifdef XAW95
 #ifdef BIGTOOLICONS
-	          XtNwidth, 550, XtNheight, 155,
+	          550, 155, 0);
 #else
-	          XtNwidth, 388, XtNheight, 106,
+	          388, 106, 0);
 #endif
 #endif
 #ifdef XAWPLAIN		      
 #ifdef BIGTOOLICONS
-		  XtNwidth, 544, XtNheight, 152,
+		  544, 152, 0);
 #else
-		  XtNwidth, 386, XtNheight, 102,
+		  386, 102, 0);
 #endif
 #endif	      
-	         NULL);   
     else
-        XtVaSetValues(vport,
+        XtResizeWidget(vport,
 #if defined(XAW3D) && !defined(XAW3DG)
 #ifdef BIGTOOLICONS
-	          XtNwidth, 173, XtNheight, 521,
+		      173, 521, 0);
 #else
-	          XtNwidth, 124, XtNheight, 361,
+		      124, 361, 0);
 #endif
 #endif
 #ifdef XAW3DG
 #ifdef BIGTOOLICONS
-	          XtNwidth, 165, XtNheight, 485,
+	              165, 485, 0);
 #else
-	          XtNwidth, 116, XtNheight, 364,
+                      122, 364, 0);
 #endif
 #endif
 #ifdef XAW95
 #ifdef BIGTOOLICONS
-	          XtNwidth, 172, XtNheight, 503,
+                      172, 503, 0);
 #else
-	          XtNwidth, 123, XtNheight, 346,
+                      123, 346, 0);
 #endif
 #endif
 #ifdef XAWPLAIN		      
 #ifdef BIGTOOLICONS
-		  XtNwidth, 167, XtNheight, 502,
+                      167, 502, 0);
 #else
-	          XtNwidth, 118, XtNheight, 340,
+                      118, 340, 0);
 #endif
 #endif	      
-	         NULL);   
+  
     AddDestroyCallback(toplevel, (DestroyCallbackFunc) exitPaint, NULL);
+
     XtAddEventHandler(toplevel, StructureNotifyMask, False,
 		      (XtEventHandler) toolsResized, (XtPointer) vport);
+
+    XtAddEventHandler(vport, 
+                ButtonPressMask | ButtonReleaseMask |
+                KeyPressMask | KeyReleaseMask, False, 
+                (XtEventHandler) processEvent, (XtPointer)NULL);
+
+    XtAddEventHandler(toplevel, 
+                ButtonPressMask | ButtonReleaseMask |
+                KeyPressMask | KeyReleaseMask, False, 
+                (XtEventHandler) processEvent, (XtPointer)NULL);
 }
 
 Image *
