@@ -64,7 +64,9 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 
     save = Global.dashnumber;
     Global.dashnumber = 0;
-    SetCapAndJoin(w, l->gc, JoinRound);
+    SetCapAndJoin(w, l->gc,
+                  ((Global.cap)?Global.cap-1:CapButt),
+		  ((Global.join)?Global.join-1:JoinRound));
     Global.dashnumber = save;
 
     if (l->isDots) {
