@@ -61,6 +61,7 @@
 #include <unistd.h>
 #endif
 
+extern void openTempDir(char *buf);
 extern void pipe_command(char *command, 
   int input(char*, int), 
   void output(const char*, int),
@@ -176,14 +177,18 @@ typedef struct {
     Widget hplus, hpplus, hminus, hmminus, hcenter;
     Widget portraitlabel, portraittoggle;
     Widget landscapelabel, landscapetoggle;
+    Widget graylabel, graytoggle;
+    Widget compresslabel, compresstoggle;
+    Widget pdflabel, pdftoggle;
     Widget sizelabel, size;
     Widget positionlabel, position;
     Widget samplinglabel, sampling;
     Widget printlabel, printcmd, printtoggle;
-    Widget filelabel, filename, filetoggle;
+    Widget filelabel, filename, fileselect, filetoggle;
     Widget psviewlabel, psviewcmd;
     Widget printerlist, printentry;
     Widget printermenu;
+    Widget resultlabel, printresult;
     Widget formatChecks[XtNumber(formatMenu)];
 } PrintInfo;
 
@@ -200,7 +205,9 @@ typedef struct {
 
 typedef struct {
     float wpercent, hpercent;
-    int wbbox, hbbox, width, height, rx, ry, wsubdiv, hsubdiv, orient;
+    int wbbox, hbbox, width, height, rx, ry, wsubdiv, hsubdiv;
+    Boolean orient, eps, gray, compress, pdf;
+    char *title;
 } PageInfo;
 
 static PrintInfo *printinfo = NULL;
@@ -407,6 +414,12 @@ drawArrows(Widget w, PrintInfo * l, XEvent * event, Boolean * flg)
     XDrawLine(dpy, win, l->gc2, i, top-6, i-5, top+4);
 }
 
+static void 
+closeMenus(Widget w, PrintInfo * l, XEvent * event, Boolean * flg)
+{
+    PopdownMenusGlobal();
+}
+
 static void
 setPosition(PrintInfo *l)
 {
@@ -437,13 +450,26 @@ doWritePS(PrintInfo *info, char *filename)
 {
     Image * image;
     PageInfo pageinfo;
+    char *title;
 
     image = PixmapToImage((Widget)info->paintwidget, 
-			  info->paintwidget->paint.current, info->cmap);
+			  info->paintwidget->paint.current.pixmap, info->cmap);
 
     memcpy(&pageinfo, info, sizeof(PageInfo));
     pageinfo.wbbox = paper_sizes[info->papertype][0];
     pageinfo.hbbox = paper_sizes[info->papertype][1];
+    pageinfo.eps = False;
+
+    XtVaGetValues(info->graytoggle, XtNstate, &pageinfo.gray, NULL);
+    XtVaGetValues(info->compresstoggle, XtNstate, &pageinfo.compress, NULL);
+    XtVaGetValues(info->pdftoggle, XtNstate, &pageinfo.pdf, NULL);
+    XtVaGetValues(info->filename, XtNstring, &title, NULL);
+
+    if (title && *title) 
+        pageinfo.title = title;
+    else
+        pageinfo.title = filename;
+
     if ((pageinfo.orient = info->orient)) {
         int exch = pageinfo.hsubdiv;
 	pageinfo.hsubdiv = pageinfo.wsubdiv;
@@ -453,43 +479,109 @@ doWritePS(PrintInfo *info, char *filename)
     ImageDelete(image);
 }
 
+static void FlashString(char *str)
+{
+    int i;
+    for (i=1; i<=4; i++) {
+	XtVaSetValues(printinfo->printresult, XtNstring, str, NULL);
+        XFlush(XtDisplay(printinfo->printresult));
+        if (i==4) return;
+        usleep(300000);
+	XtVaSetValues(printinfo->printresult, XtNstring, "", NULL);
+        XFlush(XtDisplay(printinfo->printresult));
+        usleep(100000);
+    }
+}
+
 static void 
 printCallback(Widget w, XtPointer infoArg, XtPointer junk2)
 {
     PrintInfo *info = (PrintInfo *) infoArg;
     char cmd[512];
+    char *name;
+    Boolean dopdf;
+
+    PopdownMenusGlobal();
 
     if (info->output) {
-        char *ptr;
-        XtVaGetValues(printinfo->filename, XtNstring, &ptr, NULL);
-        doWritePS(info, ptr);
-	Notice(w, msgText[FILE_WRITTEN], ptr);
+        XtVaGetValues(printinfo->filename, XtNstring, &name, NULL);
+        XtVaGetValues(info->pdftoggle, XtNstate, &dopdf, NULL);
+
+        if (dopdf) {
+            doWritePS(info, info->tmpfile);
+            sprintf(cmd, "ps2pdf %s %s", info->tmpfile, name);
+            system(cmd);
+            unlink(info->tmpfile);
+	} else
+            doWritePS(info, name);
+        sprintf(cmd, msgText[FILE_WRITTEN], name);
     } else {
-        char *ptr;
-	XtVaGetValues(info->printcmd, XtNstring, &ptr, NULL);
+        char *name;
+	XtVaGetValues(info->printcmd, XtNstring, &name, NULL);
         doWritePS(info, info->tmpfile);
         sprintf(cmd, "( %s %s ; rm -f %s ) &", 
-            ptr, info->tmpfile, info->tmpfile);
+            name, info->tmpfile, info->tmpfile);
         system(cmd);
-	Notice(w, msgText[FILE_SENT_TO_PRINTER]);
+        sprintf(cmd, msgText[FILE_SENT_TO_PRINTER], name);
     }
+    FlashString(cmd);
+}
+
+static void 
+selectionOkCallback(Widget w, XtPointer infoArg, char* file)
+{
+    PrintInfo *info = (PrintInfo *) infoArg;
+    XtVaSetValues(info->filename, XtNstring, file, NULL);
+    PopdownMenusGlobal();
+}
+ 
+static void 
+fileselectCallback(Widget w, XtPointer infoArg, XtPointer junk2)
+{
+    PrintInfo *info = (PrintInfo *) infoArg;
+    char *name;
+    PopdownMenusGlobal();
+    XtVaGetValues(info->filename, XtNstring, &name, NULL);
+    XtVaSetValues(info->filetoggle, XtNstate, True, NULL);
+    GetFileName(GetToplevel(w), 3, name, 
+                (XtCallbackProc) selectionOkCallback, (XtPointer)infoArg);
 }
 
 static void 
 previewCallback(Widget w, XtPointer infoArg, XtPointer junk2)
 {
     PrintInfo *info = (PrintInfo *) infoArg;
-    char cmd[512];
+    char cmd[2048];
     char *ps_viewer = NULL;
+    char *name = NULL;
+    Boolean dopdf, dofile, bool;
 
+    PopdownMenusGlobal();
     XtVaGetValues(info->psviewcmd, XtNstring, &ps_viewer, NULL);
 
     if (!ps_viewer) return;
 
+    XtVaGetValues(info->filetoggle, XtNstate, &dofile, NULL);
+    XtVaGetValues(info->pdftoggle, XtNstate, &dopdf, NULL);
+    XtVaGetValues(info->filename, XtNstring, &name, NULL);
+
     doWritePS(info, info->tmpfile);
-    sprintf(cmd, "( %s %s ; rm -f %s ) &", 
-	    ps_viewer, info->tmpfile, info->tmpfile);
-    system(cmd);
+
+    if ((bool=dofile && name && *name)) {
+       if (dopdf) 
+          sprintf(cmd, "( ps2pdf %s %s ; %s %s ; rm -f %s ) &", 
+	          info->tmpfile, name, ps_viewer, name, info->tmpfile);
+       else
+          sprintf(cmd, "mv -f %s %s ; %s %s &", 
+	          info->tmpfile, name, ps_viewer, name);
+       system(cmd);
+    } else {
+       sprintf(cmd, "( %s %s ; rm -f %s ) &", 
+	       ps_viewer, info->tmpfile, info->tmpfile);
+       system(cmd);
+    }
+    sprintf(cmd, msgText[FILE_WRITTEN], (bool)? name:info->tmpfile);
+    FlashString(cmd);
 }
 
 static void 
@@ -516,6 +608,22 @@ closePrintCallback(Widget w, XtPointer infoArg, XtPointer junk2)
     info->shell = 0;
 }
 
+void
+setPageSize(int *w, int *h)
+{
+    int i, j;
+    if (!printinfo) {
+       *w = 595;
+       *h = 842;
+       return;
+    }
+    i = printinfo->papertype;
+    if (printinfo->orient) j = 1; else j = 0;
+    if (i<0 || i>=XtNumber(formatMenu)) i = 11;
+    *w = paper_sizes[i][j];
+    *h = paper_sizes[i][1-j];
+}
+
 static void
 findDrawSize(PrintInfo *l)
 {
@@ -579,6 +687,7 @@ formatCallback(Widget w, XtPointer larg, XtPointer junk)
     int i;
     char *str;
 
+    PopdownMenusGlobal();
     if (l->formatChecks[l->papertype] == w) return;
     MenuCheckItem(w, True);
 
@@ -696,7 +805,7 @@ switchCallback(Widget w, XtPointer infoArg, XtPointer junk2)
     char *name;
     struct stat buf;
     int i;
-    Boolean state;
+    Boolean state, statepdf;
 
     XtVaGetValues(w, XtNstate, &state, NULL);
 
@@ -718,16 +827,22 @@ switchCallback(Widget w, XtPointer infoArg, XtPointer junk2)
        XtVaSetValues(info->filetoggle, XtNstate, !state, NULL);
        info->output = !state;
     }
+    if (w == info->pdftoggle) {
+       w = info->filetoggle;
+       state = True;
+       XtVaSetValues(info->filetoggle, XtNstate, state, NULL);
+    }
     if (w == info->filetoggle) {
        XtVaSetValues(info->printtoggle, XtNstate, !state, NULL);
        info->output = state;
        if (state) {
           XtVaGetValues(info->filename, XtNstring, &name, NULL);
-	  if (*name=='\0') {
+	  if (!*name || !strncmp(name, "painting", 8)) {
 	     i = 0;
-	     name = malloc(20*sizeof(char));
+	     name = xmalloc(20*sizeof(char));
 	     incr:
-	     sprintf(name, "painting%d.ps", i);
+             XtVaGetValues(info->pdftoggle, XtNstate, &statepdf, NULL);
+             sprintf(name, "painting%d.%s", i, (statepdf)?"pdf":"ps");
 	     if (stat(name, &buf)!=-1) {
 	        ++i; 
 		if (i<1000) goto incr;
@@ -844,7 +959,6 @@ hmminusCallback(Widget w, XtPointer larg, XtPointer junk)
 void createPrintData() 
 {
     int fd;
-    char *tmp;
     char buf[256];
 
     printinfo = (PrintInfo *) XtMalloc(sizeof(PrintInfo));
@@ -853,11 +967,8 @@ void createPrintData()
     printinfo->output = False;
     printinfo->page = 0;
 
-    if ((tmp = getenv("TMPDIR")) == NULL)
-       tmp = P_tmpdir;
-
-    strncpy(buf, tmp, 220);
-    buf[220] = '\0';
+    openTempDir(buf);
+    buf[200] = '\0';
     strcat(buf, "/XPaint-XXXXXX");
     fd = mkstemp(buf);
     if (fd == -1) return;
@@ -896,6 +1007,7 @@ printpopupResized(Widget w, PrintInfo * l, XConfigureEvent * event,
 #endif
    
     wg = &l->wplus;
+
     for (i=0; i<5; i++) {
        XtVaGetValues(wg[i], XtNx, &x, NULL);
 #ifdef XAW3D       
@@ -906,8 +1018,8 @@ printpopupResized(Widget w, PrintInfo * l, XConfigureEvent * event,
     }
 
     wg = &l->hplus;
-    n = &l->printentry - wg;
-    for (i=0; i<n; i++) if (wg[i]) {
+    n = &l->printresult - wg;
+    for (i=0; i<=n; i++) if (wg[i]) {
        XtVaGetValues(wg[i], XtNx, &x, XtNy, &y, NULL);
        XtMoveWidget(wg[i], x+PAGEWIDTH-oldw, y);
     }
@@ -958,9 +1070,9 @@ PrintPopup(Widget w, XtPointer paintArg)
 
     if (canvasfile) {
         char *ptr;
-        canvasfile = malloc(strlen(canvasfile)+4);
+        canvasfile = xmalloc(strlen(canvasfile)+4);
 	strcpy(canvasfile, ((PaintWidget) paintArg)->paint.filename);
-	ptr = rindex(canvasfile, '.');
+	ptr = strrchr(canvasfile, '.');
 	if (ptr) 
            strcpy(ptr, ".ps");
 	else
@@ -996,6 +1108,9 @@ PrintPopup(Widget w, XtPointer paintArg)
 
     printinfo->form = XtVaCreateManagedWidget("form", formWidgetClass, printinfo->shell, NULL);
 
+    XtAddEventHandler(printinfo->form, ButtonPressMask,
+                  False, (XtEventHandler)closeMenus, (XtPointer) printinfo);
+
     printinfo->bar = MenuBarCreate(printinfo->form, XtNumber(printMenuBar), printMenuBar);
 
     printinfo->format = XtVaCreateManagedWidget("formattype",
@@ -1060,6 +1175,7 @@ PrintPopup(Widget w, XtPointer paintArg)
 	   "#override\n\t<BtnDown>,<BtnUp>,<BtnMotion>: \n");
     }
     XtVaSetValues(printinfo->page, XtNtranslations, paint_trans, NULL);
+    XtVaSetValues(printinfo->page, XtNmenuwidgets, NULL, NULL);
 
     if (pageCursor == None)
         pageCursor = XCreateFontCursor(dpy, XC_crosshair);
@@ -1145,23 +1261,24 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       labelWidgetClass, printinfo->form,
 				       XtNborderWidth, 0,
 				       XtNfromHoriz, printinfo->hpplus,
-				       XtNhorizDistance, 30,
 				       XtNfromVert, printinfo->bar,
+				       XtNhorizDistance, 30,
 				       XtNvertDistance, 6,
 				       NULL);
 
     printinfo->portraittoggle = XtVaCreateManagedWidget("portraittoggle",
 				       toggleWidgetClass, printinfo->form,
                                        XtNstate, !printinfo->orient,
-				       XtNfromHoriz, printinfo->hpplus,
+				       XtNfromHoriz, printinfo->portraitlabel,
 				       XtNfromVert, printinfo->bar,
+				       XtNhorizDistance, 12,
 				       XtNvertDistance, 6,
 				       NULL);
 
     printinfo->landscapelabel = XtVaCreateManagedWidget("landscapelabel",
 				       labelWidgetClass, printinfo->form,
 				       XtNborderWidth, 0,
-				       XtNfromHoriz, printinfo->hpplus,
+				       XtNfromHoriz, printinfo->portraittoggle,
 				       XtNfromVert, printinfo->bar,
 				       XtNvertDistance, 6,
 				       NULL);
@@ -1169,9 +1286,63 @@ PrintPopup(Widget w, XtPointer paintArg)
     printinfo->landscapetoggle = XtVaCreateManagedWidget("landscapetoggle",
 				       toggleWidgetClass, printinfo->form,
                                        XtNstate, printinfo->orient,
-				       XtNfromHoriz, printinfo->hpplus,
+				       XtNfromHoriz, printinfo->landscapelabel,
                                        XtNradioGroup,printinfo->portraittoggle,
 				       XtNfromVert, printinfo->bar,
+				       XtNhorizDistance, 12,
+				       XtNvertDistance, 6,
+				       NULL);
+
+    printinfo->graylabel = XtVaCreateManagedWidget("graylabel",
+				       labelWidgetClass, printinfo->form,
+				       XtNborderWidth, 0,
+				       XtNfromHoriz, printinfo->hpplus,
+				       XtNfromVert, printinfo->portraittoggle,
+				       XtNhorizDistance, 30,
+				       XtNvertDistance, 6,
+				       NULL);
+
+    printinfo->graytoggle = XtVaCreateManagedWidget("graytoggle",
+				       toggleWidgetClass, printinfo->form,
+                                       XtNstate, 0,
+				       XtNfromHoriz, printinfo->graylabel,
+				       XtNfromVert, printinfo->portraittoggle,
+				       XtNhorizDistance, 12,
+				       XtNvertDistance, 6,
+				       NULL);
+
+    printinfo->compresslabel = XtVaCreateManagedWidget("compresslabel",
+				       labelWidgetClass, printinfo->form,
+				       XtNborderWidth, 0,
+				       XtNfromHoriz, printinfo->graytoggle,
+				       XtNfromVert, printinfo->portraittoggle,
+				       XtNvertDistance, 6,
+				       NULL);
+
+    printinfo->compresstoggle = XtVaCreateManagedWidget("compresstoggle",
+				       toggleWidgetClass, printinfo->form,
+                                       XtNstate, 0,
+				       XtNfromHoriz, printinfo->compresslabel,
+				       XtNfromVert, printinfo->portraittoggle,
+				       XtNhorizDistance, 12,
+				       XtNvertDistance, 6,
+				       XtNstate, True,
+				       NULL);
+
+    printinfo->pdflabel = XtVaCreateManagedWidget("pdflabel",
+				       labelWidgetClass, printinfo->form,
+				       XtNborderWidth, 0,
+				       XtNfromHoriz, printinfo->compresstoggle,
+				       XtNfromVert, printinfo->portraittoggle,
+				       XtNvertDistance, 6,
+				       NULL);
+
+    printinfo->pdftoggle = XtVaCreateManagedWidget("pdftoggle",
+				       toggleWidgetClass, printinfo->form,
+                                       XtNstate, 0,
+				       XtNfromHoriz, printinfo->pdflabel,
+				       XtNfromVert, printinfo->portraittoggle,
+				       XtNhorizDistance, 12,
 				       XtNvertDistance, 6,
 				       NULL);
 
@@ -1179,7 +1350,7 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       labelWidgetClass, printinfo->form,
 				       XtNborderWidth, 0,
 				       XtNfromHoriz, printinfo->hpplus,
-				       XtNfromVert, printinfo->landscapetoggle,
+				       XtNfromVert, printinfo->graytoggle,
 				       XtNhorizDistance, 30,
 				       XtNvertDistance, 16,
 				       NULL);
@@ -1191,7 +1362,7 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNwrap, XawtextWrapNever,
 				       XtNresize, XawtextResizeWidth,
 				       XtNfromHoriz, printinfo->hpplus,
-				       XtNfromVert, printinfo->landscapetoggle,
+				       XtNfromVert, printinfo->graytoggle,
 				       XtNwidth, 150,
 				       XtNvertDistance, 16,
 				       NULL);
@@ -1277,7 +1448,7 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNheight, 16,
 				       XtNfromVert, printinfo->sampling,
 				       XtNfromHoriz, printinfo->printcmd,
-				       XtNvertDistance, 20,
+				       XtNvertDistance, 21,
 #ifdef XAW3D
                                        XtNborderWidth, 0,							    
 #endif							    
@@ -1285,7 +1456,7 @@ PrintPopup(Widget w, XtPointer paintArg)
           printinfo->printermenu = XtVaCreatePopupShell("printermenu",
 				       simpleMenuWidgetClass, 
                                        printinfo->printerlist,
-				       NULL, 0);
+				       NULL);
           for (i = 0; i<=num_printers; i++) {
 	        sprintf(wname, "%d", i);
 	        printinfo->printentry = 
@@ -1305,7 +1476,7 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNfromVert, printinfo->sampling,
 				       XtNfromHoriz, printinfo->printcmd,
 				       XtNvertDistance, 
-					  (printinfo->printerlist==0)? 32: 40,
+					  (printinfo->printerlist==0)? 32: 41,
 				       NULL);
    
     printinfo->filelabel = XtVaCreateManagedWidget("filelabel",
@@ -1331,13 +1502,23 @@ PrintPopup(Widget w, XtPointer paintArg)
 	free(canvasfile);
     }
 
+    printinfo->fileselect = XtVaCreateManagedWidget("fileselect",
+				       commandWidgetClass, printinfo->form,
+				       XtNlabel, "?",
+				       XtNwidth, 17,
+				       XtNheight, 16,
+				       XtNfromVert, printinfo->printcmd,
+				       XtNfromHoriz, printinfo->filename,
+                                       XtNvertDistance, 5,
+				       NULL);
+
     printinfo->filetoggle = XtVaCreateManagedWidget("filetoggle",
 				       toggleWidgetClass, printinfo->form,
                                        XtNstate, printinfo->output,
-				       XtNfromVert, printinfo->printcmd,
+				       XtNfromVert, printinfo->fileselect,
 				       XtNfromHoriz, printinfo->filename,
 				       XtNradioGroup, printinfo->printtoggle,
-                                       XtNvertDistance, 14,
+                                       XtNvertDistance, 5,
 				       NULL);
 
     printinfo->psviewlabel = XtVaCreateManagedWidget("psviewlabel",
@@ -1359,6 +1540,26 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNwidth, 150, XtNheight, 40,
 				       NULL);
    
+    printinfo->resultlabel = XtVaCreateManagedWidget("resultlabel",
+				       labelWidgetClass, printinfo->form,
+				       XtNborderWidth, 0,
+				       XtNfromVert, printinfo->psviewcmd,
+				       XtNfromHoriz, printinfo->hpplus,
+				       XtNvertDistance, 20,
+				       XtNhorizDistance, 30,
+				       NULL);
+
+    printinfo->printresult = XtVaCreateManagedWidget("printresult",
+				       asciiTextWidgetClass, printinfo->form,
+				       XtNtranslations, translations,
+ 				       XtNeditType, XawtextEdit,
+				       XtNwrap, XawtextWrapNever,
+				       XtNscrollHorizontal, True,
+				       XtNfromVert, printinfo->psviewcmd,
+				       XtNfromHoriz, printinfo->resultlabel,
+				       XtNvertDistance, 20,
+				       NULL);
+
     if (psviewcmd_str) 
        XtVaSetValues(printinfo->psviewcmd, XtNstring, psviewcmd_str, NULL);
 
@@ -1368,7 +1569,8 @@ PrintPopup(Widget w, XtPointer paintArg)
     XtAddEventHandler(printinfo->form, ExposureMask,
 		  False, (XtEventHandler) drawArrows, (XtPointer) printinfo);
 
-    XtAddEventHandler(printinfo->page, ButtonPressMask | ButtonMotionMask | ExposureMask,
+    XtAddEventHandler(printinfo->page,
+                  ButtonPressMask | ButtonMotionMask | ExposureMask,
 		  False, (XtEventHandler)setDrawingPosition, (XtPointer) printinfo);
 
     XtAddCallback(XtNameToWidget(printinfo->bar, "file.fileMenu.print"),
@@ -1412,6 +1614,8 @@ PrintPopup(Widget w, XtPointer paintArg)
 		  (XtCallbackProc) switchCallback, (XtPointer) printinfo);
     XtAddCallback(printinfo->landscapetoggle, XtNcallback,
 		  (XtCallbackProc) switchCallback, (XtPointer) printinfo);
+    XtAddCallback(printinfo->pdftoggle, XtNcallback,
+		  (XtCallbackProc) switchCallback, (XtPointer) printinfo);
     XtAddCallback(printinfo->printtoggle, XtNcallback,
 		  (XtCallbackProc) switchCallback, (XtPointer) printinfo);
     XtAddCallback(printinfo->filetoggle, XtNcallback,
@@ -1423,6 +1627,8 @@ PrintPopup(Widget w, XtPointer paintArg)
 		  (XtCallbackProc) printCallback, (XtPointer) printinfo);
     XtAddCallback(printinfo->preview, XtNcallback,
 		  (XtCallbackProc) previewCallback, (XtPointer) printinfo);
+    XtAddCallback(printinfo->fileselect, XtNcallback,
+		  (XtCallbackProc) fileselectCallback, (XtPointer) printinfo);
     XtAddCallback(printinfo->cancel, XtNcallback,
 		  (XtCallbackProc) closePrintCallback, (XtPointer) printinfo);
     AddDestroyCallback(printinfo->shell, (DestroyCallbackFunc) closePrintCallback, printinfo);
@@ -1512,7 +1718,7 @@ viewExternCallback(Widget w, XtPointer infoArg, XtPointer junk2)
        i++;
     }
     del = 1;
-    sep = rindex(info->cmd, ' ');
+    sep = strrchr(info->cmd, ' ');
     if (sep) {
         ++sep;
 	sprintf(cmd, ".%s", suffix);
@@ -1546,7 +1752,7 @@ viewExternCallback(Widget w, XtPointer infoArg, XtPointer junk2)
 
     StateSetBusy(True);    
     image = PixmapToImage((Widget)info->paintwidget, 
-        info->paintwidget->paint.current, info->cmap);
+        info->paintwidget->paint.current.pixmap, info->cmap);
     if (!image) {
         StateSetBusy(False);
 	free(file);
