@@ -15,6 +15,10 @@ typedef char *String;
 #include <image.h>
 #include "rwTable.h"
 
+#include <X11/Xlib.h>
+#include <X11/Intrinsic.h>
+#include "xpaint.h"
+
 #ifndef TRUE
 #  define TRUE 1
 #  define FALSE 0
@@ -28,6 +32,11 @@ typedef char *String;
 #  endif
 #endif
 
+extern int file_isSpecialImage;
+extern int file_transparent;
+
+extern void *xmalloc(size_t n);
+
 typedef struct _jmpbuf_wrapper {
   jmp_buf jmpbuf;
 } jmpbuf_wrapper;
@@ -64,8 +73,11 @@ ReadPNG(char *file)
     png_infop     info_ptr;
     png_uint_32   width, height;
     png_colorp    palette;
+    png_bytep     trans = NULL;
+    png_color_16p palette16;
+    int           num_trans;
     int           bit_depth, color_type, interlace_type, num_palette, rowbytes;
-    int           i, hasAlpha;
+    int           i, j, hasAlpha;
     int           level /* , npasses */ ;
     Image         *image = NULL;
 
@@ -85,6 +97,7 @@ ReadPNG(char *file)
     }
 
     info_ptr = png_create_info_struct(png_ptr);
+
     if (!info_ptr) {
         RWSetMsg("Error allocating PNG info_ptr memory");
         png_destroy_read_struct(&png_ptr, NULL, NULL);
@@ -115,7 +128,7 @@ ReadPNG(char *file)
 
         case PNG_COLOR_TYPE_PALETTE:
             Trace((stderr, "GRR ReadPNG:  PNG_COLOR_TYPE_PALETTE\n"));
-            if (!png_get_PLTE(png_ptr, info_ptr, &palette, &num_palette)) {
+	    if (!png_get_PLTE(png_ptr, info_ptr, &palette, &num_palette)) {
                 RWSetMsg(msgText[ERROR_PLTE_CHUNK_NOT_FOUND_IN_PALETTE_IMAGE]);
                 fprintf(stderr, "%s\n",
                   msgText[ERROR_PLTE_CHUNK_NOT_FOUND_IN_PALETTE_IMAGE]);
@@ -125,9 +138,16 @@ ReadPNG(char *file)
                 return NULL;
             }
             image = ImageNewCmap(width, height, num_palette);
+            png_get_tRNS(png_ptr, info_ptr, &trans, &num_trans, &palette16);
+            if (num_trans && trans) {
+                file_isSpecialImage = 1;
+                file_transparent = 1;
+                ImageMakeMask(image);   /* alpha is 1 byte deep */
+	    }
             for (i = 0;  i < num_palette;  ++i)
                 ImageSetCmap(image, i, palette[i].red, palette[i].green,
-                  palette[i].blue);
+                palette[i].blue);
+
             /* GRR:  still need to get image data into `image' */
             break;
 
@@ -193,8 +213,10 @@ ReadPNG(char *file)
             }
             /* need to split alpha and RGB/gray the hard way, sigh */
             hasAlpha = TRUE;
+            file_isSpecialImage = 1;
+            file_transparent = 1;
             image = ImageNew(width, height);
-            ImageMakeMask(image);   /* maskData is always 1 byte deep */
+            ImageMakeMask(image);   /* alpha is 1 byte deep */
             break;
 
         case PNG_COLOR_TYPE_GRAY_ALPHA:
@@ -207,13 +229,15 @@ ReadPNG(char *file)
                 png_set_strip_16(png_ptr);
             }
             hasAlpha = TRUE;
+            file_isSpecialImage = 1;
+            file_transparent = 1;
             image = ImageNewGrey(width, height);
-            ImageMakeMask(image);   /* maskData is always 1 byte deep */
+            ImageMakeMask(image);   /* alpha is 1 byte deep */
             break;
 
         default:
             fprintf(stderr, "ReadPNG:  %s (%d)\n", 
-              msgText[UNKNOWN_PNG_IMAGE_TYPE], color_type);
+                msgText[UNKNOWN_PNG_IMAGE_TYPE], color_type);
             fflush(stderr);
             RWSetMsg(msgText[UNKNOWN_PNG_IMAGE_TYPE]);
             png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
@@ -234,7 +258,7 @@ ReadPNG(char *file)
     if (hasAlpha) {
         png_bytep *row_pointers, png_data;
 
-        row_pointers = (png_bytep *)malloc(height * sizeof(png_bytep));
+        row_pointers = (png_bytep *)xmalloc(height * sizeof(png_bytep));
         if (!row_pointers) {
             RWSetMsg(msgText[READPNG_CANT_ALLOCATE_MEMORY_FOR_ROW_POINTERS_1]);
             fprintf(stderr, "%s\n",
@@ -250,11 +274,11 @@ ReadPNG(char *file)
          * no easy way around it:  libpng returns the image and alpha channel
          * interspersed, and interlaced alpha images just make matters worse
          */
-        png_data = (png_bytep)malloc(height*rowbytes);
+        png_data = (png_bytep)xmalloc(height*rowbytes);
         if (!png_data) {
             RWSetMsg(
               msgText[UNABLE_TO_ALLOCATE_TEMPORARY_STORAGE_FOR_ALPHA_IMAGE]);
-            fprintf(stderr, msgText[READPNG_UNABLE_TO_MALLOC_PNG_DATA]);
+            fprintf(stderr, "%s", msgText[READPNG_UNABLE_TO_MALLOC_PNG_DATA]);
             fflush(stderr);
             free(row_pointers);
             ImageDelete(image);
@@ -270,29 +294,28 @@ ReadPNG(char *file)
         png_read_image(png_ptr, row_pointers);
 
         if (color_type == PNG_COLOR_TYPE_RGB_ALPHA) {
-            png_bytep png=png_data, rgb=image->data, alpha=image->maskData;
-
+            png_bytep png=png_data, rgb=image->data, alpha=image->alpha;
             for (i = 0;  i < height*width;  ++i) {
                 *rgb++   = *png++;
                 *rgb++   = *png++;
                 *rgb++   = *png++;
-                *alpha++ = *png++;  /* GRR:  invert?  doesn't seem to be used */
+                *alpha++ = *png++;
             }
         } else {  /* PNG_COLOR_TYPE_GRAY_ALPHA */
-            png_bytep png=png_data, gray=image->data, alpha=image->maskData;
+            png_bytep png=png_data, gray=image->data, alpha=image->alpha;
 
             for (i = 0;  i < height*width;  ++i) {
                 *gray++  = *png++;
-                *alpha++ = *png++;  /* GRR:  invert? */
+                *alpha++ = *png++;
             }
         }
         free(png_data);   /* whew */
         free(row_pointers);
 
-    } else {  /* no alpha channel */
+    } else {  /* no alpha channel, except perhaps from palette */
         png_bytep *row_pointers;
 
-        row_pointers = (png_bytep *)malloc(height * sizeof(png_bytep));
+        row_pointers = (png_bytep *)xmalloc(height * sizeof(png_bytep));
         if (!row_pointers) {
             RWSetMsg(msgText[READPNG_CANT_ALLOCATE_MEMORY_FOR_ROW_POINTERS_2]);
             fprintf(stderr, "%s\n",
@@ -308,6 +331,16 @@ ReadPNG(char *file)
             row_pointers[i] = (png_bytep)image->data + i*rowbytes;
 
         png_read_image(png_ptr, row_pointers);
+
+        if (image->alpha) {
+	    if (trans) {
+                for (i = 0;  i < width*height;  ++i)
+	            image->alpha[i] = trans[image->data[i]];
+            } else {
+	        free(image->alpha);
+                image->alpha = NULL;
+	    }
+	}
         free(row_pointers);
 
     } /* end if (hasAlpha) */
@@ -336,11 +369,12 @@ ReadPNG(char *file)
 static int
 WritePNG(char *file, Image *image, int interlace_type)
 {
-    int           i;
-    int           bit_depth=0, color_type=0, num_palette;
+    int           i, j;
+    int           bit_depth=0, color_type=0, num_palette, rowbytes;
     png_colorp    palette;
     png_structp   png_ptr;
     png_infop     info_ptr;
+    png_bytep     *row_pointers;
     FILE          *fp;
 
 
@@ -381,7 +415,7 @@ WritePNG(char *file, Image *image, int interlace_type)
     bit_depth = -1;
 
     if (image->isBW) {
-        if (image->maskData) {
+        if (image->alpha) {
             color_type = PNG_COLOR_TYPE_GRAY_ALPHA;
             bit_depth = 8;   /* promote to full grayscale */
         } else {
@@ -391,7 +425,7 @@ WritePNG(char *file, Image *image, int interlace_type)
         Trace((stderr, "GRR WritePNG:  B/W, bit_depth = %d\n", bit_depth));
 
     } else if (image->isGrey) {
-        color_type = image->maskData? PNG_COLOR_TYPE_GRAY_ALPHA :
+        color_type = image->alpha? PNG_COLOR_TYPE_GRAY_ALPHA :
                                       PNG_COLOR_TYPE_GRAY;
         if (image->cmapPacked)
             bit_depth = 8;
@@ -418,13 +452,13 @@ WritePNG(char *file, Image *image, int interlace_type)
         Image *cmapImage = NULL;
 
         /* try compressing image to palette mode, but don't force if too big */
-        if (!image->maskData)   /* can't store alpha mask with palette image */
+        if (!image->alpha)   /* can't store alpha mask with palette image */
             cmapImage = ImageCompress(image, 256, 1);
 
         if (cmapImage) {
             image = cmapImage;  /* original was deleted in ImageCompress() */
         } else {
-            color_type = image->maskData? PNG_COLOR_TYPE_RGB_ALPHA :
+            color_type = image->alpha? PNG_COLOR_TYPE_RGB_ALPHA :
                                           PNG_COLOR_TYPE_RGB;
             bit_depth = 8;
             Trace((stderr, "GRR WritePNG:  RGB, bit_depth = 8\n"));
@@ -436,8 +470,8 @@ WritePNG(char *file, Image *image, int interlace_type)
      */
     if (image->scale == 1) {
         color_type = PNG_COLOR_TYPE_PALETTE;
-        if (image->maskData) {
-            fprintf(stderr,
+        if (image->alpha) {
+            fprintf(stderr, "%s", 
               msgText[WRITEPNG_CANT_USE_ALPHA_MASK_WITH_COLORMAPPED_IMAGE]);
             fflush(stderr);
         }
@@ -551,31 +585,48 @@ WritePNG(char *file, Image *image, int interlace_type)
     png_set_packing(png_ptr);   /* squish multiple pixels into each byte */
 /*  png_set_compression_level(png_ptr, Z_BEST_COMPRESSION);  */
 
-    if (image->maskData) {
+#if 0
+    if (image->alpha) {
         /* alpha channel version */
-        fprintf(stderr, msgText[WRITEPNG_SORRY_CANT_WRITE_ALPHA_IMAGES_YET]);
+        fprintf(stderr, "%s", msgText[WRITEPNG_SORRY_CANT_WRITE_ALPHA_IMAGES_YET]);
         fflush(stderr);
         png_destroy_write_struct(&png_ptr, &info_ptr);
         fclose(fp);
         return 1;
 
+    }
+#endif
+    rowbytes = (image->scale+((image->alpha)?1:0)) * image->width;
+    row_pointers = (png_bytep *)xmalloc(image->height * sizeof(png_bytep));
+    if (!row_pointers) {
+        RWSetMsg(msgText[WRITEPNG_CANT_ALLOCATE_MEMORY_FOR_ROW_POINTERS]);
+        fprintf(stderr, "%s\n",
+                msgText[WRITEPNG_CANT_ALLOCATE_MEMORY_FOR_ROW_POINTERS]);
+        fflush(stderr);
+        png_destroy_write_struct(&png_ptr, &info_ptr);
+        fclose(fp);
+        return 1;
+    }
+
+    if (image->alpha) {
+        unsigned char *png, *data, *alpha;
+        png = (unsigned char *)
+	          XtMalloc(rowbytes*image->height);
+        data = image->data;
+        alpha = image->alpha;
+        for (j = 0;  j < image->height;  ++j) {
+	    row_pointers[j] = (png_bytep)png;
+            for (i = 0;  i < image->width;  ++i) {
+	        memcpy(png, data, image->scale);
+                png += image->scale;
+                data += image->scale;
+		*png++ = *alpha++;
+	    }
+	}
     } else {
-        int rowbytes = image->scale * image->width;
-        png_bytep *row_pointers;
-
-        row_pointers = (png_bytep *)malloc(image->height * sizeof(png_bytep));
-        if (!row_pointers) {
-            RWSetMsg(msgText[WRITEPNG_CANT_ALLOCATE_MEMORY_FOR_ROW_POINTERS]);
-            fprintf(stderr, "%s\n",
-              msgText[WRITEPNG_CANT_ALLOCATE_MEMORY_FOR_ROW_POINTERS]);
-            fflush(stderr);
-            png_destroy_write_struct(&png_ptr, &info_ptr);
-            fclose(fp);
-            return 1;
-        }
-
-        for (i = 0;  i < image->height;  ++i)
-            row_pointers[i] = (png_bytep)image->data + i*rowbytes;
+        for (j = 0;  j < image->height;  ++j)
+            row_pointers[j] = (png_bytep)image->data + j*rowbytes;
+    }
 
 #ifdef XPAINT_PNG_DEBUG___disabled_for_now
         {
@@ -592,13 +643,10 @@ WritePNG(char *file, Image *image, int interlace_type)
             Trace((stderr, "\nGRR WritePNG:  done with row %d.\n", rownum));
         }
 #endif
-
-        png_write_image(png_ptr, row_pointers);
-        free(row_pointers);
-    }
-
+    png_write_image(png_ptr, row_pointers);
+    if (image->alpha) free(row_pointers[0]);
+    free(row_pointers);
     png_write_end(png_ptr, NULL);
-
     png_destroy_write_struct(&png_ptr, &info_ptr);
     fclose(fp);
 
@@ -643,7 +691,7 @@ xpaint_png_error_handler (png_structp png_ptr, png_const_charp msg)
 
     jmpbuf_ptr = png_get_error_ptr(png_ptr);
     if (jmpbuf_ptr == NULL) {         /* we are completely hosed now */
-        fprintf(stderr, msgText[XPAINT_EXTREMELY_FATAL_LIBPNG_ERROR]);
+        fprintf(stderr, "%s", msgText[XPAINT_EXTREMELY_FATAL_LIBPNG_ERROR]);
         fflush(stderr);
         exit(99);
     }
