@@ -15,21 +15,32 @@
 /* $Id: readWriteXPM.c,v 1.21 2005/03/20 20:15:34 demailly Exp $ */
 
 #include <stdio.h>
+
 #include <X11/Intrinsic.h>
-#include <xpm.h>
+#include <X11/xpm.h>
+
 #include "xpaint.h"
 #include "image.h"
 #include "rwTable.h"
 
+extern int file_isSpecialImage;
+extern int file_transparent;
+
 int WriteXPM(char *file, Image * image)
 {
     XpmAttributes attr;
+    Display *dpy = Global.display;
 
+    if (!image) return 1;
+    if (!image->sourcePixmap) return 1;
     attr.valuemask = XpmColormap;
-    attr.colormap = (Colormap) image->sourceColormap;
+    if (image->sourceColormap)
+        attr.colormap = (Colormap) image->sourceColormap;
+    else
+        attr.colormap = DefaultColormap(dpy, DefaultScreen(dpy));
 
-    return XpmWriteFileFromPixmap(Global.display, file,
-	    image->sourcePixmap, image->sourceMask, &attr) != XpmSuccess;
+    return (XpmWriteFileFromPixmap(Global.display, file,
+	      image->sourcePixmap, image->sourceMask, &attr) != XpmSuccess);
 }
 
 int TestXPM(char *file)
@@ -58,21 +69,37 @@ int TestXPM(char *file)
     return ret;
 }
 
-Image *
- ReadXPM(char *file)
+Image * ReadXPM(char *file)
 {
     Display *dpy = Global.display;
-    XImage *xim, *mim;
+    XImage *xim, *mim = NULL;
     int x, y, i;
-    Image *image;
+    Image *image, *input;
     XpmAttributes attr;
+    XpmColorSymbol bg;
+    XColor bgcolor;
     XColor *col;
-    unsigned char *ucp, *ump = NULL;
+    unsigned char c, r, g, b, *ucp, *ump = NULL, *rgb;
     unsigned short *usp;
     Colormap cmap = DefaultColormap(dpy, DefaultScreen(dpy));
     int status;
 
-    attr.valuemask = XpmReturnPixels;
+    r = (Global.background>>16)&255;
+    g = (Global.background>>8)&255;
+    b = (Global.background)&255;
+    bgcolor.flags = DoRed | DoGreen | DoBlue;
+    bgcolor.red = r*257;
+    bgcolor.green = g*257;
+    bgcolor.blue = b*257;
+    XAllocColor(dpy, cmap, &bgcolor);
+    attr.valuemask = XpmColormap | XpmReturnPixels | XpmColorSymbols;
+    attr.colormap =  cmap;
+    attr.numsymbols = 1;
+    attr.colorsymbols = &bg;
+    bg.pixel = bgcolor.pixel;
+    bg.name = NULL;
+    bg.value = "None";
+    
     if ((status = XpmReadFileToImage(dpy, file, &xim, &mim, &attr)) != XpmSuccess) {
 	switch (status) {
 	case XpmColorError:
@@ -97,11 +124,8 @@ Image *
 	XpmFreeAttributes(&attr);
 	return NULL;
     }
+
     image = ImageNewCmap(attr.width, attr.height, attr.npixels);
-    if (mim != NULL) {
-	ImageMakeMask(image);
-	ump = image->maskData;
-    }
     ucp = (unsigned char *) image->data;
     usp = (unsigned short *) image->data;
 
@@ -111,8 +135,16 @@ Image *
 	col[i].flags = DoRed | DoGreen | DoBlue;
     }
     XQueryColors(dpy, cmap, col, attr.npixels);
+
+    if (mim != NULL) {
+        file_isSpecialImage = 1;
+        file_transparent = 1;
+	ImageMakeMask(image);
+	ump = image->alpha;
+    }
+
     for (i = 0; i < attr.npixels; i++)
-	ImageSetCmap(image, i, col[i].red >> 8, col[i].green >> 8, col[i].blue >> 8);
+	ImageSetCmap(image, i, col[i].red>>8, col[i].green>>8, col[i].blue>>8);
 
     for (y = 0; y < xim->height; y++) {
 	for (x = 0; x < xim->width; x++) {
@@ -121,7 +153,7 @@ Image *
 	    if (mim != NULL) {
 		Pixel f = XGetPixel(mim, x, y);
 
-		if (!(*ump++ = (Boolean) f)) {
+		if (!(*ump++ = (f?255:0))) {
 		    if (attr.npixels > 256)
 			*usp++ = 0;
 		    else
@@ -140,6 +172,36 @@ Image *
 	}
     }
 
+    if (!mim) goto finish;
+
+    input = image;
+    image = ImageNew(input->width, input->height);
+
+    if (attr.npixels <= 256) {
+        rgb = image->data;
+        ucp = (unsigned char *) input->data;
+        for (y = 0; y < image->height; y++)
+	    for (x = 0; x < image->width; x++) {
+	        *rgb++ = input->cmapData[3*(*ucp)];
+	        *rgb++ = input->cmapData[3*(*ucp)+1];
+	        *rgb++ = input->cmapData[3*(*ucp++)+2];
+	    }
+    } else {
+        rgb = image->data;
+        usp = (unsigned short *) input->data;
+        for (y = 0; y < image->height; y++)
+	    for (x = 0; x < image->width; x++) {
+	        *rgb++ = input->cmapData[3*(*usp)];
+	        *rgb++ = input->cmapData[3*(*usp)+1];
+	        *rgb++ = input->cmapData[3*(*usp++)+2];
+	    }
+    }
+
+    image->alpha = input->alpha;
+    input->alpha = NULL;
+    ImageDelete(input);
+
+ finish:
     XtFree((XtPointer) col);
     XDestroyImage(xim);
     XpmFreeAttributes(&attr);
