@@ -56,6 +56,7 @@ static char RWtableMsg[512];
 
 extern Image *ReadBMP(char *);
 extern Image *ReadXBM(char *);
+extern Image *ReadLXP(char *);
 extern Image *ReadXWD(char *);
 extern Image *ReadPNM(char *);
 extern Image *ReadPNG(char *);
@@ -66,9 +67,10 @@ extern Image *ReadTIFF(char *);
 extern Image *ReadJPEG(char *);
 extern Image *ReadICO(char *);
 extern Image *ReadScriptC(char *);
-extern Image *ReadPS(char *);
+extern Image *ReadPS_(char *);
 extern int WriteBMP(char *, Image *);
 extern int WriteXBM(char *, Image *);
+extern int WriteLXP(char *, Image *);
 extern int WriteXWD(char *, Image *);
 extern int WritePNM(char *, Image *);
 extern int WriteXPM(char *, Image *);
@@ -84,6 +86,7 @@ extern int WritePS(char *, Image *);
 extern int TestBMP(char *);
 extern int TestICO(char *);
 extern int TestPNM(char *);
+extern int TestLXP(char *);
 extern int TestXWD(char *);
 extern int TestXBM(char *);
 extern int TestXPM(char *);
@@ -93,8 +96,7 @@ extern int TestSGI(char *);
 extern int TestTIFF(char *);
 extern int TestJPEG(char *);
 extern int TestScriptC(char *);
-extern int TestPS(char *);
-extern int TestPDF(char *);
+extern int TestPS_(char *);
 extern int TestTEX(char *);
 
 #define DEF_READ_ENTRY	0
@@ -118,15 +120,16 @@ static ImageTypes RWtable[] =
     {"BMP_Format",       ReadBMP,   WriteBMP,  TestBMP },
     {"ICO_Format",       ReadICO,   WriteICO,  TestICO },
     {"PPM_Format",       ReadPNM,   WritePNM,  TestPNM },
-    {"PS_Format",        ReadPS,    WritePS,   TestPS  },
-    {"PDF_Format",       ReadPS,    WritePDF,  TestPS  },
-    {"TEX_Format",       ReadPS,        NULL,  TestPS  },
+    {"PS_Format",        ReadPS_,   WritePS,   TestPS_ },
+    {"PDF_Format",       ReadPS_,   WritePDF,  TestPS_ },
+    {"TEX_Format",       ReadPS_,   NULL,      TestPS_ },
 #ifdef HAVE_SGI
     {"SGI_Format",       ReadSGI,   WriteSGI,  TestSGI },
 #endif
     {"XBM_Format",       ReadXBM,   WriteXBM,  TestXBM },
     {"XPM_Format",       ReadXPM,   WriteXPM,  TestXPM },
     {"XWD_Format",       ReadXWD,   WriteXWD,  TestXWD },
+    {"LXP_Format",       ReadLXP,   WriteLXP,  TestLXP },
     {"CSC_Format",       ReadScriptC,   NULL,  TestScriptC }
 };
 
@@ -139,6 +142,27 @@ static char *writeList[NUMBER + 1];
 **  Special reader that uses the above information.
  */
 static char *usedMagicReader = NULL;
+int file_isSpecialImage;
+int file_numpages;
+int file_force = 1;
+int file_bbox = 1;
+int file_transparent = 0;
+
+RWwriteFunc
+RWtableGetWriterFromSuffix(char *suffix)
+{
+    int i;
+    if (!suffix || !*suffix) return WritePNGn;
+    if (!strcasecmp(suffix, "C")) suffix = "CSC";
+    if (!strcasecmp(suffix, "JPG")) suffix = "JPEG";
+    for (i = 1; i < NUMBER; i++) {
+        if (!strncasecmp(RWtable[i].name, suffix, strlen(suffix)) &&
+            RWtable[i].write)
+	    return RWtable[i].write;
+    }
+    /* default to WritePNG otherwise */
+    return WritePNGn;
+}
 
 static Image *
 readMagic(char *file)
@@ -146,12 +170,15 @@ readMagic(char *file)
     int i;
 
     errno = 0;
+    file_isSpecialImage = 0;
+    file_transparent = 0;
+    file_numpages = 1;
 
     for (i = 0; i < NUMBER; i++) {
 	if (RWtable[i].read == NULL || RWtable[i].test == NULL)
 	    continue;
 	if (!RWtable[i].test(file))
-	    continue;
+	    continue; 
         usedMagicReader = RWtable[i].name;
 	return RWtable[i].read(file);
     }
@@ -251,6 +278,7 @@ RWtableGetWriterList()
 char *
 RWGetMsg()
 {
+#ifndef __NetBSD__
 #if defined(BSD4_4)
     __const extern char *__const sys_errlist[];
 #else
@@ -263,6 +291,7 @@ RWGetMsg()
 #endif
 #endif
 #endif
+#endif
 #endif
 
     if (RWtableMsg[0] == '\0') {
